import QtQuick 2.4

/* Data Table Element
* Show a series of ListModels as columns of data in a Table
*/

Item {
    id: root

    // REQUIRED property: for translation to work (via qsTranslate), it must know where the external model is stored.
    // Without this you will likely see blank strings in your data table.
    property string sourceFile: ""

    // default column width, used if column model has no defined width
    property int colWidth: 360

    /* list of ListModels, one for each column of the table
    // properties may be attached to each model (as shown in demoModel below), to control the following:
    //      heading - text for header)
    //      hAlign - horizontal alignment for data inside cells
    //      width - width of column
    //      bgimage - url of BorderImage to use for background of column
    //             TODO: currently, several constants for alignment which depend on current Work_Summary_Columnxxx.png images
    //      cellTextColor: color of font for data in all cells of a column

    // The "label" role of each ListElement is used as the data in each cell in the column.
    // The "cellTextColor" role of each element may be used to override the model property of the same name for a particular cell
    // Each model in the list is another column in the table.
    */

    property list<ListModel> models

    // sample ListMode data
    ListModel
    {
        id: demoModel
        property string heading: qsTr("DEMO COLUMN") + application.emptyString
        property int width: 300
        property int hAlign: Text.AlignHCenter
        property url bgimage: "graphics/summary/Work_Summary_Column4.png"

        ListElement { label: QT_TR_NOOP("ELAPSED TIME") ; halign: Text.AlignLeft }
        ListElement { label: QT_TR_NOOP("AVG HEART RATE") }
        ListElement { label: QT_TR_NOOP("MAX HEART RATE") }
        ListElement { label: QT_TR_NOOP("AVERAGE PACE (RPM)") }
        ListElement { label: QT_TR_NOOP("MAX PACE (RPM)") }
        ListElement { label: QT_TR_NOOP("DISTANCE") }
        ListElement { label: QT_TR_NOOP("CALORIES") }
    }

    Component {
        id: entryDelegate
        Item {
            property variant modelObj: ListView.view.model
            property int parentIndex: ListView.view.parentIndex
            width: parent.width
            height: (parent.parent.height -80) / ListView.view.count

            Text {
                id: entryText
                text: {
                    if (!label) {
                        return ""
                    }

                    if (typeof label === 'string') {
                        var str = qsTranslate(root.sourceFile, label)
                        if (model.metricLabel && model.usLabel)
                            str += " (" + qsTranslate(root.sourceFile, workoutScreen.unitState === "METRIC" ? metricLabel : usLabel) + ")"
                        return str + application.emptyString
                    } else {
                        return label
                    }
                }
                // TODO: need a way to set the font settings from the model, or parameter
                font.pointSize: parent.modelObj.hasOwnProperty("pointSize") ? parent.modelObj.pointSize : 16
                // choose color of text in cell from priority: cellTextColor property from ListElement, cellTextColor propery from ListModel property, or default black
                color: (typeof cellTextColor !== 'undefined') ? cellTextColor : (parent.modelObj.hasOwnProperty("cellTextColor") ? parent.modelObj.cellTextColor : "black")
                width: parent.width
                horizontalAlignment: parent.modelObj.hAlign
            }
            BorderImage {
                id: rule

                visible: parent.modelObj.hasOwnProperty("showLines") ? parent.modelObj.showLines : true
                width: {
                    return sourceFile == "WorkoutSummary" && parent.parentIndex === 0 ? 291 : parent.width + 3
                }
                height: 6
                anchors { top: entryText.bottom ; left: parent.left ; leftMargin: -5}
                source: "graphics/DataTable/Workout_Summary_Rule.png"
            }
        }
    } // end Component entryDelegate

    Component {
        id: colview

        Item {
            id: top

            property variant model

            width: model ? model.hasOwnProperty("width") ? model.width : root.colWidth : root.colWidth
            height: root.height //500
            z: index === 0 ? 10 : 0

            // background image for column
            BorderImage {
                id: bg

                // use the bgimage property from the model, if it has one. otherwise, no background
                property url bgimage: top.model ? top.model.hasOwnProperty("bgimage") ? parent.model.bgimage : "" : ""

                anchors.fill: parent
                clip: true
                border.top: 80
                border.left: 80
                border.right: 90
                border.bottom: 80
                //visible: model? true : false
                source: bgimage

                // column header text
                Text {
                    text:  top.model ? qsTranslate(root.sourceFile, top.model.heading) : ""
                    color: top.model ? top.model.hasOwnProperty("headingTextColor") ? top.model.cellTextColor : "white" : "white"
                    font.bold: true
                    font.pointSize: top.model && top.model.hasOwnProperty("headingPointSize") ? top.model.headingPointSize : 20
                    width: parent.width-80
                    horizontalAlignment: Text.AlignHCenter
                    anchors{ top: parent.top; topMargin: 32; left: parent.left ; leftMargin: 30 }
                }

                ListView {
                    property int hAlign: top.model ? top.model.hAlign : Text.AlignHCenter
                    property int parentIndex: index
                    interactive: false
                    clip: true
                    width: parent.width -57
                    height: parent.height -80
                    delegate: entryDelegate
                    anchors{ top: parent.top; topMargin: 85; left: parent.left ; leftMargin: 23 }
                    model: top.model
                }
            } // end BorderImage
        } // end Item top
    } // end Component colview

    // row of table columns
    Row {
        spacing: -110
        Repeater {
            model: root.models.length
            delegate: colview

            onItemAdded: { // assign a model from the array to each column
                item.model = root.models[index]
            }
        } // end Repeater
    } // end Row
}

