import QtQuick 2.4

Item {
    id: root

    property real minValue: 0
    property real maxValue: 100
    property real value: 0
    property int precision: 0
    property real valueProxy // declarative binding for value
    property bool valueProxyChange: false
    property bool clickableTrack: true
    property real stepValue: 1
    property int _minpos: 0
    property int _maxpos: (orientation == Qt.Horizontal) ? track.width : track.height
    property bool realtime: true  // emit values while slider is being moved
    property url thumbImage: "graphics/SliderControl/Double_Arrow_Slider_Thumb_Horz.png"
    property url trackImage: ""
    property real trackImageOpacity: 1.0
    property url trackFillImage: ""
    property real trackFillImageOpacity: 1.0
    property bool showButtons: true
    property bool showValue: true
    property int valueOffsetX: 0
    property int valueOffsetY: 0
    property int buttonOffsetX: 0
    property int buttonOffsetY: 0
    property int buttonMargin: 20
    property int orientation: Qt.Horizontal
    property url decrementButtonImage: "graphics/SliderControl/Pace_Setter_Simple_Minus.png"
    property url incrementButtonImage: "graphics/SliderControl/Pace_Setter_Simple_Plus.png"
    property bool repeat: false
    property real thumbMouseAreaScale: 1.0
    property real buttonAreaScale: 2.0
    property int dragAreaMargin: 0
    property bool rightToLeft: Qt.application.layoutDirection === Qt.RightToLeft && orientation === Qt.Horizontal
    property bool showSelectionCursor: false

    signal released()

    width: trackImage != "" ? trackGraphic.implicitWidth : (orientation == Qt.Horizontal ? 100 : 5)
    height: trackImage != "" ? trackGraphic.implicitHeight : (orientation == Qt.Horizontal ? 5 : 100)
    enabled: visible && opacity > 0

    Component.onCompleted: {
        value = validateRange(value)
    }

    onValueProxyChanged: {
        valueProxyChange = true
        setValue(valueProxy)
        valueProxyChange = false
    }

    function setValue(newValue) {
        if(!precision) {
            var change = newValue - value
            change = (change/stepValue).toFixed(0)*stepValue
            newValue = value + change
        }
        else
            newValue = parseFloat(newValue).toFixed(precision)
        newValue = validateRange(newValue)
        value = newValue //newValue.toFixed(precision)
    }

    function validateRange(newValue) {
        if( newValue < root.minValue )
            newValue = root.minValue
        if( newValue > root.maxValue )
            newValue = root.maxValue
        return newValue
    }

    function positionFromValue(value) {
        var pos = (value-minValue) * (_maxpos / (maxValue - minValue))
        if (isNaN(pos)) {
            pos = 0
        }
        return rightToLeft ? _maxpos-pos : pos
    }

    function valueFromPosition(position) {
        if (rightToLeft)
            position = _maxpos - position
        var newValue = minValue + position *(maxValue-minValue)/(_maxpos-_minpos)
        newValue = validateRange(newValue)
        return newValue.toFixed(precision)
    }

    function incrementBy(n) {
        root.value = validateRange(value + n)
    }

    function decrementBy(n) {
        value = validateRange(value - n)
    }

    function incrementValue() {
        root.value = validateRange( value + stepValue )
    }

    function decrementValue() {
        value = validateRange( value - stepValue )
    }

    Item {
        id: track
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        width: root.width - (showButtons ? (10 + incrementButton.width + decrementButton.width + thumbImage.width/2) : 0)
        height: root.height - (showButtons ? (10 + incrementButton.height + decrementButton.height + thumbImage.height/2) : 0)

        // track line when there's no background image
        Rectangle {
            id: track_line
            height: 3
            width: parent.width
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            color: "black"
            visible: root.trackImage != "" ? false : true
            onVisibleChanged: {console.log("viz is", visible) }
        }

        // image to display over the "filled" area of slider
        BorderImage {
            anchors.left: parent.left
            anchors.leftMargin: 1
            anchors.right: sliderKnob.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            source: root.trackFillImage
            opacity: trackFillImageOpacity
        }
        BorderImage {
            id: trackGraphic
            source: root.trackImage
            opacity: trackImageOpacity
            anchors {
                top: parent.top
                bottom: parent.bottom
                horizontalCenter: root.orientation == Qt.Vertical ? parent.horizontalCenter : undefined
            }
        }

        Text {
            id: valueDisplay
            visible: root.showValue
            //anchors { left: root.left ; leftMargin: root.valueOffsetX } //; bottom: sliderKnob.top ; bottomMargin: -valueOffsetY }
            text: root.value
        }

        // mouse area for clicking the track to move the slider
        MouseArea {
            id: track_mouse

            enabled: clickableTrack
            anchors.fill: parent
            onPressed: {
                var position = (root.orientation == Qt.Horizontal ? (_minpos + mouseX ) : (_maxpos -_minpos - mouseY))
                value = valueFromPosition( position )
                buzzer.buttonPress()
            }
            Rectangle{
                opacity: 0.3
                color: "green"
                anchors.fill: parent
                visible: main.showStimulusArea
            }
        }

        Image {
            id: sliderKnob

            anchors.left: parent.left
            anchors.leftMargin: positionFromValue(value) - width/2
            anchors.bottomMargin: positionFromValue(value) - height/2
            source: root.thumbImage

            BorderImage {
                id: selectionCursor
                visible: root.showSelectionCursor
                border { left: 40; top: 5; bottom: 5; right: 40 }
                source: "graphics/SliderControl/maintenance_menus_cursor_new.png"
                width: 100
                anchors.centerIn: parent
                anchors.horizontalCenterOffset: -3
                horizontalTileMode: BorderImage.Stretch
            }

            MouseArea {
                id: knobMouseArea
                property int knobMargin: (root.orientation == Qt.Horizontal ? (sliderKnob.width/2) : (sliderKnob.height/2))

                anchors.fill: parent
                anchors.margins: root.dragAreaMargin
                scale: root.thumbMouseAreaScale

                drag {
                    target: sliderKnob ;
                    axis: (root.orientation === Qt.Horizontal) ? Drag.XAxis : Drag.YAxis;
                    minimumX: _minpos - knobMargin ; minimumY: _minpos - knobMargin
                    maximumX: _maxpos - knobMargin + 5  ; maximumY: _maxpos - knobMargin + 5    // +5 fudge factor need for some reason to allow full sliding to max value
                }

                onPressed: {
                    sliderKnob.anchors.left = undefined
                    sliderKnob.anchors.bottom = undefined
                    buzzer.buttonPress()
                } // un-anchor to allow drag
                onReleased: {
                    if(root.orientation == Qt.Horizontal)
                        sliderKnob.anchors.left = sliderKnob.parent.left
                    else
                        sliderKnob.anchors.bottom = sliderKnob.parent.bottom
                    root.released()
                } // re-anchor at end of drag

                onMouseXChanged: {
                    if( root.orientation == Qt.Horizontal )
                        setValue(valueFromPosition(sliderKnob.x + sliderKnob.width/2))
                }
                onMouseYChanged: {
                    if( root.orientation != Qt.Horizontal )
                        setValue(valueFromPosition(_maxpos - _minpos - sliderKnob.y - sliderKnob.height/2))
                }

                Rectangle{
                    opacity: 0.3
                    color: "blue"
                    anchors.fill: parent
                    visible: main.showStimulusArea
                }
            }
        }
    }

    ImageButton {
        id: decrementButton
        LayoutMirroring.enabled: root.rightToLeft
        visible: showButtons
        repeat: root.repeat
        anchors { left: parent.left ; leftMargin: -root.buttonMargin ; verticalCenter: parent.verticalCenter ; verticalCenterOffset: root.buttonOffsetY ; horizontalCenterOffset: root.buttonOffsetX ; topMargin: root.buttonMargin; bottomMargin: root.buttonMargin}
        downimage: root.decrementButtonImage
        upimage: root.decrementButtonImage
        onClicked: {
            root.decrementValue()
        }
        mouseAreaScale: root.buttonAreaScale
        z:-1 // ensure button is under slider thumb
    }

    ImageButton {
        id: incrementButton
        LayoutMirroring.enabled: root.rightToLeft
        visible: showButtons
        repeat: root.repeat
        anchors { right: parent.right ; rightMargin: -root.buttonMargin ; verticalCenter: parent.verticalCenter ; verticalCenterOffset: root.buttonOffsetY ; horizontalCenterOffset: root.buttonOffsetX ; topMargin: root.buttonMargin; bottomMargin: root.buttonMargin }
        downimage: root.incrementButtonImage
        upimage: root.incrementButtonImage
        onClicked: {
            root.incrementValue()
        }
        mouseAreaScale: root.buttonAreaScale
        z:-1 // ensure button is under slider thumb
    }

    states: [
        State {
            name: "Horizontal"
            when: orientation == Qt.Horizontal

            PropertyChanges {
                target: track
                width: root.width - (showButtons ? (10 + incrementButton.width + decrementButton.width) : 0)
                height: root.height
            }

            AnchorChanges {
                target: track
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
            }

            PropertyChanges {
                target: track_line
                height: 3
                width: parent.width
            }

            AnchorChanges {
                target: sliderKnob
                anchors { verticalCenter: parent.verticalCenter ; left: parent.left }
            }

            AnchorChanges {
                target: incrementButton
                anchors { right: parent.right ; verticalCenter: parent.verticalCenter }
            }

            AnchorChanges {
                target: decrementButton
                anchors { left: parent.left ; verticalCenter: parent.verticalCenter }
            }
        },
        State {
            name: "Vertical"
            when: orientation == Qt.Vertical

            PropertyChanges {
                target: track
                height: root.height - (showButtons ? (10 + incrementButton.height + decrementButton.height) : 0)
                width: root.width
            }

            AnchorChanges {
                target: track
                anchors {
                    left: undefined
                    right: undefined
                    top: root.top
                    bottom: root.bottom
                    horizontalCenter: root.horizontalCenter
                    verticalCenter: undefined
                }
            }

            PropertyChanges {
                target: track_line
                height: track.height;
                width: 3
                opacity: 0.5
            }

            AnchorChanges {
                target: sliderKnob
                anchors { verticalCenter: undefined ; left: undefined ; bottom: parent.bottom ; horizontalCenter: parent.horizontalCenter }
            }

            AnchorChanges {
                target: incrementButton
                anchors { bottom: track.top; left: undefined; right: undefined; horizontalCenter: parent.horizontalCenter ; verticalCenter: undefined}
            }

            AnchorChanges {
                target: decrementButton
                anchors { top: track.bottom ; left: undefined ; right: undefined ; horizontalCenter: parent.horizontalCenter ; verticalCenter: undefined}
            }

        }
    ]

}
