// Diagnostics Software Update Page

import QtQuick 2.4
import StorageMgr 1.0
import BootLoader 1.0
import Process 0.1
import ".."
import Components 1.0

Rectangle {
    id: root
    width: 1366
    height: 768
    anchors.fill: parent
    color: "transparent"

    property int fontSize: 20
    property int topOffset: 130
    property int itemSpacing: 5
    property string firmwareUpdateFile: nucleusfirmwareloader.updateFile
    property bool firmwareUpdating: nucleusfirmwareloader.isUpdating
    property bool firmwareUpdateAvailable: nucleusfirmwareloader.isUpdateAvailable

    property string cursorState: "firmware" // check, firmware, or software

    Connections {
        target: main.isTft && main.state === "DiagnosticState" && root.visible && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0 && cursorState !== "" && !firmwareUpdating  ? keymapper : null

        onEnter2Pressed: {
            main.debugKeys("enter2Pressed", "DiagnosticsUpdate")
            if (firmwareUpdating) // safeguard
                return

            if (cursorState === "firmware" && bootloadButton.enabled) {
                bootloadButton.clicked()
            }
        }
    }

    Connections {
        target: main.isTft && main.state === "DiagnosticState" && root.visible && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0 && !softwareUpdating && !firmwareUpdating ? keymapper : null

        onBackPressed: {
            main.debugKeys("backPressed", "DiagnosticsUpdate")
            diagnostics_screen.item.unselectScreen()
        }
    }

    onFirmwareUpdatingChanged: {
        if(firmwareUpdating)
            firmwareUpdatingDialog.show()
        else
            firmwareUpdatingDialog.hide()
    }

    Connections {
        target: nucleusfirmwareloader
        onFault: {
            print("Diagnostics Update: " + message)
            firmwareUpdatingDialog.hide()
            bootloadErrorReceivedDialogText.text = message
            bootloadErrorReceivedDialog.show()
        }
        onStatus: {
            firmwareUpdatingDialog.text = message
        }
        onUpdateSuccessful: {
            main.restartRequired = true
            restartMsg.visible = true
        }
    }

    signal close()

    ModalDialog{
        id:bootloadErrorReceivedDialog
        blockKeys: true
        dismissable: true

        BorderImage {   // pop-up dialog background
            id: errorborder

            x: 332
            y: 161
            width: 800
            height: 350
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: 152; bottom: 165; left: 145; right: 144 }

            Text {
                id: bootloadErrorReceivedDialogText
                anchors.fill: parent
                anchors.verticalCenter: parent.verticalCenter
                color: "#ffffff"
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.pixelSize: 50

            }
            MouseArea{
                anchors.fill: parent
                onClicked: bootloadErrorReceivedDialog.hide()
            }
        }
        Component.onCompleted: hide()
    }

    Item{
        id: informationSection
        width: childrenRect.width
        height: childrenRect.height
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: root.top
        anchors.topMargin: 50
        Row{
            spacing: 200
            Column{
                id: labels
                spacing: itemSpacing

                Text{
                    text: qsTr("Nucleus Firmware Version") + application.emptyString
                    font.pointSize: fontSize
                }

            }
            Column{
                id:values
                spacing: itemSpacing

                Text{
                    text: machineinfo.nucleusFirmwareVersion === "" ? qsTr("NOT AVAILABLE") + application.emptyString : machineinfo.nucleusFirmwareVersion
                    font.pointSize: fontSize
                }
            }
        }
    }

    Item{
        id: firmwareUpdateSection

        width: parent.width
        height: childrenRect.height


        anchors {
            top: informationSection.bottom ; topMargin: 50
            left: parent.left ; leftMargin: 80
        }

        Text{
            id: hexFileStatusText
            font.pointSize: fontSize
            anchors { left: bootloadButton.right ; verticalCenter: bootloadButton.verticalCenter }
            visible: !restartMsg.visible
            text:
            {
                if( bootloadButton.visible ) {
                    return qsTr("Firmware to load: ") + application.emptyString + firmwareUpdateFile
                }
                else
                    return ""
            }
        }

        Image {
            source: "../graphics/tft/NAV_selector_2.png"
            visible: main.isTft && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0 && cursorState !== "" && !restartMsg.visible
            anchors.centerIn: {
                switch (cursorState) {
                case "firmware":
                    return bootloadButton
                }
            }
            anchors.horizontalCenterOffset: -26
            z: bootloadButton.z - 1
        }

        ImageButton {
            id: bootloadButton
            label_color: "#000000"
            label_under: true
            label_y_offset: 10
            upimage: "../graphics/205/IPOD_Wheel_Center_Up.png"
            downimage: "../graphics/205/IPOD_Wheel_Center_Down.png"
            anchors { left: parent.left ; top: restartMsg.bottom }

            label: qsTr("Load Firmware") + application.emptyString
            visible: firmwareUpdateAvailable && !restartMsg.visible
            onVisibleChanged: {
                if (visible)
                    root.cursorState = "firmware"
            }
            onClicked: {
                print("Diagnostics Update: Loading firmware:" + firmwareUpdateFile)
                nucleusfirmwareloader.startFirmwareLoad(firmwareUpdateFile)
            }
        }

        Text {
            id: restartMsg
            visible: false
            text: qsTr("Firmware updated. Restart console.") + application.emptyString
            font.pointSize: fontSize
            anchors { horizontalCenter: parent.horizontalCenter ; top: parent.top }
        }
    }

    Text {
        id: instructions

        width: parent.width - 100

        anchors { bottom: parent.bottom ; bottomMargin: 90 ;  horizontalCenter: parent.horizontalCenter }
        horizontalAlignment: Text.AlignLeft
        wrapMode: Text.WordWrap
        visible: !restartMsg.visible
        text: {
            if(root.firmwareUpdating)
                return qsTr("Firmware is now being updated.  DO NOT power off the machine while software update is taking place.")
            else
                return qsTr("Nucleus firmware appears to be current.")
        }
    }

    Text {
        id: bottomHelpText
        visible: main.isTft && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0 && !checkingForUpdates && !firmwareUpdating && !softwareUpdating

        height: 48
        font { family: "arial"; pixelSize: 36; bold: true }
        color: "#6c6c6c"
        text: qsTr("use <img src='../graphics/tft/arrow_up.png' /><img src='../graphics/tft/arrow_down.png' /> to navigate, press ENTER to select") + application.emptyString

        anchors.bottom: parent.bottom
        anchors.bottomMargin: 25
        anchors.horizontalCenter: parent.horizontalCenter
    }

    ModalDialog{
        id: waitingForBootloaderDialog
        blockKeys: true

        //visible: bootloader.connected && bootloader.progress > 0
        BorderImage {   // pop-up dialog background
            id: waitingForBootloaderDialogBackground

            x: 332
            y: 161
            width: 800
            height: 350
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: 152; bottom: 165; left: 145; right: 144 }

            Text{
                id: waitingForBootloaderDialogText
                text: qsTr("Waiting for bootloader...") + application.emptyString
                color: "white"
                font.bold: true
                font.pixelSize: 50
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
            }
    }
    Component.onCompleted: hide()
    }

    Timer {
        id: hexPollTimer
        interval: 5000
        repeat: true
        running: root.visible && (root.opacity >0) && !firmwareUpdating
        onTriggered: {
            nucleusfirmwareloader.findHexFile()
        }
    }

    ModalDialog{
        id: firmwareUpdatingDialog
        blockKeys: true

        //visible: bootloader.connected && bootloader.progress > 0
        property alias text: bootLoadStatusText.text

        BorderImage {   // pop-up dialog background
            id: firmwareUpdatingDialogBackground

            x: 332
            y: 161
            width: 800
            height: 350
            source: "../graphics/Pop_Up_Lrg_Grey.png"
            border { top: 152; bottom: 165; left: 145; right: 144 }

            Text{
                id: bootLoadStatusText
                text:""
                color: "white"
                font.bold: true
                font.pixelSize: 50
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.bottom: bootloadProgress.top
                anchors.bottomMargin: 5
            }

            ProgressBar {
                id: bootloadProgress
                width: 400
                height: 50
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                value: nucleusfirmwareloader.progress
            }
        }
        Component.onCompleted: hide()

    }

    Component.onCompleted: {
        console.log("starting software update check...")
        nucleusfirmwareloader.findHexFile()
    }
}

