import QtQuick 2.4
import ".."
import ConfigurationData 1.0
import StorageMgr 1.0
import FileIO 1.0
import Components 1.0
import "../string_format.js" as Format

Rectangle {
    id: root
    width: 1366
    height: 768
    anchors.fill: parent
    state: ""
    color: "transparent"

    property int topOffset: 130
    property int itemSpacing: -10

    property int menuItemWidth: 150
    property int menuItemHeight: 75

    property bool languageDiagnosticMenu: false

    property bool datadirty: false

    signal close()

    function showLanguageMenu() {
        languageDiagnosticMenu = true
        language_menu.opacity = 1
    }

    function deleteSavedWorkouts() {
        var storedFiles = storage.listFiles("/data/saved_programs", "*.xml")
        for (var i = 0; i < storedFiles.length; i++) {
            var filepath = "/data/saved_programs/"+storedFiles[i]
            console.log("Deleting file:", filepath)
            fileio.remove(filepath)
        }
    }

    Component.onCompleted: {
        metricUnitsMode.value = options.metricUnits ? true : false
        saveWorkoutEnabledMode.value = options.saveWorkoutEnabled ? true : false
        contactHRPriorityMode.value = options.contactHRPriority ? true : false
        systemVolumeControl.value = options.systemVolume
        maxSpeedRegulatorToggle.value = options.maxSpeedRegulatorEnabled
    }

    Component.onDestruction: {
        console.debug("Closing...")
        //if(datadirty){
        //    console.debug("Data is dirty! Saving changes!")
            brainboard.saveToEeprom()
        //}
    }

    StorageMgr {
        id: storage
    }

    FileIO {
        id: fileio
        onError: {
            console.log("**WARNING: FileIO error -", error)
        }
    }

    Connections {
        target: main.isTft && main.state === "DiagnosticState" && root.visible && !languageDiagnosticMenu && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0 ? keymapper : null

        // handle exiting the screen
        onBackPressed: {
            main.debugKeys("backPressed", "DiagnosticsOption")
            diagnostics_screen.item.unselectScreen()
        }
    }

    KeyedControlList {
        id: controlList
        listening: main.isTft && main.state === "DiagnosticState" && root.visible && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0 && !languageDiagnosticMenu
        listView: mode

        ToggleButtonLabeled{
            id: metricUnitsMode

            labelString: qsTr("Metric Units") + application.emptyString
            onValueChanged:
            {
                options.metricUnits = value ? true : false
                brainboard.setUnitSystem(value)
                datadirty = true
            }
        }
        ToggleButtonLabeled{
            id: saveWorkoutEnabledMode

            labelString: qsTr("Save Workout Enabled") + application.emptyString
            onValueChanged:
            {
                options.saveWorkoutEnabled = value ? true : false
                if (!options.saveWorkoutEnabled) {
                    deleteSavedWorkouts()
                }
                datadirty = true
            }
        }
        ToggleButtonLabeled{
            id: contactHRPriorityMode
            labelString: qsTr("Contact Heart Rate Has Priority") + application.emptyString
            onValueChanged:
            {
                options.contactHRPriority = value ? true : false
                datadirty = true
            }
        }

        ToggleButtonLabeled{
            id: maxSpeedRegulatorToggle
            labelString: qsTr("Max Speed Regulator Enable") + application.emptyString
            visible: machineinfo.productTypeString == "Treadmill"
            onValueChanged:
            {
                options.maxSpeedRegulatorEnabled = value ? true : false

                if(options.maxSpeedRegulatorEnabled === false) {
                    options.maxSpeedRegulatorValue = machineinfo.treadmillSpeedMaxMph
                }

                datadirty = true
            }
        }

        SliderControlLabeled{
            id: maxSpeedRegulator
            headingString: qsTr("Max Speed") + application.emptyString
            value: options.maxSpeedRegulatorValue
            minValue: 60
            maxValue: machineinfo.treadmillSpeedMaxMph
            displayValue: value / 10.0 + "MPH"

            visible: options.maxSpeedRegulatorEnabled

            onValueChanged: {
                options.maxSpeedRegulatorValue = value
                datadirty = true
            }

            Component.onCompleted: value = options.maxSpeedRegulatorValue
        }

        SliderControlLabeled{
            id: lcdBrightnessControl
            headingString: qsTr("LCD Brightness") + application.emptyString
            value: options.lcdBrightness
            minValue: 10
            maxValue: 100
            displayValue: value + "%"

            onValueChanged: {
                datadirty = true
                options.lcdBrightness = value
            }
            Component.onCompleted: value = options.lcdBrightness
        }

        SliderControlLabeled{
            id: systemVolumeControl
            headingString: qsTr("System Sound Volume") + application.emptyString
            minValue: 0
            maxValue: 100
            stepValue: 5
            displayValue: value + "%"
            onValueChanged:
            {
                options.systemVolume = value
                datadirty = true
            }
        }

        Item{
            id: defaultLanguageItem
            height: menuItemHeight
            width: root.width
            x: 30

            // attributes needed to be used in a KeyedControlList
            property bool selected: false
            property int navCursorWidth: defaultLanguageLabel.width
            function leftPressed() {}
            function rightPressed() {}
            function enterPressed() { root.showLanguageMenu() }

            Text{
                id: defaultLanguageLabel
                text: qsTr("Default Language") + application.emptyString
                anchors.left: parent.left
                anchors.verticalCenter: parent.verticalCenter
                anchors.verticalCenterOffset: -8
                color: "black"
                width: 250
            }

            Text{
                id: defaultLanguageText
                text: language_menu.getLanguageName(options.defaultLanguage)

                x: 400
                anchors.verticalCenter: parent.verticalCenter
            }

            Image{
                id: defaultLanguageIcon

                source: "../LanguageMenu/" + language_menu.getLanguageIcon(options.defaultLanguage)
                anchors.top: parent.top
                anchors.bottom: parent.bottom
                anchors.left: defaultLanguageText.right
            }

            MouseArea{
                anchors.fill: parent
                onClicked: { root.showLanguageMenu() }
            }
        }

    }


    Column {
        id: swipeArrows

        visible: mode.interactive
        anchors { left: mode.right ; leftMargin: 10 ; verticalCenter: mode.verticalCenter }
        spacing: 275
        Image {
            id: up
            source: "../graphics/200/Swipe_Arrow_Up_Large.png"
        }

        Image {
            id: down
            source: "../graphics/200/Swipe_Arrow_Down_Large.png"
        }

    }

    ListView{
        id: mode
        anchors.horizontalCenter: root.horizontalCenter
        anchors.verticalCenter: root.verticalCenter
        width: root.width * 0.8
        height: root.height * 0.8
        model: controlList.model
        snapMode: ListView.SnapToItem
        interactive: false
        highlightRangeMode: ListView.ApplyRange
        preferredHighlightBegin: height/2 - 30
        preferredHighlightEnd: height/2 + 30
        highlightMoveVelocity: 10000
        currentIndex: controlList.curIndex
//        highlightFollowsCurrentItem: false
        highlight: Component {
            Item {
                BorderImage {
                    id: selectionCursor
                    visible: main.isTft && controlList.listening
                    border { left: 70; top: 5; bottom: 5; right: 70 }
                    source: "../graphics/tft/maintenance_menus_cursor.png"
                    width: mode.currentItem ? mode.currentItem.navCursorWidth + 70 : 0
                    height: 56
                    horizontalTileMode: BorderImage.Stretch
                }
            }
        }
    }

    Text {
        id: bottomHelpText
        visible: main.isTft && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0

        height: 48
        font { family: "arial"; pixelSize: 36; bold: true }
        color: "#6c6c6c"
        text: {
            if (controlList.curIndex === 5)
                return qsTr("press ENTER to select") + application.emptyString
            else
                return qsTr("use <img src='../graphics/tft/arrow_up.png' /><img src='../graphics/tft/arrow_down.png' /> to navigate, use <img src='../graphics/tft/arrow_left.png' /> <img src='../graphics/tft/arrow_right.png' /> to change value") + application.emptyString
        }

        anchors.bottom: parent.bottom
        anchors.bottomMargin: 25
        anchors.horizontalCenter: parent.horizontalCenter
    }

    Connections {
        target: language_menu
        onLanguageSelected: {
            if (languageDiagnosticMenu) {
               main.state = "DiagnosticState"
               if (lang)
               {
                   application.selectLanguage(lang)
                   options.defaultLanguage = lang
                   defaultLanguageText.text = language_menu.getLanguageName(options.defaultLanguage)
                   defaultLanguageIcon.source = "../LanguageMenu/" + language_menu.getLanguageIcon(options.defaultLanguage)
               }
               languageDiagnosticMenu = false
            }
        }
    }
}
