// import QtQuick 2.4 // to target S60 5th Edition or Maemo 5
import QtQuick 2.4
import Process 0.1

Process {
    id: root

    property int winid
    property string mrl
    property real aspect:0
    property bool playing: false
    property bool tuning: false
    property bool debug: false
    property int analogInput: 0
    property bool muted: false
    property bool deinterlace: true
    property int oldWinid: 0
    property string videoDriver: "xv"
    property bool isRunning: (processState == Process.Running)
    property real frameRate: 0.0

    signal closed()
    signal error(string message)

    program: "mplayer"

    function debugLog(string) {
        if(root.debug)
            print("mplayer " + winid + ":" + string)
    }

    function buildArgs() {
        root.arguments = [ "-wid", winid,
            "-idle","-slave",
            "-framedrop",
            "-quiet", "-msgmodule", "-msglevel", "all=1:identify=5:global=4:cplayer=4",
            "-tv", "input=" + analogInput,
            "-vo", root.videoDriver,
            mrl]


        debugLog("mplayer args: " + root.arguments)
    }

    function sendCommand(cmd) {
        debugLog("sending command: " + cmd)
        write(cmd + "\n")
    }

    onMutedChanged: {
        if(muted)
            sendCommand("mute 1")
        else
            sendCommand("mute 0")
    }

    onReadyReadStandardOutput: {
        var output = String(readAllStandardOutput())
        debugLog( "output: " + output)
        parseMessage(output)
    }

    onReadyReadStandardError: {
        var output = String(readAllStandardError())
        debugLog( "mplayer error output: " + output)
        parseMessage(output)
    }

    function quit() {
        sendCommand("quit")
        print("setting tuning to false")
        root.playing = false
        root.tuning = false
    }

    function parseMessage(output) {
        var lines = output.split("\n")
        for (var i in lines){
            var line = String(lines[i]).trim()
            var msgparts = line.split(":")
            var module = String(msgparts[0]).trim()

            switch(module) {
                case "IDENTIFY":
                    identifyMsg(msgparts[1].trim()+ msgparts[2])
                    break;
                case "DVBIN":
                    dvbMsg(msgparts[1].trim());
                    break;
                case "CPLAYER":
                    playerMsg(msgparts[1].trim())
                    break;
                default:
                    //print("unknown message: " + line)
                    break;
            }
        }
    }

    function playerMsg(msg) {
        if (msg == "Starting playback...") {
            playing = true
            tuning = false
            debugLog("media started.")
        }
    }

    function identifyMsg(msg) {
        var values = msg.split("=")
        if(values.length > 1) {
            if(values[0] === "ID_VIDEO_ASPECT") {
                root.aspect = parseFloat(values[1])
            }
            if(values[0] === "ID_EXIT") {
                debugLog("ended playback.")
                root.playing = false
                root.tuning = false
            }
            if(values[0] == "ID_VIDEO_FPS") {
                if( root.deinterlace ){
                    root.frameRate = parseFloat(values[1])
               }
            }
        }
    }

    function dvbMsg(msg) {
        error(msg)
    }

    onPlayingChanged: {
        if(!root.playing)
            return
        // deinterlace lower framerate sources (e.g. 29.97) as they are generally interlaced
//        if( (root.frameRate < 31) )
//            sendCommand("set_property deinterlace 1")
//        else
//            sendCommand("set_property deinterlace 0")
    }

    onMrlChanged: {
        debugLog("mrl changed: " + mrl)
        if (mrl === "")  // stop playback if null mrl
        {
            sendCommand("stop")
            playing = false
            tuning = false
        }
        else if( root.isRunning ) {
            sendCommand("loadfile " + "\"" + mrl + "\"")
            playing = false // playing will be set to true after tuning complete
            tuning = true
        }
        else {
            debugLog("not running, restarting..")
            playing=false
            tuning = true
            buildArgs()
            restart()
        }
    }

    onAnalogInputChanged: {
        debugLog("analog input changed: restarting")
        root.buildArgs()
        restart()
    }

    onWinidChanged: { // restart process with new window id
        if(winid == 0)
            return;
        if(winid == oldWinid)
            return;
        debugLog("new winid:" + winid)
        root.buildArgs()
        oldWinid = winid
        if(root.isRunning) {
            restart()
        }
        else
            start()
    }
}
