import QtQuick 2.4
import Netpulse 0.1
import WorkoutRunData 1.0
import Process 0.1
import QtQuick.XmlListModel 2.0

Item {
    id: root

    property bool connected: false
    property bool loggedIn: userLogin ? userLogin.loggedIn : false
    property bool debug: false
    property UserData userLogin
    property variant pages: ({})
    property string currentPage: ""
    property bool frozen: false
    property bool playing: (currentPage === "watch") && visible
    property bool _prefreezeVisibility: false
    property string machineConfigFile: "/netpulse/data/config.xml"
    property string machineID
    property string wirelessPSK
    property string wirelessESSID

    signal touchTime(int time)
    signal visibleChanging(bool willBeVisible)

    visible: netpulseCom.visible

    Component.onCompleted: wifiPSKRead.start()

    // temporarily hide Netpulse when it is "frozen" to display an overlay of some kind.
    onFrozenChanged: {
        if(frozen) {
            // retain the current visiblity state
            root._prefreezeVisibility = netpulseCom.visible
            if(netpulseCom.visible)
                root.visibleChanging(false)
            netpulseCom.visible = false
        }
        else {
            // restore the visibility state from before the freeze
            if(netpulseCom.visible != root._prefreezeVisibility)
                root.visibleChanging(root._prefreezeVisibility)
            netpulseCom.visible = root._prefreezeVisibility
        }
    }

    function resumeWorkout() {
        if (currentPage != null)
            showPage(currentPage)
    }

    function debugPrint(message) {
        if(debug)
            errorManager.debugMessage("Netpulse.qml: " + message)
    }

    function sendMessage(msg) {
        if(connected)
            netpulseCom.sendMessage(msg)
    }

    function touchQuery() {
        if(connected)
            netpulseCom.sendMessage({"op" : "touch"})
    }

    function logOut() {
        userLogin.userID = ""
        userLogin.userName = ""
        userLogin.userAge = 0
        userLogin.userGender = 0
        userLogin.loggedIn = false
        userLogin.uuid = ""
    }

    function startWorkout() {
        sendMessage({"op":"sow"});
    }

    function endWorkout() {
        sendMessage({"op":"eow"});
    }

    // link a netpulse page with a proxy QML element, to use for the position and size when shown.
    function setPageProxy(pagename, element) {
        debugPrint("setting proxy for netpulse page: " + pagename)
        var p = root.pages
        p[pagename] = element
        root.pages = p
    }

    // remove the link between a nepulse page an the currently associated QML element
    function removePageProxy(pagename) {
        debugPrint("removing proxy for netpulse page: " + pagename)
        var p = root.pages
        delete p[pagename]
        root.pages = p
    }

    function hidePage(pagename) {
        debugPrint("hide page " + pagename)
        if(netpulse.pages[pagename]) {
            netpulse.pages[pagename].visible = false
        }
        if(pagename == root.currentPage) {
            debugPrint("hiding current page, hiding app")
            audioManager.disableNetpulse()
            netpulseCom.setShown(false)
        }
    }

    // set the visibility of all proxy elements to hidden
    function hideAll() {
        for( var p in netpulse.pages ) {
            debugPrint("hideAll :" + p)
            netpulse.pages[p].visible = false
        }
        netpulseCom.setShown(false)
        netpulse.currentPage = ""
    }

    // hide current page
    function hide() {
        if(netpulse.currentPage in netpulse.pages)
            hidePage(netpulse.currentPage)
        netpulseCom.setShown(false)
    }

    // pass a NetpulseWindow derived element to show
    function showPage(pagename, args) {
        // only one page is visible at a time, so set all others to hidden
        debugPrint("show page:" + pagename)
        for( var p in netpulse.pages ) {
            if(p == pagename)
                continue
            debugPrint("hiding page:" + p)
            if(netpulse.pages[p])   // page proxy may have been destroyed
                netpulse.pages[p].visible = false
        }

        var newpage = netpulse.pages[pagename]
        if(!newpage) {
            debugPrint("no netpulse proxy element for page:" + pagename)
            return
        }

        debugPrint("show page:::" + pagename)
        netpulseCom.page = newpage.page
        root.currentPage  = pagename
        netpulseCom.x = (newpage.absx > -1 ? newpage.absx : newpage.x)
        netpulseCom.y = (newpage.absy > -1 ? newpage.absy : newpage.y)
        netpulseCom.width = (newpage.npw > -1 ? newpage.npw : newpage.width)
        netpulseCom.height = (newpage.nph > -1 ? newpage.nph : newpage.height)

        // if additional arguments are provided, send an explicit show command
        if( typeof args !== 'undefined') {
            args.page = newpage.page
            args.op = "show";
            netpulseCom.sendMessage(args)
        }

        netpulseCom.setShown(true)
        newpage.visible = true

        if (pagename === "landingPage")
            audioManager.disableNetpulse()
    }

    function setLanguage(lang) {
        netpulse.sendMessage({op:"locale",id:lang})
    }

    NetpulseManager { id: netpulseCom

        // maintain the top of the visiblity stack when netpulse app shown and hidden
        function setShown(show) {
            root._prefreezeVisibility = show
            if(!root.frozen) {
                if(netpulseCom.visible != show)
                    root.visibleChanging(show)
                netpulseCom.visible = show
            }
        }

        onPageChanged: {
            debugPrint("mgr changing to page:" + page)
        }
        debug: root.debug

        Component.onCompleted: {
            netpulseCom.setVisible(false)
            sendMessage({"op":"status"})
        }
        onButtonPressed: debugPrint("Button press: " + id )

        onMessageReceived: {
            debugPrint("message received: " + message.op)

            if(message.op == "touch") {
                root.touchTime(message.resp)
            }

            // pass the message to the proxy element
            if(pages[currentPage]) {
                pages[currentPage].messageReceived(message)
            }
            if(message.op == "viewchange") {
                if (root.currentPage != "loginreg") {
                    if( currentPage in netpulse.pages )
                        netpulse.pages[currentPage].visible = false
                    root.currentPage = message.page
                    if( currentPage in netpulse.pages )
                        netpulse.pages[currentPage].visible = true
                    if(root.currentPage == "watch")
                        audioManager.netpulseRequest()
                    else if (root.currentPage === "landingpage" || root.currentPage === "virtualactive")
                        audioManager.disableNetpulse()
                }
            }
            else if(message.op == "status") {
                debugPrint( "server: " + message.server )
                debugPrint( "gateway: " + message.gateway )
                if(message.gateway=="up")
                    root.connected = true
            }
            else if(message.op == "user") {
                debugPrint("netpulse user info.")
                userLogin.userID = message.id
                userLogin.userName = message.name
                userLogin.userAge = message.age
                userLogin.userGender = (message.gender == "m" ? 0 : 1)
                userLogin.loggedIn = true
                userLogin.uuid = message.uuid
            }

        }
    }

    Timer { id: statusCheckTimer
        interval: 2000
        repeat: true
        running: !connected
        onTriggered: {
            netpulseCom.sendMessage({"op":"status"})
        }
    }

    XmlListModel {
        id: machineConfig

        source: "file://" + root.machineConfigFile
        query: "/config/machine"

        XmlRole { name: "mac" ; query: "@mac/string()" }
        XmlRole { name: "mid" ; query: "@mid/string()" }

        onStatusChanged: {
            if(status == XmlListModel.Ready) {
                if(machineConfig.count > 0){
                    root.machineID = machineConfig.get(0).mid
                    print("Netpulse machine ID:" + root.machineID)
                }
                else {
                    print("could not read machine ID from config file "+ root.machineConfigFile)
                }
            }
        }
    }

    // Read netpulse wireless access point info from /etc/wpa_supplicant_netpulse.conf
    Process {
        id: wifiPSKRead

        program: "cat"
        arguments: ["/etc/wpa_supplicant_netpulse.conf"]
        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            var pskSearch = output.match(/[^#]psk="(.*)"/)
            var essidSearch = output.match(/ssid="(.*)"/)
            if(essidSearch)
                root.wirelessESSID = essidSearch[1]
            if(pskSearch)
                root.wirelessPSK = pskSearch[1]
        }
    }
}

