import QtQuick 2.4
import WorkoutManager 1.0
import Components 1.0

import "string_format.js" as Format
import "program_setup.js" as Fields
import "setup_params.js" as SetupJS

Item {
    id: root
    property alias model: myListView.model
    property QtObject timeItem
    property QtObject distItem
    property QtObject calItem
    property bool timeGoal: true
    property bool distGoal: false
    property int maxCalPerHr: 1600

    onModelChanged: SetupJS.modelPostProcessing()

    function setParameter(type, value) {
        return SetupJS.setParameter(type, value)
    }

    function validateEntries() {
        keypad.submit() // submit current item
        return SetupJS.validateEntries()
    }

    function recalcMaxSegments() {
        SetupJS.calculateMaxSegmentsFromTime()
    }

    // Validate data relationships when any value is changed
    Connections {
        target: root.model ? root.model : null
        ignoreUnknownSignals: true
        onDataChanged: {
            if(isTreadmill && calItem) // Treadmill calorie goal setup
                SetupJS.validateTreadCalorieGoal()
            if (!isTreadmill && calItem) {
                SetupJS.validateNonTreadCalorieGoal()
            }
        }
    }

    Connections {
        target: timeItem
        onDataChanged: {
            if(timeItem.dataValue > 0) {
                root.timeGoal = true
                root.distGoal = false
            }
        }
    }
    Connections {
        target: distItem
        onDataChanged: {
            if(distItem.dataValue > 0) {
                root.timeGoal = false
                root.distGoal = true
            }
        }
    }

    Item {
        id: calWarning

        visible: false
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 50
        anchors.left: parent.left
        anchors.leftMargin: 10
        width: 450
        height: 50

        Image{
            id: calCautionSymbol
            source: "graphics/Icon_Warning.png"
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
        }

        Text {
            text: qsTr( "The calorie goal chosen is too high for selected workout time.") + application.emptyString
            wrapMode: Text.WordWrap
            font.pixelSize: 16
            anchors.left: calCautionSymbol.right
            anchors.verticalCenter: parent.verticalCenter
        }
    }

    ListView {
        id: myListView
        anchors.fill: parent
        interactive: false
        delegate: myDelegate

        onCurrentIndexChanged: {
            keypad.clear()
            var item = model.get(currentIndex)
            if(!item)
                return
            if(item.units === "%") {
                keypad.length = 3
                keypad.show_entry = true
                keypad.decimalPlaces = 1
            }
            else {
                keypad.length = SetupJS.getMaxDigit(item.maxValue) + item.precision
                keypad.decimalPlaces = item.precision
                keypad.show_entry = false
            }

            keypad.units = item.units
        }

        function getBaseFieldHeight(fieldType)
        {
            switch(fieldType)
            {
            case WorkoutProgram.MAX_SPEED_TYPE:
            case WorkoutProgram.MAX_INCLINE_TYPE:
            case WorkoutProgram.LEVEL_TYPE:
            case WorkoutProgram.SPEED_TYPE:
            case WorkoutProgram.RUN_SPEED_TYPE:
            case WorkoutProgram.WORK_SPEED_TYPE:
            case WorkoutProgram.WALK_SPEED_TYPE:
            case WorkoutProgram.REST_SPEED_TYPE:
            case WorkoutProgram.DISTANCE_TYPE:
            case WorkoutProgram.DISTANCE_QUICK_TYPE:
            case WorkoutProgram.SEGMENT_TYPE:
            case WorkoutProgram.TIME_TYPE:
            case WorkoutProgram.PACE_TYPE:
            case WorkoutProgram.WORKLOAD_TYPE:
            case WorkoutProgram.AGE_TYPE:
            case WorkoutProgram.WEIGHT_TYPE:
            case WorkoutProgram.GENDER_TYPE:
            case WorkoutProgram.INCLINE_TYPE:
            case WorkoutProgram.CALORIES_TYPE:
            case WorkoutProgram.MIN_STRIDE_TYPE:
            case WorkoutProgram.MAX_STRIDE_TYPE:
                return 80;

            case WorkoutProgram.WORK_HEARTRATE_TYPE:
            case WorkoutProgram.REST_HEARTRATE_TYPE:
            case WorkoutProgram.TARGET_HEARTRATE_TYPE:
            case WorkoutProgram.HEARTRATEREADONLY_TYPE:
            case WorkoutProgram.FIXED_DISTANCE_TYPE:
            case WorkoutProgram.TIMEREADONLY_TYPE:
            case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                return 120;
            }
        }

        function getBackgroundSource(fieldType)
        {
            switch(fieldType)
            {
            case WorkoutProgram.MAX_SPEED_TYPE:
            case WorkoutProgram.MAX_INCLINE_TYPE:
            case WorkoutProgram.LEVEL_TYPE:
            case WorkoutProgram.SPEED_TYPE:
            case WorkoutProgram.RUN_SPEED_TYPE:
            case WorkoutProgram.WORK_SPEED_TYPE:
            case WorkoutProgram.WALK_SPEED_TYPE:
            case WorkoutProgram.REST_SPEED_TYPE:
            case WorkoutProgram.DISTANCE_TYPE:
            case WorkoutProgram.DISTANCE_QUICK_TYPE:
            case WorkoutProgram.SEGMENT_TYPE:
            case WorkoutProgram.TIME_TYPE:
            case WorkoutProgram.PACE_TYPE:
            case WorkoutProgram.WORKLOAD_TYPE:
            case WorkoutProgram.AGE_TYPE:
            case WorkoutProgram.WEIGHT_TYPE:
            case WorkoutProgram.GENDER_TYPE:
            case WorkoutProgram.INCLINE_TYPE:
            case WorkoutProgram.CALORIES_TYPE:
            case WorkoutProgram.MIN_STRIDE_TYPE:
            case WorkoutProgram.MAX_STRIDE_TYPE:
                return "graphics/setup/Setup_Small_Down.png"

            case WorkoutProgram.WORK_HEARTRATE_TYPE:
            case WorkoutProgram.REST_HEARTRATE_TYPE:
            case WorkoutProgram.TARGET_HEARTRATE_TYPE:
            case WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE:
            case WorkoutProgram.FIXED_DISTANCE_TYPE:
            case WorkoutProgram.TIMEREADONLY_TYPE:
            case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                return "graphics/setup/Setup_Large_Down.png"
            }
        }

        Keypad {
            id: keypad

            property variant model: myListView.model
            property int currentFieldType: getFieldType()

            function getFieldType() {
                if(!model)
                    return -1
                var field = model.get(myListView.currentIndex)
                if(!field)
                    return -1
                return field.fieldType
            }

            width: 320
            anchors.right: parent.right
            anchors.rightMargin: 500
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: show_entry ? 0 : 50
            show_entry: false
            allow_close: false

            visible: !(currentFieldType == WorkoutProgram.GENDER_TYPE || currentFieldType == WorkoutProgram.FIXED_DISTANCE_TYPE)
            enabled: !(currentFieldType == WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE || currentFieldType == WorkoutProgram.TIMEREADONLY_TYPE)  // don't allow changing read-only field

            onValueEntered: {
                if(keypad.value && keypad.value != "0.0") {
                    var item = model.get(mainProgItem.selectedIndex)
                    var tmp
                    mainProgItem.valueEntered = true;
                    if(item.units === "%"){
                        tmp = workoutProgram.tableIndexOfClosestValue(workoutProgram.inclineTable,keypad.value*10)
                        console.debug(tmp + " index")
                        model.setProperty(mainProgItem.selectedIndex, "dataValue", tmp);
                    } else if (SetupJS.isSpeedType(item.fieldType) && options.metricUnits) {
                        // when the user is using metric, find the closest possible mph value in the speed table to correct their entry
                        tmp = Globals.kmToMile(parseFloat(value))
                        tmp = workoutProgram.tableClosestValue(workoutProgram.speedTable, Math.round(tmp*10))
                        if (isNaN(tmp))
                            tmp = workoutProgram.speedTable[0]
                        model.setProperty(mainProgItem.selectedIndex, "dataValue", tmp/10)
                    } else if (SetupJS.isStrideType(item.fieldType) && options.metricUnits) {
                        tmp = Globals.cmToInch(parseFloat(value))
                        tmp = Math.round(tmp)
                        model.setProperty(mainProgItem.selectedIndex, "dataValue", tmp)
                    }
                    else
                        model.setProperty(mainProgItem.selectedIndex, "dataValue", keypad.value);
                    mainProgItem.valueEntered = false;

                    var corrections = SetupJS.validateParam(mainProgItem.selectedIndex)
                    if (item.fieldType === WorkoutProgram.SEGMENT_TYPE) {
                        mainProgItem.numSegs = model.get(mainProgItem.selectedIndex).dataValue
                    }
                    mainProgItem.valueEntered = false;

                    // go to next field when enter pressed, if entry was not corrected by validation
                    if(corrections == 0) {
                        myListView.incrementCurrentIndex()
                        mainProgItem.selectedIndex = myListView.currentIndex
                        mainProgItem.fieldIndex = myListView.currentIndex+1;
                    }
                }
            }
            onKeyPress: {
                if (key != "ENTER") {
                    if(model.get(mainProgItem.selectedIndex).units === "%"){
                        ;
                    }
                    else {
                        if (keypad.value) {
                            var item = model.get(mainProgItem.selectedIndex)
                            var val = keypad.value
                            if (SetupJS.isSpeedType(item.fieldType) && options.metricUnits) {
                                val = Globals.kmToMile(parseFloat(val))
                            } else if (SetupJS.isStrideType(item.fieldType) && options.metricUnits) {
                                val = Globals.cmToInch(parseFloat(val))
                                val = Math.round(val)
                            }
                            model.setProperty(mainProgItem.selectedIndex, "dataValue", val);

                            if (item.fieldType === WorkoutProgram.TIME_TYPE) {
                                SetupJS.calculateMaxSegmentsFromTime()
                            }

                        }
                    }
                }
            }
        }

        Text {
            id: prog_setup_desc_text
            anchors.top: parent.top
            anchors.topMargin: 200
            anchors.right: parent.right
            anchors.rightMargin: 10
            width: 350
            text: {
                if (programIndex >= WorkoutProgram.MANUAL_PROG && programIndex < WorkoutProgram.SAVED_WORKOUT) {
                    // Subtract 3 from programIndex to account for NO_PROG and QUICK_START_PROG and TREAD_QUICK_START_PROG
                    var str = Fields.workout_programs[(programIndex - 3)][(fieldIndex - 1)]
                    return str ? application.translateText("Setup", str) + application.emptyString : ""
                }
                else if (programIndex == WorkoutProgram.SAVED_WORKOUT) {
                    if (mainProgItem.customProgType == WorkoutProgram.CUSTOM_PACE_PROG ||
                            mainProgItem.customProgType == WorkoutProgram.CUSTOM_HILLS_PROG ||
                            mainProgItem.customProgType == WorkoutProgram.CUSTOM_ULTRA_PROG ||
                            mainProgItem.customProgType == WorkoutProgram.TREAD_CUSTOM_HILLS_PROG ||
                            mainProgItem.customProgType == WorkoutProgram.TREAD_CUSTOM_SPEED_PROG ||
                            mainProgItem.customProgType == WorkoutProgram.TREAD_CUSTOM_ULTRA_PROG) {
                        var str = Fields.workout_programs[(mainProgItem.customProgType - 3)][(fieldIndex - 1)]
                        return str ? application.translateText("Setup", str) + application.emptyString : ""
                    } else {
                        return "NA"
                    }
                }
                else {
                    return "NA"
                }
            }
            color: "black"
            wrapMode: Text.WordWrap
            lineHeight: 1.3
            horizontalAlignment: Text.AlignRight
            font.pixelSize: (text.length > 300) ? 18 : 20
        }

        Component {
            id: myDelegate
            Item {
                id: dataElement

                property real detailsOpacity : 0
                property real levelDetailsOpacity : 0
                property real expandedDetails1Opacity : 0
                property real expandedDetails2Opacity : 0
                property real segmentDetailsOpacity : 0
                property string expandedState: getExpandedState(fieldType)
                property string closedState: getClosedState(fieldType)
                property alias _slider3: sliderObject3

                function getExpandedState(fieldtype) {
                    switch(fieldtype)
                    {
                    case WorkoutProgram.MAX_SPEED_TYPE:
                    case WorkoutProgram.MAX_INCLINE_TYPE:
                    case WorkoutProgram.SPEED_TYPE:
                    case WorkoutProgram.RUN_SPEED_TYPE:
                    case WorkoutProgram.WORK_SPEED_TYPE:
                    case WorkoutProgram.WALK_SPEED_TYPE:
                    case WorkoutProgram.REST_SPEED_TYPE:
                    case WorkoutProgram.TIME_TYPE:
                    case WorkoutProgram.PACE_TYPE:
                    case WorkoutProgram.WORKLOAD_TYPE:
                    case WorkoutProgram.AGE_TYPE:
                    case WorkoutProgram.WEIGHT_TYPE:
                    case WorkoutProgram.DISTANCE_TYPE:
                    case WorkoutProgram.INCLINE_TYPE:
                    case WorkoutProgram.CALORIES_TYPE:
                    case WorkoutProgram.MIN_STRIDE_TYPE:
                    case WorkoutProgram.MAX_STRIDE_TYPE:
                        return "Details"
                        break;
                    case WorkoutProgram.LEVEL_TYPE:
                        return isTreadmill ? "LevelDetails" : "Details"
                    case WorkoutProgram.GENDER_TYPE:
                        return "GenderDetails"
                        break;
                    case WorkoutProgram.DISTANCE_QUICK_TYPE:
                        return "ExpandedDetails1"
                        break;
                    case WorkoutProgram.WORK_HEARTRATE_TYPE:
                    case WorkoutProgram.REST_HEARTRATE_TYPE:
                    case WorkoutProgram.TARGET_HEARTRATE_TYPE:
                        return "ExpandedDetails2"
                        break;
                    case WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE:
                        return "Expanded2"
                        break;
                    case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                    case WorkoutProgram.FIXED_DISTANCE_TYPE:
                    case WorkoutProgram.TIMEREADONLY_TYPE:
                        return "Expanded3"
                        break;
                    case WorkoutProgram.SEGMENT_TYPE:
                        return "SegmentDetails"
                        break;
                    }
                }

                function getClosedState(fieldtype) {
                    switch(fieldtype)
                    {
                    case WorkoutProgram.WORK_HEARTRATE_TYPE:
                    case WorkoutProgram.REST_HEARTRATE_TYPE:
                    case WorkoutProgram.TARGET_HEARTRATE_TYPE:
                    case WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE:
                        return "Expanded2"
                        break;
                    case WorkoutProgram.FIXED_DISTANCE_TYPE:
                    case WorkoutProgram.TIMEREADONLY_TYPE:
                    case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                        return "Expanded3"
                        break;
                    default:
                        return ""
                    }
                }

                state: ListView.isCurrentItem ? expandedState : closedState
                width: 425
                height: myListView.getBaseFieldHeight(model.fieldType)
                x: 10

                Image {
                    id: background
                    anchors.left: parent.left
                    anchors.leftMargin: 5
                    anchors.top: parent.top
                    anchors.topMargin: 10
                    width: 425
                    height: dataElement.height
                    source: myListView.getBackgroundSource(model.fieldType)
                    z: 1

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            // validate a field as it is exited
                            SetupJS.validateParam(myListView.currentIndex)

                            var item = myListView.model.get(mainProgItem.selectedIndex)
                            if (SetupJS.isSpeedType(item.fieldType) && options.metricUnits) {
                                // when the user is using metric, find the closest possible mph value in the speed table to correct their entry
                                var tmp
                                tmp = parseFloat(item.dataValue)
                                tmp = workoutProgram.tableClosestValue(workoutProgram.speedTable, Math.round(tmp*10))
                                if (isNaN(tmp))
                                    tmp = workoutProgram.speedTable[0]
                                myListView.model.setProperty(mainProgItem.selectedIndex, "dataValue", tmp/10)
                            }

                            // change to item when clicked, unless it is a read-only type
//                            if(fieldType != WorkoutProgram.HEARTRATEREADONLY_TYPE && fieldType != WorkoutProgram.FIXED_DISTANCE_TYPE)
                                myListView.currentIndex = index

                            // follow selected item
                            mainProgItem.selectedIndex = indexValue.text;
                            mainProgItem.fieldIndex = model.fieldIndex;
                        }
                    }
                }
                // folded up state elements
                Text {
                    id: itemName
                    anchors.top: parent.top
                    anchors.topMargin: 24
                    anchors.left: parent.left
                    anchors.leftMargin: 20
                    text: qsTranslate("WorkoutSetup", SetupJS.getItemLabel(model)) + SetupJS.getItemAdditionalLabel(model) + application.emptyString
                    font.pointSize: 20
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                }

                Text {
                    id: itemUnits
                    anchors.top: parent.top
                    anchors.topMargin: 32
                    anchors.right: parent.right
                    anchors.rightMargin: 20
                    text: model.units
                    font.pointSize: 14
                    wrapMode: Text.WordWrap
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                }
                Text {
                    id: itemValue
                    anchors.bottom: itemUnits.bottom
                    anchors.bottomMargin: -2
                    anchors.right: itemUnits.left
                    anchors.rightMargin: 5
                    text: SetupJS.getValueString(model)
                    font.pointSize: 20
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                }
                // expanded state elements
                Text {
                    id: itemUnitsDetails
                    anchors.top: parent.top
                    anchors.topMargin: 87
                    anchors.right: parent.right
                    anchors.rightMargin: 15
                    text: model.units
                    font.pointSize: 20
                    wrapMode: Text.WordWrap
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                }
                ImageButton {
                    id: hrChartButton
                    anchors.top: parent.top
                    anchors.topMargin: 15
                    anchors.right: parent.right
                    anchors.rightMargin: -60
                    downimage: "graphics/224/Data_HRC_Cruise_Down.png"
                    upimage: "graphics/224/Data_HRC_Cruise_Up.png"
                    icon: ""
                    z: 2
                    visible: {
                        if (model.fieldType == WorkoutProgram.WORK_HEARTRATE_TYPE || model.fieldType == WorkoutProgram.REST_HEARTRATE_TYPE || model.fieldType == WorkoutProgram.TARGET_HEARTRATE_TYPE || model.fieldType == WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE) {
                            return true;
                        }
                        else {
                            return false;
                        }
                    }

                    onClicked: {
                        showHRChart();
                    }
                }
                Text {
                    // Value seen while editing expanded item
                    id: itemValueDetails
                    anchors.bottom: itemUnitsDetails.bottom
                    anchors.bottomMargin: -4
                    anchors.right: itemUnitsDetails.left
                    anchors.rightMargin: 5
                    text: SetupJS.getValueString(model)

                    font.pointSize: 34
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                    onTextChanged: {
                        SetupJS.itemValueDetailsTextChanged(sliderObject, sliderObject2, sliderObject3, genderSlider, model)
                    }
                }

                Text {
                    id: hrDividerText
                    anchors.top: parent.top
                    anchors.topMargin: 72
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.horizontalCenterOffset: 40
                    text: "<Center>/<br>"+qsTr("OR")+"</Center>" + application.emptyString
                    font.pointSize: 14
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                }

                Text {
                    id: hrPercentageUnits
                    anchors.top: parent.top
                    anchors.topMargin: 83
                    anchors.right: parent.right
                    anchors.rightMargin: 20
                    text: qsTr("%MAX") + application.emptyString
                    font.pointSize: text.length > 6 ? 12 : 18
                    wrapMode: Text.WordWrap
                    width: 80
                    height: 40
                    verticalAlignment: Text.AlignBottom
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                }

                Text {
                    id: hrPercentageValue
                    anchors.bottom: hrPercentageUnits.bottom
                    anchors.bottomMargin: -6
                    anchors.right: hrPercentageUnits.left
                    anchors.rightMargin: 5
                    text: model.heartRatePercentage
                    font.pointSize: 32
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                }

                Text {
                    id: indexValue
                    text: model.selectedIndex
                    font.pointSize: 24
                    color: "black"
                    visible: false
                }

                Item {
                    // slider control box seen after expanding most simple items (age, weight, time, etc.)
                    id: details
                    enabled: visible && opacity > 0

                    property int value: sliderObject.value

                    height: 155
                    width: 460
                    anchors.top: dataElement.top
                    anchors.topMargin: 85
                    anchors.left: dataElement.left
                    anchors.leftMargin: 50
                    opacity: dataElement.detailsOpacity

                    Image {
                        id: dataIndicatorBackground
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: "graphics/setup/Setup_Large_Up_Step2.png"
                    }

                    SliderControl {
                        id: sliderObject
                        repeat: true

                        Component.onCompleted: {
                            valueProxy = model.dataValue
                        }

                        onValueChanged: {
                            SetupJS.sliderObjectValueChanged(model, value, valueProxyChange, index)
                        }
                        showValue: false
                        maxValue: model.maxValue
                        minValue: model.minValue
                        value: model.dataValue
                        precision: model.precision
                        stepValue: {
                            if (model.precision == 0) {
                                return 1;
                            }
                            else if (model.precision == 1) {
                                return 0.1;
                            }
                        }

                        anchors.top: parent.top
                        anchors.topMargin: 75
                        anchors.left: parent.left
                        anchors.leftMargin: 45
                        width: 350
                        height: 34
                    }
                }

                Item {
                    // slider control box seen after expanding gender control
                    id: genderDetails
                    enabled: visible && opacity > 0
                    height: 155
                    width: 460
                    anchors.top: dataElement.top
                    anchors.topMargin: 85
                    anchors.left: dataElement.left
                    anchors.leftMargin: 50
                    opacity: 0

                    Image {
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: "graphics/setup/Setup_Large_Up_Step2.png"
                    }

                    SliderControl {
                        id: genderSlider

                        onValueChanged: {
                            SetupJS.genderSliderValueChanged(model, value, index)
                        }
                        showValue: false
                        maxValue: 1 // FEMALE
                        minValue: 0 // MALE
                        value: model.dataValue
                        precision: 0
                        showButtons: false
                        anchors.top: parent.top
                        anchors.topMargin: 75
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.horizontalCenterOffset: -20
                        width: 300 - femaleText.width - maleText.width
                        height: 34

                        Text {
                            id: maleText
                            width:paintedWidth
                            anchors { right: parent.left ; rightMargin: 40; verticalCenter: parent.verticalCenter }
                            text: qsTr("MALE") + application.emptyString
                            MouseArea {
                                anchors.fill: parent ; onClicked: genderSlider.value = 0
                            }
                        }

                        Text {
                            id: femaleText
                            width:paintedWidth
                            text: qsTr("FEMALE") + application.emptyString
                            anchors { left: parent.right ; leftMargin: 40; verticalCenter : parent.verticalCenter  }
                            MouseArea {
                                anchors.fill: parent ; onClicked: genderSlider.value = 1
                            }
                        }
                    }
                }

                Item {
                    // distance selection drop-down with preset distance buttons
                    id: expandedDetails1
                    enabled: visible && opacity > 0
                    height: 155
                    width: 460
                    anchors.top: dataElement.top
                    anchors.topMargin: 85
                    anchors.left: dataElement.left
                    anchors.leftMargin: 40
                    opacity: dataElement.expandedDetails1Opacity

                    Image {
                        id: dataIndicatorBackground2
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: "graphics/setup/Setup_Large_Up_Step2.png"
                    }

                    ImageButton {
                        id: fiveKButton
                        property bool valueMatches: (dataValue == 5) && (units == "K")

                        anchors.left: dataIndicatorBackground2.left
                        anchors.leftMargin: 50
                        anchors.verticalCenter: dataIndicatorBackground2.verticalCenter
                        anchors.verticalCenterOffset: 10
                        downimage: "graphics/205/SM_Button_Down.png"
                        upimage: valueMatches ? "graphics/205/SM_Button_Down.png": "graphics/205/SM_Button_Up.png"
                        fontsize: 14
                        label_color: "black"
                        label: qsTr("5K") + application.emptyString
                        onClicked: {
                            myListView.model.setProperty(index, "dataValue", 5);
                            myListView.model.setProperty(index, "units", "K");
                            dataItemChanged()
                        }
                    }

                    ImageButton {
                        id: tenKButton
                        property bool valueMatches: (dataValue == 10) && (units == "K")
                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG

                        anchors.left: fiveKButton.right
                        anchors.leftMargin: 25
                        anchors.verticalCenter: dataIndicatorBackground2.verticalCenter
                        anchors.verticalCenterOffset: 10
                        downimage: "graphics/205/SM_Button_Down.png"
                        upimage: valueMatches ? "graphics/205/SM_Button_Down.png": "graphics/205/SM_Button_Up.png"
                        fontsize: 14
                        label_color: "black"
                        label: qsTr("10K") + application.emptyString
                        onClicked: {
                            myListView.model.setProperty(index, "dataValue", 10);
                            myListView.model.setProperty(index, "units", "K");
                            dataItemChanged()
                        }
                    }

                    ImageButton {
                        property bool valueMatches: (dataValue == 2) && (units == "MI")

                        id: twoMileButton
                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                        anchors.left: tenKButton.right
                        anchors.leftMargin: 25
                        anchors.verticalCenter: dataIndicatorBackground2.verticalCenter
                        anchors.verticalCenterOffset: 10
                        downimage: "graphics/205/SM_Button_Down.png"
                        upimage: valueMatches ? "graphics/205/SM_Button_Down.png": "graphics/205/SM_Button_Up.png"
                        fontsize: 14
                        label_color: "black"
                        label: qsTr("2MI") + application.emptyString
                        onClicked: {
                            myListView.model.setProperty(index, "dataValue", 2);
                            myListView.model.setProperty(index, "units", "MI");
                            dataItemChanged()
                        }
                    }

                    ImageButton {
                        property bool valueMatches: (dataValue == 4) && (units == "MI")

                        id: fourMileButton
                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                        anchors.left: twoMileButton.right
                        anchors.leftMargin: 25
                        anchors.verticalCenter: dataIndicatorBackground2.verticalCenter
                        anchors.verticalCenterOffset: 10
                        downimage: "graphics/205/SM_Button_Down.png"
                        upimage: valueMatches ? "graphics/205/SM_Button_Down.png": "graphics/205/SM_Button_Up.png"
                        fontsize: 14
                        label_color: "black"
                        label: qsTr("4MI") + application.emptyString
                        onClicked: {
                            myListView.model.setProperty(index, "dataValue", 4);
                            myListView.model.setProperty(index, "units", "MI");
                            dataItemChanged()
                        }
                    }
                }

                Item {
                    // heart rate setting drop down sliders
                    id: expandedDetails2
                    enabled: visible && opacity > 0
                    height: 155
                    width: 460
                    anchors.top: dataElement.top
                    anchors.topMargin: 85
                    anchors.left: dataElement.left
                    anchors.leftMargin: 40
                    opacity: dataElement.expandedDetails2Opacity

                    Image {
                        id: dataIndicatorBackground3
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: "graphics/setup/Setup_Large_Up_Step2.png"
                    }

                    SliderControl {
                        id: sliderObject2

                        onValueChanged: {
                            SetupJS.sliderObject2ValueChanged(model, value, index)
                        }
                        showValue: false
                        maxValue: model.maxValue
                        minValue: model.minValue
                        value: model.dataValue
                        anchors.top: parent.top
                        anchors.topMargin: 75
                        anchors.left: parent.left
                        anchors.leftMargin: 45
                        width: 350
                        height: 34
                    }
                }

                Item {
                    // drop down slider for setting number of segments for custom profile programs
                    id: segmentDetails
                    enabled: visible && opacity > 0
                    height: 155
                    width: 460
                    anchors.top: dataElement.top
                    anchors.topMargin: 85
                    anchors.left: dataElement.left
                    anchors.leftMargin: 50
                    opacity: dataElement.segmentDetailsOpacity

                    Image {
                        id: dataIndicatorBackground5
                        anchors.top: sliderObject3.top
                        anchors.topMargin: 15
                        anchors.left: sliderObject3.left
                        anchors.leftMargin: 15
                        source: "graphics/setup/Setup_Large_Up_Step3.png"
                    }

                    Text {
                        id: calculatedSegTime1
                        anchors.verticalCenter: dataIndicatorBackground5.verticalCenter
                        anchors.left: dataIndicatorBackground5.left
                        anchors.leftMargin: 30
                        text: qsTr("SEGMENT TIME") + application.emptyString
                        font.pointSize: 18
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Text {
                        id: calculatedSegTimeValue1
                        anchors.verticalCenter: dataIndicatorBackground5.verticalCenter
                        anchors.right: dataIndicatorBackground5.right
                        anchors.rightMargin: 40
                        text: Format.formatTimeString(mainProgItem.segTime);
                        font.pointSize: 32
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Text {
                        id: calculatedSegTimeNote
                        anchors.verticalCenter: dataIndicatorBackground5.verticalCenter
                        anchors.verticalCenterOffset: 30
                        anchors.right: dataIndicatorBackground5.right
                        anchors.rightMargin: 40
                        text: qsTr("(MINIMUM SEGMENT TIME IS 1:00)") + application.emptyString
                        font.pointSize: 12
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Image {
                        id: dataIndicatorBackground4
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: "graphics/setup/Setup_Large_Up_Step2.png"
                    }

                    SliderControl {
                        id: sliderObject3

                        onValueChanged: {
                            SetupJS.sliderObject3ValueChanged(model, value, index)
                        }
                        showValue: false
                        minValue: model.minValue
                        maxValue: model.maxValue
                        value: model.dataValue
                        precision: model.precision
                        stepValue: {
                            if (model.precision == 0) {
                                return 1;
                            }
                            else if (model.precision == 1) {
                                return 0.1;
                            }
                        }

                        anchors.top: parent.top
                        anchors.topMargin: 75
                        anchors.left: parent.left
                        anchors.leftMargin: 45
                        width: 350
                        height: 34
                    }
                }

                Item {
                    id: levelDetails
                    enabled: visible && opacity > 0
                    height: 155
                    width: 460
                    anchors.top: dataElement.top
                    anchors.topMargin: 175
                    anchors.left: dataElement.left
                    anchors.leftMargin: 110
                    z: details.z - 1
                    opacity: dataElement.levelDetailsOpacity

                    Text {
                        id: levelSubLabel
                        anchors.verticalCenter: dataIndicatorBackground7.verticalCenter
                        anchors.left: dataIndicatorBackground7.left
                        anchors.leftMargin: 30
                        text: SetupJS.levelDetailsGetLabel() + application.emptyString
                        font.pointSize: 18
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Text {
                        id: levelSubValue
                        anchors.verticalCenter: dataIndicatorBackground7.verticalCenter
                        anchors.right: levelSubUnits.left
                        anchors.rightMargin: 1
                        text: SetupJS.levelDetailsGetSubValue(details.value)
                        font.pointSize: 32
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Text {
                        id: levelSubUnits
                        anchors.bottomMargin: height/2
                        anchors.bottom: levelSubValue.bottom
                        anchors.right: dataIndicatorBackground7.right
                        anchors.rightMargin: 40
                        text: SetupJS.levelDetailsGetUnits() + application.emptyString
                        font.pointSize: 12
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Image {
                        id: dataIndicatorBackground7
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: "graphics/setup/Setup_Large_Up_Step3.png"
                    }

                }

                states: [
                    State {
                        name: "Details"
                        PropertyChanges { target: background; height: 160; width: 455; anchors.leftMargin: -1; anchors.topMargin: 1; source: "graphics/setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 225 } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; detailsOpacity: 1} // Make details visible
                    },
                    State {
                        name: "LevelDetails"
                        PropertyChanges { target: background; height: 160; width: 455; anchors.leftMargin: -1; anchors.topMargin: 1; source: "graphics/setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 315 } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; detailsOpacity: 1} // Make details visible
                        PropertyChanges { target: dataElement; levelDetailsOpacity: 1} // Make details visible
                    },
                    State {
                        name: "ExpandedDetails1"
                        PropertyChanges { target: background; height: 160; width: 455; anchors.leftMargin: -1; anchors.topMargin: 1; source: "graphics/setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 225 } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; expandedDetails1Opacity: 1} // Make details visible
                    },
                    State {
                        name: "Expanded2"
                        PropertyChanges { target: itemUnits; anchors.top: parent.top;
                            anchors.topMargin: application.currentLanguage == "ar" ? 60 : 90;
                            anchors.right: parent.right; anchors.rightMargin: 200;
                            width: application.currentLanguage == "ar" ? 120 : implicitWidth;
                            font.pointSize: application.currentLanguage == "ar" ? 12 : 18
                        }
                        PropertyChanges { target: itemValue; anchors.bottomMargin: -6; font.pointSize: 32 }
                        PropertyChanges { target: hrDividerText; visible: true }
                        PropertyChanges { target: hrPercentageValue; visible: true }
                        PropertyChanges { target: hrPercentageUnits; visible: true }
                    },
                    State {
                        name: "Expanded3"
                        PropertyChanges { target: itemUnits; anchors.top: parent.top; anchors.topMargin: 90; anchors.right: parent.right; anchors.rightMargin: 20; font.pointSize: 18 }
                        PropertyChanges { target: itemValue; anchors.bottomMargin: -6; font.pointSize: 32 }
                    },
                    State {
                        name: "ExpandedDetails2"
                        PropertyChanges { target: background; height: 160; width: 455; anchors.leftMargin: -1; anchors.topMargin: 1; source: "graphics/setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemUnitsDetails; anchors.top: parent.top;
                            anchors.topMargin: application.currentLanguage == "ar" ? 81 : 90;
                            anchors.right: parent.right; anchors.rightMargin: 200;
                            font.pointSize: application.currentLanguage == "ar" ? 12 : 18;
                            width: application.currentLanguage == "ar" ? 140 : implicitWidth
                        }
                        PropertyChanges { target: itemValueDetails; anchors.bottomMargin: -6; font.pointSize: 32 }
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: hrDividerText; visible: true }
                        PropertyChanges { target: hrPercentageValue; visible: true }
                        PropertyChanges { target: hrPercentageUnits; visible: true }
                        PropertyChanges { target: dataElement; height: 225 } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; expandedDetails2Opacity: 1} // Make details visible
                    },
                    State {
                        name: "SegmentDetails"
                        PropertyChanges { target: background; height: 160; width: 455; anchors.leftMargin: -1; anchors.topMargin: 1; source: "graphics/setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 315 } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; segmentDetailsOpacity: 1} // Make details visible
                    },
                    State {
                        name: "GenderDetails"
                        PropertyChanges { target: background; height: 160; width: 455; anchors.leftMargin: -1; anchors.topMargin: 1; source: "graphics/setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 225 } // Fill the entire list area with the detailed view
                        PropertyChanges { target: genderDetails; opacity: 1} // Make details visible
                    }

                ]

                transitions: [
                    Transition {
                        to: "Details"
                        reversible: true
                    },
                    Transition {
                        to: "ExpandedDetails1"
                        reversible: true
                    },
                    Transition {
                        to: "ExpandedDetails2"
                        reversible: true
                    },
                    Transition {
                        to: "SegmentDetails"
                        reversible: true
                    }
                ]
            }
        }
    }

    Rectangle { id: cautionMessage

        property variant item: myListView.model ? myListView.model.get(selectedIndex) : undefined
        property bool showInclineMessage: item !== undefined && isTreadmill && (item.fieldType === WorkoutProgram.INCLINE_TYPE || item.fieldType === WorkoutProgram.MAX_INCLINE_TYPE || (item.fieldType === WorkoutProgram.LEVEL_TYPE && workoutProgram.programIndex !== WorkoutProgram.TREAD_LEG_SHAPER_PROG))
        property bool showWorkloadMessage: item !== undefined && !isTreadmill && (item.fieldType === WorkoutProgram.LEVEL_TYPE || item.fieldType === WorkoutProgram.WORKLOAD_TYPE)
        property bool showSpeedMessage: item !== undefined && (SetupJS.isSpeedType(item.fieldType) || (item.fieldType === WorkoutProgram.LEVEL_TYPE && workoutProgram.programIndex === WorkoutProgram.TREAD_LEG_SHAPER_PROG))

        visible: showInclineMessage || showWorkloadMessage || showSpeedMessage

        anchors.bottom: parent.bottom
        anchors.bottomMargin: 10
        anchors.left: parent.left
        anchors.leftMargin: 10
        width: childrenRect.width
        height:childrenRect.height
        color: "#e8e8e8"

        Image{
            id: cautionSymbol
            source: "graphics/Icon_Warning.png"
        }

        Text{
            id: cautionTitle
            text: qsTr("CAUTION") + application.emptyString
            font.bold: true
            font.pixelSize: 22
            anchors.left: cautionSymbol.right
            anchors.leftMargin: 5
            anchors.verticalCenter: cautionSymbol.verticalCenter
            anchors.verticalCenterOffset: 0
        }

        Text{
            id: cautionText
            width: 450
            wrapMode: Text.WordWrap
            text: {
                if (cautionMessage.showInclineMessage)
                    return qsTr("Use caution when selecting incline level. Injury can result if not prepared for incline levels beyond your physical limits.") + application.emptyString
                else if (cautionMessage.showWorkloadMessage)
                    return qsTr("Use caution when selecting level. Injury can result if not prepared for workload resistance beyond your physical limits.") + application.emptyString
                else if (cautionMessage.showSpeedMessage)
                    return qsTr("Use caution when selecting speed. Injury can result if not prepared for speeds beyond your physical limits.") + application.emptyString
                else
                    return ""
            }
            font.pixelSize: 16
            anchors.left: cautionSymbol.right
            anchors.leftMargin: 5
            anchors.top: cautionTitle.bottom
            anchors.topMargin: 5
        }
    }

}

