import QtQuick 2.4
import FileIO 1.0
import QtQuick.XmlListModel 2.0

// ChannelList manages the list of television programming sources.
// The "channels" property is a ListModel containing the programming list.
// roles:
// id, name, number, freq, input, standard, modulation, program, logo, mrl
//
// Each channel has an "id" role which is unique within the list.

Item {
    id: root

    property string norm: "NTSC-M"
    property string analogDevice: "/dev/video0"
    property string digitalDevice: "/dev/dvb/adapter0"
    property string digitalModulation: "8VSB"
    property alias channels: channelList
    property string tunerType: tv.tunerType
    property string channelFile: "/data/tv_conf.xml"

    signal loadComplete()

    function getMrlStandard(tunertype) {
            switch (tunertype) {
            case "ATSC":
                return "atsc://"
            case "DVB-T":
                return "dvb-t://"
            case "DVB-T2":
                return "dvb-t2://"
            case "DVB-C":
                return "dvb-c://"
            case "QAM":
                return "cqam://"
            case "ISDB-T":
                return "isdb-t://"
            case "DTMB":
                return "dtmb://"
            default:
                return "dvb://"
            }
    }

    // return index of channel with given unique ID.
    // Return -1 if not found.
    function findChannel(ID) {
        for(var i=0 ; i < channelList.count ; i++)
            if(ID == channelList.get(i).id)
                return i
        return -1
    }

    function filterDuplicatePrograms() {
        // remove subchannels that share the same video PID, keeping only the first
        var seen = {}
        for(var i=0 ; i < channelList.count ; i++) {
            var chan = channelList.get(i)
            if (seen[chan.freq] === undefined) {
                seen[chan.freq] = {}
                seen[chan.freq][chan.vpid] = 1
            } else if (seen[chan.freq][chan.vpid] === undefined) {
                seen[chan.freq][chan.vpid] = 1
            } else if (seen[chan.freq][chan.vpid] !== undefined) {
                channelList.remove(i)
                i = i - 1
                continue
            }
        }
    }

    function addChannel(item) {
        if(!item.id){ // no ID defined
            item.id = channelIDString(item)
        }
        var idx = findChannel(item.id)
        if(idx >=0 ) {
            channels.set(idx, item)  // replace existing channel with corresponding ID
        }
        else {
            var blank = {id: item.id, enabled:"true", number:"", icon:"", position:"", tunertype:"", frontend:"",
                freq:"", mrl:"", analog:false, input:0, norm:"", name:"", icon:"", source: "", category: "",
                vpid:"", apids:"", ttypid:"", program:"", modulation:"",
                symbolrate:"", fec:"", inversion:"", bandwidth:"", transmode:"", hier:"", guard:"" }
            channels.append(blank)
            channels.set(channels.count-1, item)  // new ID, just append
        }
    }

    function removeChannel(ID) {
        for(var i=0 ; i < channelList.count ; i++) {
            if(ID == channelList.get(i).id) {
                channelList.remove(i)
                return
            }
        }
    }

    /* Return channel item with given index, return null if index out of bounds */
    function getChannelItem(index) {
        if(channelList.count == 0){
            return null
        }

        if(index >= channelList.count){
            return null
        }

        if(index < 0){
            return null
        }

        return channelList.get(index);
    }

    function getAuxMrl(norm) {
        var mrl = ""
        mrl += "v4l2://"+root.analogDevice
        mrl += ":v4l2-input=1"
        mrl += ":v4l2-standard=" + norm
        return mrl
    }

    // return MRL string (vlc style) from index in channel list.  Return empty if not found.
    function mrl(index) {
        if(index > channelList.count)
            return ""
        var chan = channelList.get(index)
        if(chan.mrl)
            return chan.mrl // file MRL pre-defined

        // build a TV mrl from the components
        var mrl = ""
        if(chan.analog) {
            mrl += "v4l2://"+root.analogDevice
            mrl += ":v4l2-vbidev=/dev/vbi0"
            if(chan.freq && (chan.freq !== "0"))
                mrl += ":v4l2-tuner-frequency=" + chan.freq
            if("input" in chan)
                mrl += ":v4l2-input=" + chan.input
            var norm = chan.norm ? chan.norm : root.norm
            norm = norm.replace("-", "_")
            mrl += ":v4l2-standard=" + norm
        }
        else { // digital TV
            mrl += root.getMrlStandard(chan.tunertype)

            mrl += ":frequency=" + chan.freq
            var mod = chan.modulation ? chan.modulation : root.digitalModulation
            mrl += ":dvb-modulation=" + mod
            mrl += ":program=" + chan.program
            if(chan.frontend == "Cable") {
                mrl += ":dvb-srate=" + (chan.symbolrate * 1000)
                mrl += ":dvb-inversion=" + chan.inversion
                mrl += ":dvb-fec=" + chan.fec
            }
            else if(chan.frontend == "Terrestrial") {
                mrl += ":dvb-bandwidth=" + chan.bandwidth
                mrl += ":dvb-fec=" + chan.fec
                mrl += ":dvb-transmission=" + chan.transmode
                mrl += ":dvb-guard=" + chan.guard
                mrl += ":dvb-hierarchy=" + chan.hier
            }
        }

        if (tv.bcasCardPresent) {
            mrl += ":dvb-adapter=8"
        }

        return mrl
    }

    // parse an Mplayer channels.conf line and append it to the channels list
    function addChannelFromMplayerString(line) {
        var fields = line.trim().split(":")
        var item = { name:fields[0], number: (fields[1] / 6000000).toFixed(0), freq:fields[1], modulation:fields[2],
            analog: false, vpid:fields[3], apids:fields[4], ttypid:"-1",
            program:fields[5], logo:"", enabled:true}
        item.id = channelIDString(item)
        addChannel(item)
    }

    // read a MPlayer formatted channels.conf file into the channel list
    function readMplayerChannelsConf() {
        fileio.source = "/data/channels.conf"
        fileio.open()
        while(!fileio.atEnd())
            addChannelFromMplayerString(fileio.readLine())
        fileio.close()
    }

    // pack tuning data into a string for the "extra" property in tv_conf.xml file.
    function channelTuningDataString(item) {
        // tuning data format:
        // tunertype(ATSC, DVB, ISDB, ) | frequency | A=analog or D=digital |
        //   + Analog format
        // input | norm |
        //   + digital format
        // vpid | apids(, sep) | ttypid | service id | stream id |
        // constellation (8, 16, 64, 128, 256)modulation=(VSB, QAM, OFDM, QPSK)  (e.g. 8VSB, 64QAM)
        // | symbolrate | fec (0, 12, 23, 34, 45, 56, 67, 78, 89) |
        // inversion (0,1,-1) | bandwidth (5,6,7,8) |
        // QAM transmission mode (-1=auto, 2,4,8)
        // QAM guard (-1=auto, 1,4,8,16,32)
        // OFDM Hierarchy (-1=auto, 0,1,2,4)

        var extra = item.tunertype + "|" + (item.freq || "") + "|"

        if(item.mrl) { // predefined MRL
            return "MRL|"+item.mrl
        }

        if(item.analog) {
            var norm = item.norm ? item.norm : root.norm
            var input = (("input" in item) && (item.input !== "")) ? item.input : "0"
            extra += "A|" + input + "|" + (norm || "") + "|"
        }
        else {
            extra += "D|" +
                    (item.vpid || "") + "|" + (item.apids || "") + "|" + (item.ttypid || "") + "|" +
                    (item.program || "") + "|" +
                    (item.modulation || "") + "|" +
                    (item.frontend || "") + "|" +
                    (item.symbolrate || "") + "|" +
                    (item.fec || "") + "|" +
                    (item.inversion || "") + "|" +
                    (item.bandwidth || "") + "|" +
                    (item.transmode || "") + "|" +
                    (item.hier || "") + "|" +
                    (item.guard || "")
        }
        extra = extra.replace(/\,/g, ";")  // replace all commas with semicolons (to avoid problems in Netpulse netstrings)
        return extra
    }

    // create a unique ID based on frequency and other data
    // item - javascript dict representing the channel.
    function channelIDString(item) {
        var id
        // if an id is already defined, return it.
        if(item.id)
            return item.id
        // otherwise, build a unique ID from tuning data
        if(item.analog)
            id = "A" + (item.freq || "")
        else
            id = "D" + (item.freq || "") +"." + (item.program || "")

        return id
    }

    function escapeCDATA(str) {
        var cdata = String(str)
        return cdata.replace(/&/g, '&amp;')
                  .replace(/</g, '&lt;')
                  .replace(/>/g, '&gt;')
                  .replace(/"/g, '&quot;')
                  .replace(/'/g, '&apos;');
    }

    function xmlChannelString(item) {
        var id = channelIDString(item)
        var extra = "extra=\"" + escapeCDATA(channelTuningDataString(item)) + "\""
        var line = "<channel number=\"" + escapeCDATA(item.number) + "\" " +
                "name=\"" + escapeCDATA(item.name || "") + "\" "
        if(item.icon)
            line += "icon=\"" + (item.icon || "") + "\" "

        if (item.source) {
            line += "source=\"" + (item.source || "") + "\" "
        }

        if (item.category) {
            line += "category=\"" + (item.category || "") + "\" "
        }

        line += "id=\"" + escapeCDATA(id) + "\" " + "enabled=\"" + (item.enabled || false) + "\" " + extra
        line += "/>\n"

        return line
    }

    function saveChannels(path) {
        if(path == "")
            path = root.channelFile
        fileio.source = path
        fileio.fileMode = FileIO.READ_WRITE
        fileio.open()
        fileio.clear()
        fileio.writeLine("<tvconf version=\"2\">\n")
        fileio.writeLine("<channels>\n")
        for(var i=0 ; i < channels.count ; i++) {
            var item = channels.get(i)
            var line = xmlChannelString(item)

            fileio.writeLine(line)
        }
        fileio.writeLine("</channels>\n")
        fileio.writeLine("</tvconf>\n")
        fileio.close()
        syncDisk.start()
    }

    function readChannels(path) {
        if(path == "")
            path = root.channelFile
        xmlChannels.source = path
        xmlChannels.reload()
    }

    function clearChannels() {
        var path = root.channelFile
        fileio.source = path
        fileio.fileMode = FileIO.READ_WRITE
        fileio.open()
        fileio.clear()
        fileio.close()
        readChannels(root.channelFile)
    }

    function sortByCategory() {
        // This will be done with insertion sort if we switch back to on-the-fly SAP updates
        var buckets = {}
        for (var i = 0; i < channels.count; i++) {
            var chanItem = channels.get(i)
            if (!buckets[chanItem.category]) {
                // QML doesn't let you store a ListModel item in an array, so store its index
                buckets[chanItem.category] = [i]
            } else {
                buckets[chanItem.category].push(i)
            }
        }
        // Now re-insert in order of buckets
        var key
        dummyChanList.clear()
        for (key in buckets) {
            if (!buckets.hasOwnProperty(key))
                continue
            for (var i = 0; i < buckets[key].length; i++) {
                var item = channels.get(buckets[key][i])
                dummyChanList.append(item)
            }

        }
        // Why is it so terrible
        channels.clear()
        for (var i = 0; i < dummyChanList.count; i++) {
            channels.append(dummyChanList.get(i))
        }
    }

    ListModel {
        id: dummyChanList
        dynamicRoles: true
    }

    FileIO {    // digital channel scan file
        id: fileio
        onError: print("Channels File ERROR: " + error)
    }

    XmlListModel {
        id: xmlChannels

        //source: "file://" + root.channelFile
        query: "/tvconf/channels/channel"
        XmlRole { name: "number" ; query: "@number/string()" }
        XmlRole { name: "name" ; query: "@name/string()" }
        XmlRole { name: "enabled" ; query: "@enabled/string()" }
        XmlRole { name: "icon" ; query: "@icon/string()" }
        XmlRole { name: "source"; query: "@source/string()" }
        XmlRole { name: "category"; query: "@category/string()" }
        XmlRole { name: "position" ; query: "@position/string()" }
        XmlRole { name: "id" ; query: "@id/string()" }
        XmlRole { name: "extra" ; query: "@extra/string()" }

        onStatusChanged: {
            if(status == XmlListModel.Ready) {
                if(source != "")      // clear the list if reading from a file
                    channelList.clear()
                for(var i=0 ; i < xmlChannels.count ; i++) {
                    var item = xmlChannels.get(i)
                    var channel = {}
                    channel.name = item.name
                    channel.number = item.number
                    channel.icon = item.icon
                    channel.source = item.source
                    channel.category = item.category
                    channel.position = item.position
                    channel.id = item.id
                    channel.enabled = item.enabled
                    var extra = item.extra
                    var fields = extra.split("|")

                    if(fields[0] == "MRL") { // Raw MRL in tuning data
                        channel.mrl = fields[1]
                    }
                    else {
                        channel.tunertype = fields[0]
    //                    if(channel.tunertype != root.tunerType)
    //                        continue    // skip incompatible channels
                        channel.freq = fields[1]
                        channel.analog = (fields[2] == "A")
                        if(channel.analog) {
                            channel.input = parseInt(fields[3], 10)
                            channel.norm = fields[4]
                        }
                        else {
                            channel.vpid = fields[3]
                            channel.apids = fields[4]
                            channel.ttypid = fields[5]
                            channel.program = fields[6]
                            channel.modulation = fields[7]
                            channel.frontend = fields[8]
                            channel.symbolrate = fields[9]
                            channel.fec = fields[10]
                            channel.inversion = fields[11]
                            channel.bandwidth = fields[12]
                            channel.transmode = fields[13]
                            channel.hier = fields[14]
                            channel.guard = fields[15]
                        }
                    }
                    addChannel(channel)
                }
                root.loadComplete()
            }
        }
    }

    ListModel {
        id: channelList
        dynamicRoles: true
    }
}
