import QtQuick 2.4
import VLCVideo 1.0
import WorkoutRunData 1.0
import "string_format.js" as Format
import Components 1.0

// Video window to play video
Rectangle {
    id: root

    property alias videoMRL: videoPlayer.mrl
    property variant c360program: workoutProgram.c360Program
    property int phase: c360program.currentPhase
    property string videoPath: "/data/video/Cardio360/Spectrum"
    property string videoSubpath: options.modestyMode ? "Male" : (Math.random() >= 0.5 ? "Female" : "Male")
    property string introVideo: "Intro Clip.mp4"
    property int segmentTime: c360program.phaseTimeRemaining
    property bool inIntroMode: true

    signal phaseTitleChanged(string text)
    signal playbackComplete()

    color: "white"
    anchors.fill: parent

    function beginWorkout() {
        root.inIntroMode = false
        if (setVideoTimer.running) {
            setVideoTimer.stop()
        }
        workoutManager.start()
        root.phaseChanged(phase)
    }

    Connections {
        target: workoutManager
        onWorkoutModeChanged: {
            if (mode === WorkoutRunData.COOL_DOWN_MODE) {
                if (root.inIntroMode) {
                    workoutManager.resume()
                }

                root.inIntroMode = false
            }
        }
    }

    Component.onCompleted: {
        // if (intro mode enabled) {
        setVideoTimer.start()
        // }
    }

    Timer {
        id: setVideoTimer
        interval: 300
        running: false
        onTriggered: {
            // Set video on delay to avoid race condition where video is not set
            videoMRL = videoFullPath(introVideo)
            bannerForIntro.display()
        }
    }

    onPhaseChanged: {
        if (root.inIntroMode) {
            return
        }

        var text = qsTranslate("C360ViewContainer", c360Modes.title(phase))
        root.phaseTitleChanged(text)
        videoMRL = videoFullPath(c360Modes.get(phase).video)
        console.log("videoMRL=", videoMRL)
    }

    function videoFullPath(path) {
        return "file://" + root.videoPath + "/" + videoSubpath + "/" + path
    }

    function stopVideo() {
        videoPlayer.stop()
    }

    SimpleMessageBanner {
        id: bannerForIntro

        beep: false
        x:185
        y:170
        width:1000
        dismissable: false
        z: 10000

        property int index: 0
        ListModel {
            id: messageList
            ListElement { text: QT_TR_NOOP("Welcome to Cardio360") }
            ListElement { text: QT_TR_NOOP("Your Total Body Workout") }
        }

        function display() {
            buzzer.buttonPress()
            index = 0
            showMessage(qsTr(messageList.get(index).text), 3)
        }
        onTimedOut: {
            index++
            if (index >= messageList.count) {
                index = 0
                return
            }
            showMessage(qsTr(messageList.get(index).text), 3)
        }
    }

    VLCSurface {
        id: videoSurface

        backgroundColor: "white"
        player: videoPlayer
        aspect: videoPlayer.aspect
        playing: !videoPlayer.fullscreen && videoPlayer.playing

        anchors.fill: parent
    }

    VLCVideo {
        id: videoPlayer

        property bool fullscreen

        debug: false
        deinterlace: false
        loop: !inIntroMode
        onPlayCompleted: {
            if (inIntroMode) {
                beginWorkout()
            }
        }
    }

    Image {
        id: motionMan
        visible: !inIntroMode
        anchors {
            top: parent.top
            topMargin: 85
            left: parent.left
            leftMargin: 40
        }
        fillMode: Image.PreserveAspectFit
        smooth: true
        height: 260
        width: 262

        source: "graphics/c360/MotionMan/" + c360Modes.get(phase).icon
    }

    Text {
        id: instText
        width: 345
        visible: !inIntroMode
        anchors {
            right: parent.right
            rightMargin: 15
            top: parent.top
            topMargin: 175
        }

        text: qsTranslate("C360ViewContainer", c360Modes.instructions(phase)) + application.emptyString
    }

    Text {
        id: guideTextForIntro
        wrapMode: Text.WordWrap
        width: 345
        visible: inIntroMode
        anchors {
            right: parent.right
            rightMargin: 15
            top: parent.top
            topMargin: 175
        }

        text: qsTr("Watch the on-screen video for instructions to get the most out of your workout.") + application.emptyString
    }

    Item {
        id: segRect
        visible: !inIntroMode
        width: 270
        height: childrenRect.height
        anchors {
            top: parent.top
            topMargin: 275
            right: parent.right
            rightMargin: 36
        }

        Text {
            id: segTimeLabel

            width: 270
            horizontalAlignment: Text.AlignHCenter
            anchors.horizontalCenter: parent.horizontalCenter
            text: qsTranslate("C360View", "SEGMENT TIME REMAINING") + application.emptyString
            font.pointSize: 14
            wrapMode: Text.Wrap
        }
        Text {
            id: segTimeIndicator
            anchors {
                top: segTimeLabel.bottom
                horizontalCenter: parent.horizontalCenter
            }
            text: Format.formatTimeString(root.segmentTime)
            font.pointSize: 45
            font.bold: true
        }
    }

    Item {
        id: skipIntroText
        visible: root.inIntroMode
        width: childrenRect.width
        height: childrenRect.height

        anchors {
            bottom: parent.bottom
            bottomMargin: 35
            right: parent.right
            rightMargin: 40
        }

        Text {
            id: skipText1
            text: qsTr("SKIP INTRO")
            font.pixelSize: 20
        }
        Text {
            text: "X"
            anchors.left: skipText1.right
            anchors.leftMargin: 5
            font.pixelSize: 24
        }

        MouseArea {
            anchors.fill: parent
            onClicked: {
                bannerForIntro.hide()
                beginWorkout()
            }
        }
    }

}

