import TrackView 1.0
import QtQuick 2.4
import WorkoutRunData 1.0
import PID 1.0
import Brainboard 1.0
import WorkoutManager 1.0
import StorageMgr 1.0
import Components 1.0
import TrueTouch 1.0

import "MediaBrowser"
import "views.js" as TheViews

FocusScope {
    id: root
    width: 1366
    height: 768

    focus: main.state === "RunState"

    property alias container: workoutViewContainer

    property bool hrTargetVisibility: workoutProgram.isHRCProgram || workoutProgram.programIndex === WorkoutProgram.TREAD_GERKIN_TEST_PROG || workoutProgram.programIndex === WorkoutProgram.FITNESS_TEST_PROG
    property bool c360Available: workoutProgram.isC360Program
    property bool netpulseAvailable: options.netpulseEnabled && netpulse.connected
    property bool c360VideoAvailable: workoutProgram.isVideoProgram
    property bool showPaceSetter: workoutProgram.displayDataVars.paceSetterAvailable
    property alias currentViewName: workoutViewContainer.currentViewName
    property bool freezeTV: workoutViewContainer.isMoving || (workoutViewContainer.currentViewName != "TV_VIEW")
    property bool showingTV: currentViewName === "TV_VIEW"
    property bool workoutActive:  (aWorkoutRunData.mode == WorkoutRunData.WORKOUT_MODE) || (aWorkoutRunData.mode == WorkoutRunData.WARM_UP_MODE) || (aWorkoutRunData.mode == WorkoutRunData.COOL_DOWN_MODE)
    property bool bikeModeEnabled: machinecontrol.machineMode === MachineControl.MODE_CONSTANT_TORQUE && machineinfo.productTypeString == "Bike"
    property string unitState
    property variant stopPrompt: main.isTft ? tftStopPrompt : _stopPrompt
    property alias sideDataDisplay: sideDataDisplay
    property bool browserFullscreen: false
    property bool bikeModePromptShowing: bikeModePrompt.visible
    property bool tvMenuShowing: false
    property bool ipodVideoShown: false

    signal workoutStopped()
    signal workoutStarted()
    signal workoutPaused()
    signal workoutResumed()

    onUnitStateChanged: {
        if(unitState === "ENGLISH")
            brainboard.setUnitSystem(0)
        else if(unitState === "METRIC")
            brainboard.setUnitSystem(1)
    }

    function showC360Message(message) {
        if(netpulseAvailable) {
            infoBanner.showMessage(message, 3)
        } else {
            viewContainerBanner.showMessage(message, 5)
        }
    }

    function showMessage(message) {
        print("message shown:"+message)
        if(netpulseAvailable) {
            infoBanner.showMessage(message, 3)
        }
        else {
            infoPopup.showMessage(message, 3)
        }
    }

    function showBannerMessage(message) {
        infoBanner.showMessage(message, 3)
    }

    function stopButtonPressedHandler(){
        print("Stop/Pause Pressed")
        urlKeypad.process.write("CMD hide\n")
        inputKeypad.process.write("CMD hide\n")

        // CSA Cardio 360 can be in "intro mode". Show popup anyway.
        if (workoutActive || (main.isCSA && workoutProgram.isC360Program)) {
            if(options.resumeWorkoutTimeout){
                stopPrompt.show()
                stopPrompt.startTimeout(options.resumeWorkoutTimeout)
                workoutPaused()
            }
            else {
                stopPrompt.show()
                workoutPaused()
            }
        }
        if(workoutProgram.isFitnessTest)
            stopWorkout();
        if(workoutProgram.isHRCProgram)
            workoutManager.stopHRC()
    }

    function showBikeModePrompt() {
        bikeModePrompt.show()
        bikeModePrompt.startTimeout(5)
    }

    function resumeWorkout() {
        if (isTreadmill) {
            workoutManager.treadSpeed = 5
        }
        workoutManager.resume()
        aWorkoutRunData.resumeTime()
        workoutResumed()
    }

    // connect key presses from brainboard
    Connections {
        target: main.state === "RunState" ? keymapper : null

        onLevelUpPressed: if(workoutActive && !stopPrompt.visible) {
                              workoutManager.stopHRC()
                              workoutManager.stopFitnessTest()
                              workoutManager.increaseWorkloadLevel();
                          }
        onLevelDownPressed: if(workoutActive && !stopPrompt.visible) {
                                workoutManager.stopHRC()
                                workoutManager.stopFitnessTest()
                                workoutManager.decreaseWorkloadLevel();
                            }
        onInclineUpPressed: if(workoutActive && !stopPrompt.visible) {
                                workoutManager.stopHRC()
                                workoutManager.increaseIncline()
                            }
        onInclineDownPressed: if(workoutActive && !stopPrompt.visible) {
                                  workoutManager.stopHRC()
                                  workoutManager.decreaseIncline();
                              }
        onSpeedUpPressed: if(workoutActive && !stopPrompt.visible) {
                              workoutManager.stopHRC()
                              workoutManager.increaseSpeed(workoutManager.withLimit);
                          }
        onSpeedUpFirstPressed: if(workoutActive && !stopPrompt.visible) workoutManager.speedUpEnabled = true;
        onSpeedUpReleased: if(workoutActive && !stopPrompt.visible) workoutManager.speedUpEnabled = false;

        onSpeedDownPressed: if(workoutActive && !stopPrompt.visible) {
                                workoutManager.stopHRC()
                                workoutManager.decreaseSpeed();
                            }
        onStopPausePressed: stopButtonPressedHandler()

        onStrideUpPressed: {
            workoutManager.increaseStrideLevel()
        }

        onStrideDownPressed: {
            workoutManager.decreaseStrideLevel()
        }

        // TFT handling
        onStopPressed: {
            stopButtonPressedHandler()
        }

        onStartPressed: {
            if(workoutActive && isTft) {
                if (stopPrompt.visible) {
                    stopPrompt.hide()
                    root.resumeWorkout()
                }
            }
        }

        onStartQuickStartPressed: {
            if(workoutActive && isTft) {
                if (stopPrompt.visible) {
                    stopPrompt.hide()
                    root.resumeWorkout()
                }
            }
        }

        onPlusPressed: if(workoutActive) workoutManager.increaseWorkloadLevel();
        onMinusPressed: if(workoutActive) workoutManager.decreaseWorkloadLevel();
    }

    Connections {
        target: main.isTft && main.state === "RunState" && !isTreadmill && !tftStopPrompt.visible && !(currentViewName === "IPOD_VIEW" && root.ipodVideoShown) ? keymapper : null

        onBackPressed: {
            main.debugKeys("backPressed", "WorkoutScreen")
            stopButtonPressedHandler()
        }
    }

    Connections {
        target: main.state === "RunState" && ["TV_VIEW", "IPOD_VIEW"].indexOf(workoutViewContainer.currentViewName) === -1 ? keymapper : null

        onUpButtonPressed: {
            main.debugKeys("upButtonPressed", "WorkoutScreen")
            volume_controls.volumeUp();
        }

        onDownButtonPressed: {
            main.debugKeys("downButtonPressed", "WorkoutScreen")
            volume_controls.volumeDown();
        }
    }

    Connections{
        target: workoutManager
        onChangingSpeed: {
            if (newspeed > 0) {
                speedChangePrompt.newSpeed = newspeed
                speedChangePrompt.oldSpeed = oldspeed
                speedChangePrompt.show()
            }
        }

    }

    Loader {
        source: workoutProgram.programIndex === WorkoutProgram.STRIDE_INTERVALS_PROG ? "progs/stride_intervals.qml" : ""

    }

    TelevisionFullscreen {
        id: browserFullscreenFrame

        showHeadsetButton: !options.netpulseEnabled
        leftSideDisplay: sideDataDisplay.leftDisplay
        rightSideDisplay: sideDataDisplay.rightDisplay
        visible: root.browserFullscreen
        showHUD: true
        anchors.fill: parent

        onStopButtonPressed: stopButtonPressedHandler()
        z:9999999999

        onSizeClicked: {
            root.browserFullscreen = false
        }
    }

    SpeedChangePrompt{
        id:speedChangePrompt

        banner: netpulseAvailable
        isTransparent: true
        onTimedOut: hide()
        onVisibleChanged: if(visible) {
                              buzzer.warningBeep()
                              startTimeout(3)
                          }
        onTriggered: buzzer.warningBeep()
        Component.onCompleted: hide()
    }

    StrideChangePrompt{
        id:strideChangePrompt

        banner: netpulseAvailable
        isTransparent: true
        onTimedOut: hide()
        onVisibleChanged: if(visible) {
                              buzzer.buttonPress()
                              startTimeout(3)
                          }
        Component.onCompleted: hide()
    }

    Timer {
        id: safetyKeyPollTimer

        repeat: true
        running: root.workoutActive && isTreadmill
        interval: 1000
        onTriggered: {
            // end workout if safety key found to be out
            if(!machinecontrol.estop)
                stopWorkout()
        }
    }

//    onOpacityChanged: {
//        if(opacity)
//        {
//            volume_controls.vol_setting = 5
//        }
//    }

    function start() {
        print("Workout start.")
        // check safety key just before workout starts
        if(!machinecontrol.estop && isTreadmill) {
            stopWorkout()
            return
        }
        if(options.metricUnits == 0) {
            root.unitState = "ENGLISH"
            brainboard.setUnitSystem(0)
        } else {
            root.unitState = "METRIC"
            brainboard.setUnitSystem(1)
        }
        setViews()
        sideDataDisplay.reset()
        videoOverlay.reset()
        workoutStarted()

        for(var i = 0 ; i < dataToggleModel.count; i++){
            var item = dataToggleModel.get(i)
            item.thestate = "Enabled"
        }
    }

    function stopWorkout() {
        print("Workout stop.")
        fan.fanSpeed = 0
        toolbar.close()
        tv_menu.menuOpen = false
        fullscreenVideoFrame.endFullscreen()

        workoutStopped();
        heartRateCenter.hrCruiseEnabled = false
        speedKeyPadIcon.isDown = false
        inclineKeyPadIcon.isDown = false

        brainboard.reset()
    }

    function setViews()
    {
        var viewGroup = { views: [], defaultview: TheViews.trackViewDef }

        if(netpulseAvailable) {
            viewGroup.views.push(TheViews.landingViewDef)
            viewGroup.views.push(TheViews.entertainmentViewDef)
        }
        else {
            viewGroup.views.push(TheViews.ipodViewDef)
            if(tv.channelList.channels.count)
                viewGroup.views.push(TheViews.tvViewDef)
        }

        if(c360VideoAvailable) {
            viewGroup.views.push(TheViews.videoWorkoutViewDef)
            if (main.isTft) viewGroup.views.push(TheViews.toolsViewDef)
        }
        else if(c360Available) {
            viewGroup.views.push(TheViews.c360ViewDef)
            if (main.isTft) viewGroup.views.push(TheViews.toolsViewDef)
        }
        else {
            viewGroup.views.push(TheViews.trackViewDef)
            viewGroup.views.push(TheViews.profileViewDef)
            viewGroup.views.push(TheViews.simpleViewDef)
            if (main.isTft) viewGroup.views.push(TheViews.toolsViewDef)
        }

        if(netpulseAvailable) {
            viewGroup.views.push(TheViews.NPsceneViewDef)
        }
        else if (!main.isTft) {
            viewGroup.views.push(TheViews.sceneViewDef)
        }

        // only add facility view button if some slides exist
        var slides = storageMgr.listFiles("/data/facility", ["*.jpg", "*.png"])
        var userSlides = storageMgr.listFiles("/data/facility/user", ["*.jpg", "*.png"])
        if (slides.length > 0 || userSlides.length > 0)
            viewGroup.views.push(TheViews.facilitiesViewDef)

        dataExport.getNetworkAddresses()
        if(!main.isTft && options.browserEnabled && dataExport.ipAddress) {
            viewGroup.views.push(TheViews.webViewDef)
        }

        if(netpulseAvailable)
            viewGroup.views.push(TheViews.logonViewDef)

        workoutViewContainer.setViews(viewGroup)
    }

    function displayHRPrompt(index) {
        if(netpulseAvailable) {
            infoBanner.showMessage(heartRatePrompt.getMsg(index),5)
        }
        else {
            heartRatePrompt.setMsg(index);
            heartRatePrompt.show();
            heartRatePrompt.startTimeout(5);
        }
    }

    Item{
        id: dataToggleEnums

        property int timeEnum: 0
        property int timeRemainingEnum: 1
        property int paceEnum: 2
        property int paceSetterEnum: 3
        property int distanceEnum: 4
        property int distanceRemainingEnum: 5
        property int caloriesEnum: 6
        property int calsPerHourEnum: 7
        property int wattsEnum: 8
        property int metsEnum: 9
        property int heartRateEnum: 10
        property int targetHREnum: 11
        property int strideLengthEnum: 12
    }

    ListModel{
        id: dataToggleModel

        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
        ListElement{thestate: "Enabled"}
    }

    Image {
        id: leftDataBackground

        anchors.top: parent.top
        anchors.topMargin: -32
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: -226
        width: 510
        height: 260
        source: "graphics/224/224_Data_Data_Point_Window_Left.png"
    }

    Image {
        id: rightDataBackground

        anchors.top: parent.top
        anchors.topMargin: -32
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: 226
        width: 510
        height: 260
        source: "graphics/224/224_Data_Data_Point_Window_Right.png"
    }

    HeartRatePanel {
        id: heartRateCenter

        heartRate: aWorkoutRunData.heartRate
        heartRateMax: 220 - setupScreen.ageForProgSummary
        showTargetHeartRate: root.hrTargetVisibility
        targetHeartRate: workoutProgram.targetHR
        cruiseAllowed: heartRateManager.heartRateAvailable && !workoutProgram.isHRCProgram && !workoutProgram.isC360Program && workoutProgram.currentProgram != WorkoutProgram.FITNESS_TEST_PROG && (aWorkoutRunData.mode == WorkoutRunData.WORKOUT_MODE)
        anchors.horizontalCenter: parent.horizontalCenter

        onHrCruiseEnabledChanged: {
            workoutManager.cruiseControlEnabled = heartRateCenter.hrCruiseEnabled
            console.debug("HR Cruise: "+hrCruiseEnabled + " at HR:" + aWorkoutRunData.heartRate)
        }

        Connections {
            target: workoutManager
            onCruiseControlEnabledChanged: {
                heartRateCenter.hrCruiseEnabled = workoutManager.cruiseControlEnabled
            }
        }

        Connections{
            target: keymapper
            onCruiseControlPressed: {
                if (main.state === "RunState") {
                    heartRateCenter.hrCruiseEnabled = !heartRateCenter.hrCruiseEnabled
                }
            }
        }
    }

    PID {
        id: hrcPid
        kP: 1.0
        kI: 0.0
        kD: 1.0
        interval: 3000
        running: false
        actual: heartRateCenter.heartRate
        onOutChanged: {
            console.debug("machinecontrol.workload: " + machinecontrol.workload)
            machinecontrol.workload = machinecontrol.workload + out
            console.debug("Pid output: " + out + "  machinecontrol.workload: " + machinecontrol.workload)
        }
    }

    SideDataDisplays{
        id:sideDataDisplay

        visible: workoutViewContainer.currentViewName != TheViews.simpleButtonDesc.name
    }

    ViewContainer{
        id:workoutViewContainer

        opacity: root.opacity
        focus: visible && (opacity > 0)
        workoutScreen: root
        defaultView: {
            if(netpulseAvailable)
                return "LANDING_VIEW"
            else
                return tv.currentChannel >=0 ? "TV_VIEW" : workoutProgram.defaultView
        }

        Binding { target: application; property: "swipeStartX"; value: workoutViewContainer.x; when: main.state === "RunState" }
        Binding { target: application; property: "swipeEndX"; value: workoutViewContainer.x + workoutViewContainer.width; when: main.state === "RunState" }
        Binding { target: application; property: "swipeStartY"; value: workoutViewContainer.y; when: main.state === "RunState" }
        Binding { target: application; property: "swipeEndY"; value: workoutViewContainer.y + workoutViewContainer.height; when: main.state === "RunState" }
    }

    Text {
        id: userHelpText
        visible: main.isTft && root.visible
        opacity: visible ? 1.0 : 0.0
        font { family: "arial"; pixelSize: 36; bold: true }
        color: "#6c6c6c"
        text: {
            if (container.currentViewName === "PROFILE_VIEW") {
                return qsTr("press ENTER to change graph tab") + application.emptyString
            } else if (container.currentViewName === "TV_VIEW") {
                if (tvMenuShowing)
                    return qsTr("use <img src=\"graphics/tft/REW_icon.png\" /> &nbsp;<img src=\"graphics/tft/FWD_icon.png\" /> to navigate, press ENTER to select") + application.emptyString
                else
                    return qsTr("use") + " <img src=\"graphics/tft/arrow_left.png\" /> " + qsTr("or") + " <img src=\"graphics/tft/arrow_right.png\" /> " + qsTr("to change workout view") + application.emptyString
            } else {
                return qsTr("use") + " <img src=\"graphics/tft/arrow_left.png\" /> " + qsTr("or") + " <img src=\"graphics/tft/arrow_right.png\" /> " + qsTr("to change workout view") + application.emptyString
            }
        }
        anchors { top: container.bottom; topMargin: -5 }
        anchors.horizontalCenter: parent.horizontalCenter
    }

    BikeModePrompt{
        id:bikeModePrompt

        anchors.horizontalCenter: root.horizontalCenter
        anchors.verticalCenter: root.verticalCenter

        onAcceptClicked: {machinecontrol.machineMode = MachineControl.MODE_CONSTANT_TORQUE; hide();}
        onCancelClicked: {machinecontrol.machineMode = MachineControl.MODE_CONSTANT_POWER; hide();}
        onTimedOut: hide()

        Component.onCompleted: hide()
    }

    WorkoutToolbar {
        id: toolbar
        visible: main.isTouch

        onFanButtonPressed: {

            if (fan.fanSpeed === 100)
                fan.fanSpeed = 0
            else
                fan.fanSpeed = 100

        }
        showQuickKeys: machineinfo.productTypeString === "Treadmill"
        showStrideBar: main.isCSA
        onSpeedChange: {
            workoutManager.stopHRC()
            if (speed <= workoutManager.workoutProgram.maxSpeed)
                workoutManager.treadSpeed = speed * 10
        }
        currentSpeed: workoutManager.treadSpeed
        currentIncline: workoutManager.treadIncline
        onInclineChange: {
            if(incline <= workoutManager.workoutProgram.maxIncline){
                workoutManager.stopHRC()
                workoutManager.treadIncline = incline*10
            }
       }
        cooldownEnabled: workoutProgram.cooldownEnabled
        anchors { left: parent.left ; leftMargin: 20 }
        anchors { bottom: parent.bottom ; bottomMargin: 10 }

        onBikeModeButtonPressed: {
            if(!bikeModeEnabled){
                root.showBikeModePrompt()
            }
        }
        workoutMode: aWorkoutRunData.mode
        onCooldownPressed: workoutManager.startCooldown()
    }

    Image{
        id: bikeModeMPHGauge
        visible: bikeModeEnabled
        source: "graphics/224/SPEED_METER_bikemode.png"
        anchors.bottom: main.isTft ? stopTftArea.top : stopButton.top
        anchors.bottomMargin: 10
        anchors.horizontalCenter: stopButton.horizontalCenter
        height: implicitHeight
        width: implicitWidth

        Text{
            id:value
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: -5
            anchors.horizontalCenter: parent.horizontalCenter
            text: {
                if (root.unitState === "ENGLISH")
                    aWorkoutRunData.bikeSpeedMph.toFixed(1)
                else
                    Globals.mileToKm(aWorkoutRunData.bikeSpeedMph).toFixed(1)
            }
            font.pointSize: 36
            font.bold: true
        }

        Text{
            id: label
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
            text: root.unitState === "ENGLISH" ? "MPH" : "KPH"
            font.pointSize: 14
            font.bold: true
        }
        z: sideDataDisplay.z - 1
    }

    ImageButton {
        id: stopButton
        visible: main.isTouch

        anchors.bottom: parent.bottom
        anchors.bottomMargin: 15
        anchors.right: parent.right
        anchors.rightMargin: 20
        fontsize: 14
        label_color: "black"
        label_under: true
        downimage: "graphics/224/224_Data_Stop_Down.png"
        upimage: "graphics/224/224_Data_Stop_Disabled.png"
        label: qsTr("STOP") + "/" + qsTr("PAUSE") + application.emptyString
        label_y_offset: 10

        onClicked: stopButtonPressedHandler()
        z: workoutViewContainer.z + 1
        mouseAreaScale: 1.15
    }

    Item {
        id: stopTftArea
        width: 125
        height: childrenRect.height
        visible: main.isTft && !isTreadmill
        anchors { bottom: parent.bottom; bottomMargin: 15; right: parent.right; rightMargin: 20 }

        Text {
            id: backText
            text: qsTr("press BACK for") + application.emptyString
            width: 100
            font { pixelSize: 16; bold: true }
            wrapMode: Text.WordWrap
            horizontalAlignment: Text.AlignHCenter
            anchors { top: parent.top; horizontalCenter: parent.horizontalCenter }
        }
        Text {
            id: stopPauseText
            text: qsTr("STOP") + "/" + qsTr("PAUSE") + application.emptyString
            width: 100
            font { pixelSize: 16; bold: true }
            wrapMode: Text.WordWrap
            horizontalAlignment: Text.AlignHCenter
            anchors { top: backText.bottom; topMargin: 5; horizontalCenter: parent.horizontalCenter }
        }
    }

    Connections
    {
        target: popupManager
        onEstopPopupPopped: if(workoutActive){stopWorkout()}
    }

    Connections {
        target: errorManager
        onPreventUseChanged: {
            if(workoutActive && errorManager.preventUse)
                stopWorkout()
        }
    }

    TftWorkoutEndPrompt {
        id: tftStopPrompt

        rpmPause: rpmPauseTimer.wasTriggered
        x: 300; y: 250
        visible: false
        onVisibleChanged: {
            if (!visible)
                rpmPauseTimer.wasTriggered = false
        }
        onTimedOut: {
            hide()
            stopWorkout()
        }
    }

    Connections {
        target: main.isTft && main.state === "RunState" && tftStopPrompt.visible ? keymapper : null

        onBackPressed: {
            main.debugKeys("backPressed", "WorkoutScreen")
            tftStopPrompt.hide()
            stopWorkout()
        }

        onStopPausePressed: {
            main.debugKeys("stopPausePressed", "WorkoutScreen")
            tftStopPrompt.hide()
            stopWorkout()
        }

        onStopPressed: {
            main.debugKeys("stopPressed", "WorkoutScreen")
            tftStopPrompt.hide()
            stopWorkout()
        }
    }

//    Connections
//    {
//        target: personPresentDetectionManager
//        onDisplayStopPrompt: if(workoutActive){stopPrompt.show(); stopPrompt.startTimeout(30)}
//    }

    WorkoutEndPrompt {
        id: _stopPrompt

        x: 300 ; y: 250
        visible: false
        onTimedOut: {
            stopWorkout()
            hide()
        }
        onStopClicked: {
            stopWorkout()
            hide()
        }
        onResumeClicked: {

            hide()
            root.resumeWorkout()
        }
    }

    Timer {
        id: rpmPauseTimer
        property bool wasTriggered: false

        interval: 10000
        running: !options.disableNoPedalStop && workoutActive && !isTreadmill && !stopPrompt.visible && machinecontrol.pedalRPM == 0
        onTriggered: {
            if(!workoutActive)
                return

            rpmPauseTimer.wasTriggered = true
            if(options.resumeWorkoutTimeout){
                print("No pedal. Prompt end of workout.")
                stopPrompt.show()
                stopPrompt.startTimeout(options.resumeWorkoutTimeout)
                workoutPaused()
            }
            else{
                stopPrompt.show()
                workoutPaused()
            }

            reset()
        }

    }

    Connections {
        target: brainboard
        onIPodConnected: {
            if(!netpulseAvailable && connected)
                workoutViewContainer.setDefaultView("IPOD_VIEW")
            // TODO: Need to switch EntertainmentViewContainer to iPod view in Netpulse mode
        }
    }

    Connections {
        target: machinecontrol
        onPedalRPMChanged: {
            if(workoutActive && (pedalRPM > 0) && !options.disableNoPedalStop) {
                rpmPauseTimer.restart()
                if (stopPrompt.visible && isTft && rpmPauseTimer.wasTriggered) {
                    stopPrompt.hide()
                    rpmPauseTimer.wasTriggered = false
                    root.resumeWorkout()
                }
            }
        }
        onStrideLengthTargetChanged: {
            if (!main.isCSA)
                return

            if(!workoutActive)
                return

            strideChangePrompt.oldStride = strideChangePrompt.newStride
            strideChangePrompt.newStride = strideLengthTarget
            strideChangePrompt.show()
            strideChangePrompt.startTimeout(3)
        }
    }

    HeartRatePrompts {
        id: heartRatePrompt

        x: 260
        y: 220
        visible: false
        onTimedOut: {
            hide();
        }
    }

    PaceSetter {
        id: paceSetter

        property bool isShowing: currentViewName !== TheViews.simpleButtonDesc.name

        rpm: aWorkoutRunData.rpm
        paceTarget: workoutManager.paceTarget
        paceLevel: workoutManager.paceLevel
        onLevelUp: workoutManager.increasePaceLevel()
        onLevelDown: workoutManager.decreasePaceLevel()
        showPaceSetter: root.showPaceSetter

        anchors.top: parent.top
        anchors.topMargin: -20
        anchors.right: parent.right
        anchors.rightMargin: 189

        bikeModeEnabled: root.bikeModeEnabled
        visible: isShowing && !isTreadmill
    }

    Item {
        id: levelControls

        property bool isShowing: currentViewName !== TheViews.simpleButtonDesc.name

        anchors.top: parent.top
        anchors.topMargin: 10
        anchors.left: parent.left
        anchors.leftMargin: 175
        width: 260
        height: 150
        visible: isShowing && !isTreadmill

        z: workoutViewContainer.z + 1

        function getIcon()
        {
            if( isTreadmill )
                return "graphics/netpulse/WORKLOAD-INCLINE_icon_GRAY.png"
            else if(root.bikeModeEnabled)
                return "graphics/224/GEAR-LEVEL_icon_bikemode.png"
            else
                return "graphics/224/WORKLOADlevel_icon_GRAY.png"
        }

        function getLabel()
        {
            if(root.bikeModeEnabled)
                return qsTr("GEAR")
            else
                return qsTr("LEVEL")
        }

        ImageButton {
            id: levelUpButton
            visible: main.isTouch
            repeat: true
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: -45
            downimage: "graphics/netpulse/UpArrow_PRESSED.png"
            upimage: "graphics/netpulse/UpArrow_blue.png"
            icon: "graphics/netpulse/PLUS_icon_BLACK.png"
            onClicked: keymapper.keyFunctionHandler(7) //workoutManager.increaseWorkloadLevel()
            mouseAreaScale: 1.25
        }

        Text {
            id: levelValue
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.horizontalCenterOffset: main.isTft ? 15 : 0
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: main.isTft ? -20 : 0
            color: "black"
            font.pointSize: main.isTft ? 32 : 22
            font.bold: true
            text: workoutManager.workloadLevel
        }

        Text {
            id: levelLabel
            anchors.verticalCenter: main.isTouch ? levelValue.verticalCenter : undefined
            anchors.top: main.isTft ? levelValue.bottom : undefined
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.horizontalCenterOffset: main.isTouch ? 65 : 15
            color: "black"
            font.pointSize: 12
            font.bold: true
            text: parent.getLabel() + application.emptyString
        }

        ImageButton {
            id: levelDownButton
            visible: main.isTouch

            onClicked: keymapper.keyFunctionHandler(8) //workoutManager.decreaseWorkloadLevel()
            repeat: true
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: 45
            downimage: "graphics/netpulse/DownArrow_PRESSED.png"
            upimage: "graphics/netpulse/DownArrow_blue.png"
            icon: "graphics/netpulse/MINUS_icon_BLACK.png"
            icon_y_offset: -5
            mouseAreaScale: 1.25
        }

        Image {
            id: workloadIcon

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.horizontalCenterOffset: 90
            source: parent.getIcon()
        }
    } // end levelControls

    Item {
        id: inclineControls

        property bool isShowing: currentViewName !== TheViews.simpleButtonDesc.name

        anchors.top: parent.top
        anchors.topMargin: 10
        anchors.left: parent.left
        anchors.leftMargin: 185
        width: 250
        height: 150
        visible: isShowing && isTreadmill

        z: workoutViewContainer.z + 1

        Text {
            id: inclineValue
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.horizontalCenterOffset: 10
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: 0
            color: "black"
            font.pointSize: 32
            font.bold: true
            text: (workoutManager.treadIncline / 10.0).toFixed(1)
        }

        Text {
            id: inclineLabel
            anchors.verticalCenter: inclineValue.verticalCenter
            anchors.horizontalCenter: inclineValue.horizontalCenter
            anchors.verticalCenterOffset: -30
            color: "black"
            font.pointSize: 12
            font.bold: true
            text: qsTr("INCLINE") + application.emptyString
        }

        Text {
            id: inclineUnits
            anchors.verticalCenter: inclineValue.verticalCenter
            anchors.horizontalCenter: inclineValue.horizontalCenter
            anchors.verticalCenterOffset: 30
            color: "black"
            font.pointSize: 12
            font.bold: true
            text: qsTr("%") + application.emptyString
        }

        Image {
            id: inclineIcon

            anchors.top: parent.top
            anchors.topMargin: 0
            anchors.right: parent.right
            anchors.rightMargin: -5
            source: "graphics/netpulse/WORKLOAD-INCLINE_icon_GRAY.png"
        }


        Connections{
            target:root
            onWorkoutStarted:inclineKeyPadIcon.isDown = false
        }
    }

    ImageButton {
        id: inclineKeyPadIcon
        anchors.bottom: inclineControls.bottom
        anchors.horizontalCenter: inclineControls.horizontalCenter
        anchors.horizontalCenterOffset: -20
        upimage: "graphics/224/KEYPAD_icon_tread.png"
        downimage: "graphics/224/KEYPAD_icon_tread.png"
        toggle: true
        visible: main.isTouch && isTreadmill && !netpulseAvailable
        mouseAreaScale: 3.0
    }

    Item {
        id: speedControls

        property bool isShowing: currentViewName !== TheViews.simpleButtonDesc.name

        anchors.top: parent.top
        anchors.topMargin: 10
        anchors.right: parent.right
        anchors.rightMargin: 185
        width: 250
        height: 150
        visible: isShowing && isTreadmill

        z: workoutViewContainer.z + 1

        Text {
            id: speedValue
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            color: "black"
            font.pointSize: 32
            font.bold: true
            text: {
                var tempSpeed = workoutManager.treadSpeed / 10;
                if (root.unitState === "ENGLISH")
                    return tempSpeed.toFixed(1)
                else
                    return Globals.mileToKm(tempSpeed).toFixed(1)
//                var tempSpeed;
//                tempSpeed = ((workoutManager.speedLevel + 4.0) / 10.0);
//                return tempSpeed.toFixed(1);
            }
        }

        Text {
            id: speeedLabel
            anchors.verticalCenter: speedValue.verticalCenter
            anchors.horizontalCenter: speedValue.horizontalCenter
            anchors.verticalCenterOffset: -30
            color: "black"
            font.pointSize: 12
            font.bold: true
            text: qsTr("SPEED") + application.emptyString
        }

        Text {
            id: speedUnits
            anchors.verticalCenter: speedValue.verticalCenter
            anchors.horizontalCenter: speedValue.horizontalCenter
            anchors.verticalCenterOffset: 30
            color: "black"
            font.pointSize: 12
            font.bold: true
            text: root.unitState === "ENGLISH" ? "MPH" : "KPH"
        }

        Image {
            id: speedIcon

            anchors.top: parent.top
            anchors.topMargin: -3
            anchors.left: parent.left
            anchors.leftMargin: 5
            source: "graphics/netpulse/PACE-SPEED_icon_GRAY.png"
        }

        Connections{
            target:root
            onWorkoutStarted:speedKeyPadIcon.isDown = false
        }
    }

    ImageButton {
        id: speedKeyPadIcon
        anchors.bottom: speedControls.bottom
        anchors.horizontalCenter: speedControls.horizontalCenter
        anchors.horizontalCenterOffset: 20
        upimage: "graphics/224/KEYPAD_icon_tread.png"
        downimage: "graphics/224/KEYPAD_icon_tread.png"
        toggle: true
        visible: main.isTouch && isTreadmill && !netpulseAvailable
        mouseAreaScale: 3.0
    }
    function getInclineLevelFromTable(incline){

        var table = workoutManager.workoutProgram.inclineTable
        var i
        incline *= 10
        for(i = 0; i < table.length; i++)
        {
            if(table[i] === incline) return i;
            if(incline > table[i] && incline < table[i+1] && i < table.length){
                var interval = table[i+1] - table[i]
                if((incline - table[i]) < (interval / 2)) return i;
                else return i+1
            }
        }
        return -1
    }

    Connections {
        target: brainboard
        onStrideNotAvailable: {
            showMessage(qsTr("Stride Fault"))
        }
    }

    Keypad{
        id:inclineKeyPad
        units: "%"
        length: 3
        decimalPlaces: 1
        clear_after_enter: false
        icon: "../graphics/netpulse/WORKLOAD-INCLINE_icon_GRAY.png"
        state: inclineKeyPadIcon.isDown ? "showing" : "notshowing"
        anchors.left: parent.left
        anchors.leftMargin: -11
        anchors.verticalCenter: parent.verticalCenter
        anchors.verticalCenterOffset: -35
        z:inclineControls.z
        onClose: inclineKeyPadIcon.isDown = false

        Connections {
            target: workoutManager
            onTreadInclineChanged: inclineKeyPad.setValue(workoutManager.treadIncline)
        }

        onValueEntered: {
            if(val <= workoutManager.workoutProgram.maxIncline){
                var incline = workoutProgram.tableClosestValue(workoutProgram.inclineTable, val*10)
                print("Incline keypad entry:" + incline)
                workoutManager.stopHRC()
                workoutManager.treadIncline = incline
            }
            else
                inclineKeyPad.setValue(workoutManager.treadIncline)

        }

        onStateChanged: {
            if(state === "showing"){
                inclineKeyPad.setValue(workoutManager.treadIncline)
            }
        }

        states: [
            State{name: "showing"
                PropertyChanges { target: inclineKeyPad; anchors.leftMargin: -11}
            },
            State{name: "notshowing"
                PropertyChanges { target: inclineKeyPad; anchors.leftMargin: -350}
            }
        ]
        transitions: [
            Transition {
                from: "notshowing"
                to: "showing"
                reversible: true
                SequentialAnimation {
                    NumberAnimation { duration: 500; properties: "anchors.leftMargin" ;easing.type:Easing.OutBack }
                }
            }
        ]
    }

    BluetoothDeviceSelectionDialog{
        id: btHRSelectionDialog
        isTop: true
        isTransparent: false
        height: root.height * 0.42
        width: root.width * 0.60
        anchors.horizontalCenter: root.horizontalCenter
        anchors.verticalCenter: root.verticalCenter
        isLowEnergy: false

        onClosed: btHRSelectionDialog.hide()
        //onHeadsetConnected: btDeviceSelectionDialog.hide()
        //onHeadsetDisconnected: btDeviceSelectionDialog.hide()

    }

    Keypad{
        id:speedKeyPad
        units: root.unitState === "ENGLISH" ? "MPH" : "KPH"
        length: 3
        decimalPlaces: 1
        clear_after_enter: false
        icon: "../graphics/netpulse/PACE-SPEED_icon_GRAY.png"
        state: speedKeyPadIcon.isDown ? "showing" : "notshowing"
        anchors.right: parent.right
        anchors.rightMargin: -11
        anchors.verticalCenter: parent.verticalCenter
        anchors.verticalCenterOffset: -35
        z:speedControls.z
        onClose: speedKeyPadIcon.isDown = false

        property int convertedTreadSpeed: root.unitState === "METRIC" ? Globals.mileToKm(workoutManager.treadSpeed) : workoutManager.treadSpeed

        onConvertedTreadSpeedChanged: speedKeyPad.setValue(speedKeyPad.convertedTreadSpeed)

        Connections {
            target: workoutManager
            onTreadSpeedChanged: speedKeyPad.setValue(speedKeyPad.convertedTreadSpeed)
        }

        onValueEntered: {
            var myVal = val
            if (root.unitState === "METRIC")
                myVal = Globals.kmToMile(parseFloat(val))
            if((myVal >= 0.5) && myVal <= workoutManager.workoutProgram.maxSpeed) {
                workoutManager.stopHRC()
                print("Speed keypad entry:" + myVal)
                workoutManager.treadSpeed = myVal*10
            }
            else if(myVal < 0.5){
                workoutManager.stopHRC()
                workoutManager.treadSpeed = 5
                speedKeyPad.setValue(convertedTreadSpeed)
            }
            else
                speedKeyPad.setValue(convertedTreadSpeed)
        }

        onStateChanged: {
            if(state === "showing"){
                speedKeyPad.setValue(convertedTreadSpeed)
            }
        }

        states: [
            State{name: "showing"
                PropertyChanges { target: speedKeyPad; anchors.rightMargin: -11}
            },
            State{name: "notshowing"
                PropertyChanges { target: speedKeyPad; anchors.rightMargin: -350}
            }
        ]
        transitions: [
            Transition {
                from: "notshowing"
                to: "showing"
                reversible: true
                SequentialAnimation {
                    NumberAnimation { duration: 500; properties: "anchors.rightMargin" ;easing.type:Easing.OutBack }
                }
            }
        ]
    }


}
