/** @file main.qml
    @brief Manage main display and transitions to workout menus and run screens.
*/

import QtQuick 2.4
import WorkoutManager 1.0
import StorageMgr 1.0
import Brainboard 1.0
import BootLoader 1.0
import ConfigurationData 1.0
import StatisticManager 1.0
import ErrorManager 1.0
import IpodManager 1.0
import WorkoutRunData 1.0
import WorkoutSimulator 0.1
import Process 0.1
import AMixer 1.0
import Netpulse 0.1
import Bluetooth 1.0
import HeartRateManager 1.0
import RecoveryManager 1.0
import BroadcastVisionManager 1.0
import SapManager 1.0
import Components 1.0
import "LanguageMenu"
import "Diagnostics"
import "MediaBrowser"
import "tft_workout_menus.js" as TftMenus
import "workout_menus.js" as TouchMenus
import "workout_menus.js" as Menus
import "string_format.js" as StrFormat
import "Television"
import "Keypad"

import VLCVideo 1.0

Item {
    id: main

    property bool netpulseAvailable: options.netpulseEnabled && netpulse.connected
    property variant topmenu
    property variant allWorkoutsMenu

    property bool _dialogShowing: false
    property bool blockKeys: restartRequired || dialogManager.anyAreBlockingKeys || popupManager.estopShown || logoImage.blinking || main.quickStartLoading

    property bool showStimulusArea: options.showStimulusArea

    property int volume: 5
    property real timeFactor: 1.0
    property string serialNumber: application.serialNumber
    property string productVersion: application.productVersion
    property bool isTreadmill: machineinfo.productTypeString == "Treadmill"
    property bool isTft: application.isTft
    property bool isTouch: !application.isTft
    property bool rightToLeft: Qt.application.layoutDirection === Qt.RightToLeft
    property string channelSelected: ""
    property bool restartRequired: false
    property bool firmwareUpdateRequired: false
    property bool _hideLanguageButton: false
    property int screenShotCount: 0
    property string consoleType: application.consoleType
    property bool _debugKeys: true
    property bool erpIsSupported: true
    property bool quickStartLoading: false
    property bool isCSA: machineinfo.modelString === "TS1000"

    property variant home_screen: home_screen_loader.item
    property variant workout_menu: workout_menu_loader.item

    width: 1366
    height: 768
    focus: state == "HomeState" || state == "LoginState" || state == "RegisterState"
    signal reset()

    Component.onCompleted: {
        QT_TRANSLATE_NOOP("QApplication", "QT_LAYOUT_DIRECTION")
        // start on the home screen
//        options.loadOptions()
        print("***** Application Start v" + application.appVersion + " *****") ;
        state = "HomeState"
       //state= "RunState"
        var lang = options.defaultLanguage
        console.log("language default: " +lang)
        application.selectLanguage(lang)
        selectWorkoutMenu()
        messagecenter.setBrightness(options.messageCenterBrightness)
        messagecenter.setContrast(options.messageCenterContrast)
    }

    Component.onDestruction: {
        console.log("in main.qml Component.onDestruction")
        urlKeypad.process.terminate()
        inputKeypad.process.terminate()
        if (bluetoothAdapter.connected)
            bluetoothAdapter.disconnect()
        if (bluetoothAdapter.connectedHeadset)
            bluetoothAdapter.disconnectHeadset()
    }

    Binding { target: application; property: "swipeStartX"; value: 0; when: main.state != "RunState" }
    Binding { target: application; property: "swipeEndX"; value: main.width; when: main.state != "RunState" }
    Binding { target: application; property: "swipeStartY"; value: 0; when: main.state != "RunState" }
    Binding { target: application; property: "swipeEndY"; value: main.height; when: main.state != "RunState" }

    NetworkConnections {
        id: netConnections
        onIsRefreshingChanged: {
            if (!isRefreshing) {
                console.log("Finished refreshing connections:")
                netConnections.printStatus()
            }
        }
    }

    Connections {
        target: nucleus
        onNucleusConnectionChanged: {
            if(connection == true) {
                brainboard.nucleusIsConnected = true
                nucleus.startInitializationProcess()
            }
        }
    }

    Connections{
        target: tv
        onTunerTypeChanged: {
            switch(tv.tunerType){
            case "ATSC":
                erpIsSupported = true
                break;
            default:
                erpIsSupported = true
                break;
            }
        }
    }

    function debugKeys(keyname, objectname, notes) {
        if (main._debugKeys)
            console.log("KEYPRESS: ",  keyname, objectname, notes)
    }

    DialogManager {
        id: dialogManager
    }

    Process { id: syncDisk
        program: "/bin/sync"
        block: true
        blockTimeout: 1000 * 60 * 60; // 1 hour
    }

    MessagePopup {
        id: blockUseMessage
        property bool showPopup: main.state !== "DiagnosticState" && main.state !== "SetupWizardState" && (restartRequired || home_screen.buttonsEnabled === false)
        onShowPopupChanged: {
            if (showPopup)
                show()
            else
                hide()
        }
        message: {
            if (restartRequired) {
                return qsTr("Power Cycle Required to Complete Update") + application.emptyString
            }else if (!machineinfo.isReady)
                return qsTr("Starting up...") + application.emptyString
            else if (!home_screen.buttonsEnabled) {
                if (machineinfo.productTypeString === "" && options.ignoreFaults == false) {
                    return qsTr("Console Configuration Required.") + application.emptyString
                } else if (machineinfo.productTypeString === "Treadmill" && machinecontrol.isCalibrated == false && options.ignoreFaults == false){
                    return qsTr("Treadmill Calibration Required.") + application.emptyString
                } else if (machineinfo.productTypeString === "" && options.ignoreFaults == true){
                    return qsTr("Product Configuration Required.") + application.emptyString
                } else {
                    return qsTr("Complete Configuration to Enable Buttons") + application.emptyString
                }
            } else {
                return qsTr("Complete Configuration to Enable Buttons") + application.emptyString
            }
        }
        Component.onCompleted: if (showPopup) show(); else hide()
        isTransparent: true
    }

    MiscStats {
        id: miscStats
    }

    WorkoutStats {
        id: workoutStats
    }

    UserData { id: userData
        loggedIn: false
    }

    DataExport {
        id: dataExport
        workoutProgram: workoutProgram
        workoutData: aWorkoutRunData
        userData: userData
        profile1: workoutProgram.profile1
        profile2: workoutProgram.profile2
        productModel: machineinfo.modelString
        machineID: netpulse.machineID
        serialNumber: main.serialNumber
        runMiles: statisticManager.totalDistance
        runHours: statisticManager.runHours / 3600  // stats manager keeps "runHours" in seconds
        modelID: configurationData.csafeID
        version: application.appVersion
        Component.onCompleted: print("ip address:" + ipAddress)
    }

    Netpulse { id: netpulse
        userLogin: userData
        frozen: main._dialogShowing
    }

    NetpulseLogin {
        id: loginWindow

        onTimeout: {
            if(main.state === "LoginState" || main.state == "RegisterState") {
                netpulse.hideAll()
                main.state="HomeState";
            }
        }
        onLoggedIn: {
            if(main.state === "LoginState" || main.state == "RegisterState") {
                main.state="HomeState";
            }
        }
        anchors.fill: parent
    }

    Connections{
        target: tv
        onTunerTypeChanged: {
            switch(tv.tunerType){
            case "ATSC":
                erpIsSupported = true
                break;
            default:
                erpIsSupported = true
                break;
            }
        }
    }

    NetpulseRegistration {
        id: registerWindow

        onTimeout: {
            netpulse.hideAll()
            main.state="HomeState";
        }
        onLoggedIn: {
            if(main.state === "LoginState" || main.state == "RegisterState") {
                main.state="HomeState";
            }
        }
        anchors.fill: parent
    }

    NetpulseInstall {
        id: netpulseInstall

        anchors.fill: parent
    }

/*
    NetpulseSummary {
        id: netpulseSummary

        anchors.fill: parent
    }
*/
    NetpulseWatch {
        id: netpulseWatch

        leftSideDisplay: workoutScreen.sideDataDisplay.leftDisplay
        rightSideDisplay: workoutScreen.sideDataDisplay.rightDisplay

        onStopButtonPressed: workoutScreen.stopButtonPressedHandler()

        z: 99
    }

    // expire Netpulse Login after 30 seconds with no activity on homescreen
    Timer {
        id: netpulseLoginTimer

        running: (main.state == "HomeState") && netpulse.loggedIn
        interval: 30 * 1000
        onTriggered: netpulse.logOut()
        function reset() { if(running) restart() }
    }

    Connections {
        // reset the login timer when there's a screen touch
        target: application
        onScreenTouched: netpulseLoginTimer.reset()
    }

    AMixer {
        id: amixer
    }

    Text{
        id:versionNumber
        font.pointSize: 10
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: demoModeText.visible ? -demoModeText.width/2 : 0
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3
        color: "black"
        opacity: 0.5
        visible: !fullscreenVideoFrame.visible
        text:qsTr("Version: ") + application.appVersion + application.emptyString

        MouseArea
        {
            anchors.fill: parent
        }
    }

    Text {
        id: demoModeText
        visible: options.ignoreFaults
        font.pointSize: 10
        font.bold: true
        anchors.left: versionNumber.right
        anchors.leftMargin: 3
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3
        color: "black"
        opacity: 0.5
        text: qsTr("Demo Mode") + application.emptyString
    }

    ConfigurationData{
        id:configurationData

        onSendConfigurationDataSignal: brainboard.sendConfigurationData(name,productType,configurationData)
        maxInclinePercentGrade: 150
        minInclinePercentGrade: -30
        maxSpeed: 150
    }

    StatisticManager{
        id:statisticManager

        onTotalDistanceChanged: errorManager.updateMileage(totalDistance)
    }

    MessagePopup {
        id: infoPopup
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    MessageBanner {
        id: infoBanner

        z: 10000
        anchors.horizontalCenter: main.horizontalCenter
        y:0
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    SimpleMessageBanner {
        id: viewContainerBanner

        z: 10000
        hideWhen: !main.isCSA && workoutScreen.container.currentViewName === "C360_VIEW"
        x:185
        y:170
        width:1000
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    PopupManager{
        id: popupManager

        safetyKeyOut: !machinecontrol.estop && isTreadmill && !brainboard.estopOverride
        // hide pop-ups while in diagnostic screen and other situations
        hidePopup: main.state === "DiagnosticState" || main.state === "SetupWizardState"  || main.restartRequired || options.ignoreFaults || main.firmwareUpdateRequired
        treadStopped: machinecontrol.treadmillSpeedActual < 1

        onEstopPopupTimedout: console.debug("estop timed out")
    }

    ErrorManager{
        id:errorManager

        property bool fatalError: false
        property bool preventUse: false
        property bool degradedUse: false

        onErrorEvent: {
            if(options.ignoreFaults) {
                console.log("Ignoring error event: " + errorcode + " " + errormessage)
                return
            }

            if(isLogged && netpulseAvailable)
                dataExport.serviceEvent(errorcode,"");

            console.log("Error event: " + errorcode + " " + errormessage)
            logger.dumpLastLog()
            if(errorcode === "CN25")
                main.firmwareUpdateRequired = true

            if(severity == ErrorManager.ERROR_FATAL)
                fatalError = true
            if(severity >= ErrorManager.ERROR_PREVENTS_USE)
                preventUse = true
            else if(severity >= ErrorManager.ERROR_DEGRADES_USE)
                degradedUse = true

            if(!options.ignoreFaults) {
                var translatedErrorMessage = qsTranslate("ErrorManager",errormessage)
                popupManager.setErrorMessage(errorcode, translatedErrorMessage, severity)

                // briefly display new error at top of screen if pop-ups are hidden (as in diagnostic screens.)
                if(popupManager.hidePopup)
                    infoBanner.showMessage(popupManager.currentMessage, 5);

                if (errorcode === "X101" || errorcode === "X102") {
                    console.log("Stride Fault")
                    machinecontrol.strideFaulted = true
                }

            }
        }
    }

    WarningText {
        id: warningText
        enabled: (visible && opacity > 0.0)
        visible: main.isTft && main.isTreadmill && (!ipod_window.visible && !tv_window.visible)
        anchors { right: parent.right; rightMargin: 30; top: parent.top; topMargin: 105 }
    }

    StorageMgr{
        id:storageMgr
    }

    Loader {
        id: firmwarePopupLoader

        source: (main.state != "DiagnosticState" && main.firmwareUpdateRequired && !options.ignoreFaults) ? "FirmwarePopup.qml" : ""
    }

    IpodManager{
        id:ipodmanager

        property int category: brainboard.iPodMenuCategory
        property string playStatus: brainboard.iPodPlayingStatus
        property string browserState: "not_connected"

        onCategoryChanged: {console.debug("cat changed")}
        onReturnToTopMenu: {brainboard.iPodReturnToTopMenu()}
        onSelectDatabaseRecord: {console.debug("select item");brainboard.iPodSelectDatabaseRecord(category,index);}
        onSelectDatabase: brainboard.iPodSelectDatabase(category)
        onSelectDatabaseRecordBackward: brainboard.iPodSelectDatabaseRecordBackward(previousCategory, currentCategory, index)
        onPlayStatusChanged: console.debug("IPOD is "+playStatus)
        onNikePlusRecordChanged: {
            if(ipodmanager.nikePlusRecord && main.state == "RunState"){
                brainboard.sendGuiWorkoutEvent(0);
                var message = qsTr("RECORDING WORKOUT TO APPLE DEVICE")
                if(isTreadmill)
                    workoutScreen.showBannerMessage(message)
                else
                    workoutScreen.showMessage(message)
            }
        }
        onIPodConnectedChanged: {
            if(ipodmanager.iPodConnected){
                miscStats.incrementiPodUsage()
            }
        }
        onNextTrackSignal: brainboard.iPodNextTrack()
        onPreviousTrackSignal: brainboard.iPodPreviousTrack()

        //nikePlusSupport: br
    }

    Connections{
        target: brainboard
        onIPodMenuChanged: {ipodmanager.clearMenu(); ipodmanager.menuHeading = menuHeading;}
        onIPodNikePlusSupport:{
            if(nikePlusSupport){
                ipodmanager.nikePlusSupport = true
                if(main.state == "HomeState")
                    ipodmanager.nikePlusRecord = true
                else
                    ipodmanager.nikePlusRecord = false
                console.debug("nike + supported")
            }
            else{
                ipodmanager.nikePlusSupport = false
                ipodmanager.nikePlusRecord = false
                popupManager.setInfoMessage(qsTr("This Apple device does not support Nike + iPod workout recording."))
            }
        }
        onIPodConnected: {
            if(connected){
                ipodmanager.browserState = "notplaying_menu"
            }
            else {
                ipodmanager.browserState = "not_connected"
                //audioManager.musicSelected = false
            }
        }
    }

    Loader {
        id: screensaver
        z:1000
        anchors.fill: parent
        visible: false

        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom ; anchors.bottomMargin: 20
            font.pointSize: 40

            color: "white"
            z:99
            text: (main.isTft ? qsTr("Press Any Button to Begin") : qsTr("Touch to Begin")) + application.emptyString
        }

        function start() {
            visible=true
            screensaver.source = "Slideshow.qml"
            screensaver.item.zoomEffect = true
            screensaver.item.fadeEffect = true
            screensaver.item.slideInterval = 10000
        }
        function stop() {
            visible=false
            screensaver.source = ""
            if(machineinfo.modelString != "----")
                main.state = "HomeState"
            else
                main.state = "SetupWizardState"
        }

        Connections {
            target: brainboard
            // wake from screensaver when any button pressed
            onButtonPressed: {
                if(main.state === "ScreenSaverState" || main.state === "SleepModeState")
                    screensaver.stop()
                else if (main.state === "HomeState") {
                    main.lcdBrightness = options.lcdBrightness
                    if (!options.screenSaveTimeout && options.dimDisplayTimeout)
                        dim_timer.restart()
                }
            }
        }

        Connections {
            target: machinecontrol
            // wake up out of screen saver when pedaled.
            onPedalRPMChanged: {
                if(pedalRPM > 0) {
                    if(main.state === "ScreenSaverState" || main.state === "SleepModeState")
                        screensaver.stop()
                    else if (main.state === "HomeState") {
                        main.lcdBrightness = options.lcdBrightness
                        if (!options.screenSaveTimeout && options.dimDisplayTimeout)
                            dim_timer.restart()
                    }
                }

                if(workoutScreen.workoutActive && (machineinfo.productType != WorkoutProgram.TREADMILL)) {
                    // update the pace profile with current pace during a workout.
                    aWorkoutRunData.setRPM(pedalRPM)
                    workoutProgram.profile2.setProperty(workoutProgram.profile2.currentSegment, "segmentMagnitude", pedalRPM)
                }
            }

            onDisplayWattsChanged: { aWorkoutRunData.setWatts(displayWatts) ; workoutManager.setWatts(displayWatts) }
            onStrideLengthTargetChanged: {
                workoutManager.strideLevel = strideLengthTarget
            }
        }

        MouseArea {
            anchors.fill: parent
            onClicked: screensaver.stop()
        }
    }

    function quickStart(filename) {
        if (main.state === "SleepModeState")
            return

        if (blockUseMessage.visible) return
        if (main.quickStartLoading) // prevent multiple consecutive loads
            return
        main.quickStartLoading = true

        print("Quickstart " + (filename ? filename : ""))
        logoImage.cancelBlink()
        main.channelSelected = ""
        if( main.state === "HomeState" || main.state == "ScreenSaverState" || main.state === "MenuState" || main.state === "AllWorkoutsState") {
            if(main.state === "ScreenSaverState")
                screensaver.stop()
            tv_window.hide()
            ipod_window.close()

            if(isTreadmill) {
                if(!filename)
                    filename ="Tread_Quick_Start.xml";
            }
            else {
                if(!filename)
                    filename = "Quick_Start.xml"
            }
            workoutStats.incrementUsage(qsTr("Quick Start"))
            workoutProgram.loadWorkoutFileAsync("Program_Files/" + filename);
//            conditionalStart()
            aWorkoutRunData.programName = qsTr("QUICK START")
        }
    }

    function startWorkout(item){
        logoImage.cancelBlink()
        main.channelSelected = ""
        main.clearQLists();
        var programFile = item.program

        workoutStats.incrementUsage(item.label)

        if(item.program === "SAVED_WORKOUTS") {
            workoutProgram.reset()
            setupScreen.showSavedWorkouts()
            finishLoading(item)
        }
        else {
            state = "LoadState";
            loadingProgramNotice.programItem = item
            workoutProgram.loadWorkoutFileAsync("Program_Files/" + programFile)
        }

    }

    Connections {
        target: workoutProgram
        onParsingFileChanged: {
            // saved programs are handled by the saved programs browser, so skip them
            if (!workoutProgram.parsingFile && workoutProgram.customProgType === WorkoutProgram.UNDEFINED_PROGRAM_TYPE) {
                finishLoading(loadingProgramNotice.programItem)
            }
        }
    }

    function finishLoading(item) {
        if(!workoutProgram.isQuickStart){
            if (item.program !== "SAVED_WORKOUTS")
                setupScreen.setProgramName(item.label)
            aWorkoutRunData.programName = item.label

            main.state = "SetupState";
        }
        else {
            aWorkoutRunData.programName = qsTr("QUICK START")
            conditionalStart()
        }
    }

    function conditionalStart(){
        brainboard.reset()
        workoutManager.resetLevels();
        aWorkoutRunData.reset()
        workoutManager.setupWorkout()
        main.state = "RunState";
        netpulse.startWorkout()
        workoutScreen.start()
        if(machineinfo.productTypeString === "Treadmill"){
            testwarning.show()
        } else if (main.isCSA && workoutProgram.isC360Program) {
            // Allow intro video to play
        } else{
            workoutManager.start()
        }
    }

    TreadmillWorkoutCountdownWarning{
        id:testwarning
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter

        onJumpStart: {
            workoutManager.jumpStart()
        }

        onTimedOut: {
            workoutManager.start()
            hide()
        }
        onVisibleChanged: if(visible) {
                              delayTimer.start()
                          }
        onTriggered: buzzer.warningBeep()
        Component.onCompleted: hide()

        Timer{
            id: delayTimer
            interval: 350
            running: false
            repeat: false
            onTriggered: {
                testwarning.startTimeout(3)
                buzzer.warningBeep()
            }
        }
    }

    function clearQLists() {
        brainboard.reset();
        workoutManager.reset();
        setupScreen.clearListView(); // Use to reset setup items
        aWorkoutRunData.reset();
    }

    Connections {
        target: application
        onIsTftChanged: {
            selectWorkoutMenu()
            workout_menu.changeMenu(main.topmenu)
        }
    }

    Connections {
        target: brainboard
        onModelStringChanged: {
            // Re-select workout menu
            selectWorkoutMenu()
            workout_menu.changeMenu(main.topmenu)

        }
    }

    function selectWorkoutMenu() {
        var Menus = isTft ? TftMenus : TouchMenus
        switch(machineinfo.productTypeString) {
            case "Treadmill":
                main.topmenu = Menus.menu_tread_top()
                main.allWorkoutsMenu = Menus.menu_tread_all()
                break;
            case "Elliptical":
            //case "StrideAdjustElliptical":
                main.topmenu = Menus.menu_elliptical_top()
                main.allWorkoutsMenu = main.isCSA ? Menus.menu_csa_all() : Menus.menu_elliptical_all()
                break;
            case "Bike":
                main.topmenu = Menus.menu_bike_top()
                main.allWorkoutsMenu = Menus.menu_bike_all()
                break;
            default:
                main.topmenu = Menus.menu_elliptical_top()
                main.allWorkoutsMenu = Menus.menu_elliptical_all()
                break;
        }
        //console.log("topmenu", JSON.stringify(main.topmenu), main.isCSA)
        workout_menu.menu = main.topmenu
    }

    ImageButton {
        id: screen_lock_button
        visible: isTouch
        enabled: (visible && opacity > 0.0)
        anchors.top: parent.top
        anchors.topMargin: 14
        anchors.rightMargin: 30
        z:1
        upimage: "graphics/LOCK_ICON_DISABLED.png"
        downimage: "graphics/LOCK_ICON_DOWN.png"
        label: isDown ? qsTr("Hold...") + application.emptyString : ""
        label_y_offset: 10
        fontsize: 14
        label_color: "black"
        label_under: true
        onPressAndHold: screen_lock_overlay.open();
        anchors.right: parent.right;
        mouseAreaScale: 2.0
    }


    // screen lock which is opened when lock icon is pressed
    ScreenLock {
        id: screen_lock_overlay
        z:99
        lockTime: 20
        visible: false
    }

    Image {
        id: background
        z: -2
        visible: !fullscreenVideoFrame.visible
        anchors.fill: parent
        source: "graphics/Background.png"

        Image {
            id: logoImage;
            enabled: (visible && opacity > 0.0)

            property bool blinking
            function startBlink() {
                logo_blink_timer.start()
                logoImage.blinking = true
            }
            function cancelBlink() {
                blinking = false
                logo_blink_timer.stop()
            }
            visible: main.isTouch || (!ipod_window.visible && !tv_window.visible)
            anchors.top: parent.top
            anchors.topMargin: 25
            anchors.left: parent.left
            anchors.leftMargin: 25

            source: "graphics/TRU_Home_Logo.png"

            MouseArea {
//                 onClicked: main.state = "DiagnosticState"   // DEBUG: easy diag access
                onPressAndHold: logoImage.startBlink()
                anchors { top: parent.top; left: parent.left
                          topMargin: -25; leftMargin: -25 }
                width: logoImage.width + 30
                height: logoImage.height + 30
                enabled: (main.state == "HomeState") && !(tv_window.opacity > 0) && !ipod_button.isDown
                Timer {
                    id: logo_blink_timer
                    property int blink_count: 0
                    running: false
                    repeat: true
                    interval: 1000
                    onTriggered: { logoImage.opacity = !logoImage.opacity ; blink_count++ }
                    onBlink_countChanged: {
                        if(blink_count > 10) {
                            logoImage.blinking = false
                            logoImage.opacity=1.0
                            blink_count=0;
                            logo_blink_timer.stop()
                        }
                    }
                } // end Timer
            } // end MouseArea for logo
        } // end Image True logo

    } // end Image background

    MouseArea {  // secret second mouse area to hold to get to diagnostics
        id: secretMouseArea
        z: logoImage.blinking ? 1000 : -10
        width: 100 ; height: 100
        anchors { bottom: parent.bottom ; right: parent.right }
        onPressAndHold: {
            if( logoImage.blinking) {
                main.state = "DiagnosticState"
                logoImage.cancelBlink()
            }
        }
    }

    NavigationBar {
        id: navbar
        visible: isTouch
        enabled: (visible && opacity > 0.0)

        anchors.top: parent.top
        anchors.topMargin: 18
        anchors.right: parent.right
        anchors.rightMargin: 15

        onClicked: {
            if(state)
                main.state = state
            if(data)
                workout_menu.changeMenu( data, highlightIndex )
            brainboard.reset();
            workoutManager.reset();
            setupScreen.clearListView(); // Use to reset setup items
        }
    }

    Loader {
        id: home_screen_loader
        sourceComponent: isTouch ? touch_home_screen_component : keyed_home_screen_component
        anchors.fill: parent
        anchors.rightMargin: 0
        anchors.bottomMargin: 0
        anchors.leftMargin: 0
        anchors.topMargin: 0
        z: 4
        opacity: 0
        enabled: (visible && opacity > 0.0)

        Component {
            id: touch_home_screen_component
            HomeScreen {
                id: touch_home_screen
                x: 0
                y: 0
                opacity: 1
                visible: !popupManager.estopShown && !errorManager.preventUse && !blockUseMessage.visible && isTouch
                /**********************************************************************

                  Truth table for buttonsEnabled
                  +-------------------+-------------------+------------++----------------+
                  | productTypeString | treadIsCalibrated | standalone || buttonsEnabled |
                  +-------------------+-------------------+------------++----------------+
                  |         ""        |     don't care    |   false    ||     false      |
                  |       "Bike"      |     don't care    |   false    ||     true       |
                  |    "Elliptical"   |     don't care    |   false    ||     true       |
                  |    "Treadmill"    |        true       |   false    ||     true       |
                  |    "Treadmill"    |        false      |   false    ||     false      |
                  |         ""        |     don't care    |    true    ||     false      |
                  |       "Bike"      |     don't care    |    true    ||     true       |
                  |    "Elliptical"   |     don't care    |    true    ||     true       |
                  |    "Treadmill"    |        true       |    true    ||     true       |
                  |    "Treadmill"    |        false      |    true    ||     true       |
                  +-------------------+-------------------+------------++----------------+

                 **********************************************************************/
                buttonsEnabled: true /*!((machineinfo.productTypeString === "" && options.ignoreFaults == false) ||
                    (machineinfo.productTypeString === "Treadmill" && machinecontrol.isCalibrated == false && options.ignoreFaults == false) ||
                    (machineinfo.productTypeString === "" && options.ignoreFaults == true))*/
                netpulseAvailable: main.netpulseAvailable
                showCardio360: machineinfo.productTypeString === "Elliptical" ? true : false
                // hide the warnings and show mini-quickstart button if ipod or tv window open
                entertainmentShown: (tv_window.visible || ipod_window.visible) ? true : false
                onQuickstartPressed: {
                    quickStart()
                }
                onLoginPressed: main.state="LoginState"
                onRegisterPressed: main.state="RegisterState"
                onSignoutPressed: {
                    netpulse.logOut()
                    main.state="HomeState"
                }
            }
        }

        Component {
            id: keyed_home_screen_component
            KeyedHomeScreen {
                id: keyed_home_screen
                x: 0
                y: 0
                opacity: 1
                enabled: (visible && opacity > 0.0)
                visible: !popupManager.estopShown && !errorManager.preventUse && !blockUseMessage.visible && !isTouch
                hideBackground: ipod_window.visible || tv_window.visible
                /**********************************************************************

                  Truth table for buttonsEnabled
                  +-------------------+-------------------+------------++----------------+
                  | productTypeString | treadIsCalibrated | standalone || buttonsEnabled |
                  +-------------------+-------------------+------------++----------------+
                  |         ""        |     don't care    |   false    ||     false      |
                  |       "Bike"      |     don't care    |   false    ||     true       |
                  |    "Elliptical"   |     don't care    |   false    ||     true       |
                  |    "Treadmill"    |        true       |   false    ||     true       |
                  |    "Treadmill"    |        false      |   false    ||     false      |
                  |         ""        |     don't care    |    true    ||     false      |
                  |       "Bike"      |     don't care    |    true    ||     true       |
                  |    "Elliptical"   |     don't care    |    true    ||     true       |
                  |    "Treadmill"    |        true       |    true    ||     true       |
                  |    "Treadmill"    |        false      |    true    ||     true       |
                  +-------------------+-------------------+------------++----------------+

                 **********************************************************************/
                buttonsEnabled: !((machineinfo.productTypeString === "" && options.ignoreFaults == false) ||
                    (machineinfo.productTypeString === "Treadmill" && machinecontrol.isCalibrated == false && options.ignoreFaults == false) ||
                    (machineinfo.productTypeString === "" && options.ignoreFaults == true))
                netpulseAvailable: main.netpulseAvailable
                showCardio360: machineinfo.productTypeString === "Elliptical" ? true : false
                // hide the warnings and show mini-quickstart button if ipod or tv window open
                entertainmentShown: (tv_window.visible || ipod_window.visible) ? true : false
                onQuickstartPressed: {
                    quickStart()
                }
                onC360QuickstartPressed: {
                    quickStart("Cardio_360_Quick_Start.xml")
                }
                onC360QuickstartHeld: {
                    demoC360Notice.showWithTimeout(3);
                    main.quickStart("Cardio_360_Demo.xml")
                }
                onWorkoutFinderPressed: {
                    if (main.isCSA) {
                        main.state = "AllWorkoutsState"
                    } else {
                        main.state = "MenuState"
                    }
                }
                onLoginPressed: main.state="LoginState"
                onRegisterPressed: main.state="RegisterState"
                onSignoutPressed: {
                    netpulse.logOut()
                    main.state="HomeState"
                }

                MessagePopup {
                    id: demoC360Notice
                    message: qsTr("Cardio360 Demo") + application.emptyString
                }
            }
        }
    }

    // Display the workout program image while program loads from XML
    Item {
        id: loadingProgramNotice

        anchors.fill: parent
        property variant programItem : undefined
        opacity: 0
        enabled: (visible && opacity > 0.0)

        Image {
            id: loadingProgramImage
            anchors {  horizontalCenter: parent.horizontalCenter ; verticalCenter: parent.verticalCenter }
            source: {
                if (!loadingProgramNotice.programItem)
                    return ""
                if (options.modestyMode && loadingProgramNotice.programItem.modestyImage) {
                    return "graphics/" + loadingProgramNotice.programItem.modestyImage
                } else {
                    return "graphics/" + loadingProgramNotice.programItem.image
                }
            }
        }

        Text {
            id: loadingProgramLabel
            text: loadingProgramNotice.programItem ? qsTranslate("Menu", loadingProgramNotice.programItem.label) : ""
            font { pointSize: 14 ; bold: true }
            anchors { horizontalCenter: loadingProgramImage.horizontalCenter
                top: loadingProgramImage.bottom
            }
        }

        Rectangle {
            id: progressBar
            color: "blue"
            height: 10
            width: (workoutProgram.parsingProgress / 100) * 360
            anchors { left: loadingProgramImage.left; leftMargin: 20; top: loadingProgramLabel.bottom; topMargin: 10 }
            opacity: 0.5
        }
    }

    Loader {
        id: workout_menu_loader
        sourceComponent: isTouch ? touch_workout_menu_component : keyed_workout_menu_component
        anchors.fill: parent
        focus: main.state === "MenuState"
        opacity: 0
        enabled: (visible && opacity > 0.0)

        Component {
            id: touch_workout_menu_component

            WorkoutMenu {
                id: touch_workout_menu

                opacity: 1
                enabled: (visible && opacity > 0.0)
                visible: (tv_window.visible || ipod_window.visible) ? false : true

                onWorkoutSelected: {
                    navbar.push("graphics/" + menu.navicon, "MenuState", menu)
                    startWorkout(item)
                }
                // signal emitted by workout menu when sub-menu selected.  menu parameter is a javascript dictionary
                onMenuSelected: {
                    // push item info onto the navigation stack. push(icon, state string, data=javascript dict)
                    // third parameter is returned in clicked(data) signal from navigation bar
                    navbar.push("graphics/" + menu.navicon, "MenuState", menu)
                }

                ImageButton {  // All workouts button
                    id: all_workouts_button
                    z:5
                    visible: (workout_menu.menu && workout_menu.menu.label === main.topmenu.label) ? 1 : 0    // show all workouts button only on top menu
                    onClicked: { main.state = "AllWorkoutsState" }
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 40
                    anchors.right: parent.right
                    anchors.rightMargin: 210
                    downimage: "graphics/200/ShowAll_SM_Button_Down.png"
                    upimage: "graphics/200/ShowAll_SM_Button_Disabled.png"
                }
                Text {
                    id: all_workouts_text
                    z: 5
                    width: 90
                    visible: all_workouts_button.visible
                    text: qsTr("SHOW ALL\nWORKOUTS") + application.emptyString
                    anchors {
                        left: all_workouts_button.right
                        leftMargin: -5
                        verticalCenter: all_workouts_button.verticalCenter
                        verticalCenterOffset: -5
                    }
                    font.pixelSize: 15
                    font.bold: true
                    wrapMode: Text.WordWrap
                }
            }
        }

        Component {
            id: keyed_workout_menu_component

            KeyedWorkoutMenu {
                id: keyed_workout_menu

                opacity: 1
                enabled: (visible && opacity > 0.0)
                visible: (tv_window.visible || ipod_window.visible) ? false : true

                onWorkoutSelected: {
                    navbar.push("graphics/" + menu.navicon, "MenuState", menu, workout_menu.currentIndex)
                    startWorkout(item)
                }
                // signal emitted by workout menu when sub-menu selected.  menu parameter is a javascript dictionary
                onMenuSelected: {
                    // push item info onto the navigation stack. push(icon, state string, data=javascript dict)
                    // third parameter is returned in clicked(data) signal from navigation bar
                    navbar.push("graphics/" + menu.navicon, "MenuState", menu, workout_menu.currentIndex)
                }

                //        ImageButton {  // All workouts button
                //            id: all_workouts_button
                //            z:5
                //            visible: false //(workout_menu.menu.label === main.topmenu.label) ? 1 : 0    // show all workouts button only on top menu
                //            onClicked: { main.state = "AllWorkoutsState" }
                //            anchors.bottom: parent.bottom
                //            anchors.bottomMargin: 40
                //            anchors.right: parent.right
                //            anchors.rightMargin: 190
                //            fontsize: 15
                //            downimage: "graphics/200/ShowAll_SM_Button_Down.png"
                //            upimage: "graphics/200/ShowAll_SM_Button_Disabled.png"
                //            label_x_offset: 90
                //            label: qsTr("SHOW ALL\nWORKOUTS") + application.emptyString
                //            label_under: false
                //            label_color: "black"
                //        }
            }

        }
    }

    WorkoutMenuAll {
        id: workouts_all

        onWorkoutSelected: {
            navbar.push("graphics/" + menu.navicon, "AllWorkoutsState", undefined)
            startWorkout(item)
        }

        menu: main.allWorkoutsMenu
        anchors.fill: parent
        opacity: 0
        enabled: (visible && opacity > 0.0)
    }

    Item {  // side panel menu
        id: side_panel

        width: 100
        height: 300
        anchors { right: parent.right ; bottom: parent.bottom  }
        visible: main.isTouch && !popupManager.estopShown
        enabled: (visible && opacity > 0.0)

        Column { // column of icons
            id: column1
            anchors { bottom: parent.bottom; bottomMargin: 30; right: parent.right ; rightMargin: 10 }
            width: parent.width
            spacing: 20
            height: childrenRect.height

           ImageButton { // TV Button
                id: tv_button ;
                visible: !main.netpulseAvailable && tv.channelList.channels.count && !blockUseMessage.visible
                toggle: true
                onClicked: ipod_button.setDown(false)
                upimage: main.netpulseAvailable ? "graphics/TV_ICON_ONDEMAND_DISABLED.png" : "graphics/TV_ICON_DISABLED.png"
                downimage: main.netpulseAvailable ? "graphics/TV_ICON_ONDEMAND_DOWN.png" : "graphics/TV_ICON_DOWN.png"
                anchors.horizontalCenter: parent.horizontalCenter;
                mouseAreaScale: 1.25
            }

            ImageButton { // iPod button
                id: ipod_button ;
                visible: !main.netpulseAvailable && !blockUseMessage.visible
                toggle: true
                onClicked: tv_button.setDown(false)
                upimage: "graphics/ITUNES_ICON_DISABLED.png"
                downimage: "graphics/ITUNES_ICON_DOWN.png"

                anchors.horizontalCenter: parent.horizontalCenter
                mouseAreaScale: 1.25
            }

            ImageButton { // language menu icon
                id: language_button;
                onVisibleChanged: {
                    if (visible && _hideLanguageButton)
                        visible = false
                }
                upimage: "graphics/LANGUAGE_ICON_DISABLED.png"
                downimage: "graphics/LANGUAGE_ICON_DOWN.png"
                anchors.horizontalCenter: parent.horizontalCenter
                mouseAreaScale: 1.25
                onClicked: {
                    main.state = "LanguageMenuState"
                }
            }
        }
    } // end side_panel Item

    ChannelMenu {
        id: tv_menu
        property bool menuOpen: false
        width: 1000
        height: (workoutScreen.workoutActive ? 406 : 510)
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.leftMargin: 183
        anchors.topMargin: 163
        z: 97
        opacity: menuOpen ? 1 : 0
        enabled: (visible && opacity > 0.0)

        channels: tv.channelList.channels
        currentChannel: tv.currentChannel
        visible: true
        onChannelSelected: {
            menuOpen = false

            if(id) {
                main.channelSelected = id
                tv.tuneChannelID(id)
            }
        }
        onClose: {
            menuOpen = false
            if(tv.currentChannel < 0)
                tv_window.hide()
        }

        Behavior on opacity {
            NumberAnimation { duration: 200; easing.type: Easing.InOutQuad }
        }
    }

    Connections {
        target: main.isTft && (main.state === "MenuState" || main.state === "SetupState" || main.state === "AllWorkoutsState" || "WorkoutSummaryState") ? keymapper : null

        onPressAndHold: {
            main.debugKeys("press and hold", "KeyedWorkoutMenu", functionIndex)
            if (functionIndex === 18 || functionIndex === 6) // 18=STOP key; 6=Center pod STOP key
                main.state = "HomeState"
        }
    }

    Connections {
        target: main.state === "HomeState" && !ipod_window.visible && !tv_window.visible ? keymapper : null

        onUpButtonPressed: {
            main.debugKeys("upButtonPressed", "main")
            volume_controls.volumeUp();
        }

        onDownButtonPressed: {
            main.debugKeys("downButtonPressed", "main")
            volume_controls.volumeDown();
        }

        onPressAndHold: {
            main.debugKeys("press and hold", "main", functionIndex)
            if (functionIndex === 21) { // double left arrow <<
                buzzer.secretBeep(0)
                logoImage.startBlink()
            } else if (functionIndex === 16 || functionIndex === 24) { // ENTER or MINUS
                if (logoImage.blinking) {
                    buzzer.secretBeep(1)
                    main.state = "DiagnosticState"
                    logoImage.cancelBlink()
                }
            }
        }
    }

    AudioManager {
        id: audioManager
        debug: true
        tvPlaying: tv.playing && !tv.isIpodInput
        ipodPlaying: (ipodmanager.playStatus === "playing")
        ipodPlayingVideo: ipodPlaying
        netpulsePlaying: netpulse.playing
        flashPlaying: flashWatcher.soundIsPlaying || html5watcher.soundIsPlaying
        btconnected: bluetoothAdapter.connected
        onAudioSourcesChanged: print("audio sources: " + audioSources)
    }

    Process {
        id: flashWatcher
        property bool soundIsPlaying: false

        program: "scripts/watch_flash.sh"

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("flashWatcher stdout", data)
            if (data === "yes") {
                flashWatcher.soundIsPlaying = true
            } else {
                flashWatcher.soundIsPlaying = false
            }
        }

        onFinished: {
            console.log("flashWatcher finished")
            flashWatcher.soundIsPlaying = false
        }

        onSoundIsPlayingChanged: {
            if (soundIsPlaying)
                audioManager.flashRequest()
        }
    }

    Process {
        id: html5watcher
        property bool soundIsPlaying: false

        program: "scripts/watch_html5.sh"

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
//            console.log("html5watcher stdout", data)
            if (data === "yes") {
                html5watcher.soundIsPlaying = true
            } else {
                html5watcher.soundIsPlaying = false
            }
        }

        onSoundIsPlayingChanged: {
            if (soundIsPlaying)
                audioManager.flashRequest()
        }
    }

    Process {
        id: flashMute
        program: "scripts/flashmute.sh"

        function setMute(flag) {
            flashMute.arguments = [flag]
            flashMute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from flashmute:",data)
        }
    }

    Process {
        id: html5Mute
        program: "scripts/html5_mute.sh"

        function setMute(flag) {
            html5Mute.arguments = [flag]
            html5Mute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from html5mute:",data)
        }
    }

    Process {
        id: vlcMute
        program: "scripts/vlcmute.sh"

        function setMute(flag) {
            vlcMute.arguments = [flag]
            vlcMute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from vlcMute:",data)
        }
    }

    Connections {
        target: audioManager
        onFlashAllowedChanged: {
            flashMute.setMute(! audioManager.flashAllowed)
            html5Mute.setMute(! audioManager.flashAllowed)
        }

        onVideoSourceChanged: {
            console.log("Video Source is: " + audioManager.videoSource)
        }

        onTvVideoAllowedChanged: {
            muteTimer.stop()
            muteTimer.start()
        }
        onVirtActiveVideoAllowedChanged: {
            muteTimer.stop()
            muteTimer.start()
        }
    }

    Timer{
        id: muteTimer
        running: false
        interval: 1000
        onTriggered: {
            running = false
            if(audioManager.tvVideoAllowed || audioManager.virtActiveVideoAllowed || audioManager.ipodVideoAllowed || (audioManager.ipodAllowed && bluetoothAdapter.connectedHeadset)){
                console.log("Unmuting VLC")
                vlcMute.setMute(0)
            } else {
                console.log("Muting VLC")
                vlcMute.setMute(1)
            }
        }
    }

    Connections {
        target: audioManager

        onBtAllowedChanged: {
            if (audioManager.btAllowed) {
                btMute.setMute(0)
            } else {
                btMute.setMute(1)
            }
        }
    }

    Process {
        id: btMute
        program: "scripts/btmute.sh"

        function setMute(flag) {
            btMute.arguments = [flag]
            btMute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from btmute:",data)
        }
    }

    BluetoothDeviceSelectionDialog{        
        id: btDeviceSelectionDialog
        isTop: true
        height: main.height * 0.42
        width: main.width * 0.60
        anchors.horizontalCenter: main.horizontalCenter
        anchors.verticalCenter: main.verticalCenter
        pairingAllowed: !(bluetoothAdapter.pairing || bluetoothAdapter.connecting)

        onClosed: btDeviceSelectionDialog.hide()
        onHeadsetConnected: btDeviceSelectionDialog.hide()
        onHeadsetDisconnected: btDeviceSelectionDialog.hide()

    }

    HeartRateDeviceSelectionDialog{
        id: heartRateDeviceSelectionDialog
        isTop: true
        height: main.height * 0.42
        width: main.width * 0.60
        anchors.horizontalCenter: main.horizontalCenter
        anchors.verticalCenter: main.verticalCenter
        isLowEnergy: true

        onClosed: hide()
        onHeadsetConnected: hide()
        onHeadsetDisconnected: hide()

    }



    VolumeControls {
        id: volume_controls

        property bool canShow: main.isTouch && (main.state == "HomeState" || main.state === "RunState" || ipod_window.visible)
        opacity: ( canShow && audioManager.audioPlaying ) ? 1.0 : 0.0
        enabled: (visible && opacity > 0.0)
        visible: !fullscreenVideoFrame.visible
        anchors.verticalCenter: parent.verticalCenter
        anchors.verticalCenterOffset: 30
        anchors.left: parent.left
        anchors.leftMargin: 32
        showHeadsetButton: !options.netpulseEnabled

        // synchronize the control display with actual volume if changed elsewhere
        Connections {
            target: audioManager
            onVolumeChanged: {
                if(audioManager.volume != volume_controls.percent)
                    volume_controls.vol_setting = audioManager.volume * (volume_controls.vol_max - volume_controls.vol_min)/100
            }

            onMuteChanged: volume_controls.muted = audioManager.mute

        }

        onBluetoothHeadsetButtonClicked: {
            if(bluetoothAdapter.connectedHeadset === true){
                bluetoothAdapter.disconnectHeadset()
            } else {
                btDeviceSelectionDialog.show()
                btDeviceSelectionDialog.state = "list"
            }
        }

        onMutedChanged: audioManager.setMute(muted)

        onPercentChanged: audioManager.setVolume(percent)

        onSourceButtonClicked: {
            print("Audio Source Button Pressed.")
            audioManager.nextSource()
        }
        showSourceButton: audioManager.audioSources.length > 1 && !audioManager.netpulsePlaying && !bluetoothAdapter.connectedHeadset

        sourceIcon: {
            if (audioManager.ipodAllowed || audioManager.btAllowed || audioManager.ipodVideoAllowed)
                return "graphics/netpulse/audio_button_iPod.png"
            else if (audioManager.tvAllowed)
                return "graphics/netpulse/audio_button_TV.png"
            else if (audioManager.virtActiveAllowed)
                return "graphics/netpulse/audio_button_VA.png"
            else if (audioManager.flashAllowed)
                return "graphics/netpulse/audio_button_WEB.png"
            else
                return ""
        }

        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }

    Television {
        id: tv

        onClicked: tv.fullscreen = tv.fullscreen ? false: true
        muted: false
        verbose: false

        Connections {
            target: audioManager
            onTvVideoAllowedChanged: {
                if(audioManager.tvVideoAllowed) {
                    tv.tuneChannelID(tv.currentChannel)
                    tv.on = true
                }
                else {
                    tv.stop()
                }
            }

            onTvAllowedChanged: {
                tv.muted = !audioManager.tvAllowed
                print("tv muted:" + audioManager.tvAllowed)
            }

        }

    }

    property alias sapManager: sapLoader.item
    Loader {
        id: sapLoader
        sourceComponent: sapManagerComponent

        function reload() {
            sapLoader.sourceComponent = undefined
            sapLoader.sourceComponent = sapManagerComponent
        }


        Component {
            id: sapManagerComponent
            Item {
                property alias item: manager
                SapManager {
                    id: manager
                    //        onChannelAdded: {
                    //            console.log("channel added:", mrl, name)
                    //            var item = {}
                    //            item.id = mrl
                    //            item.mrl = mrl
                    //            item.name = name
                    //            //item.number = 0
                    //            tv.channelList.addChannel(item)
                    //        }

                    //        onChannelRemoved: {
                    //            console.log("channel removed:", mrl, name)
                    //            var id = mrl
                    //            tv.channelList.removeChannel(id)
                    //        }

                    onChannelFound: {
                    }
                }
            }

        }
    }


    Timer {
        running: true; interval: 3000; repeat: true
        onTriggered: {
//            sapManager.printSources()
        }
    }

    TvRecordingManager {
        id: tvRecordingManager
    }

    TvRecordingMessage {
        id: tvRecordingMessage
        visible: tvRecordingManager.recording || tvRecordingManager.copying
        z: 9999999
    }


    TelevisionWindow {  // pop-up TV window available from main screen only
        id: tv_window

        LayoutMirroring.enabled: main.rightToLeft
        LayoutMirroring.childrenInherit: true

        function hide() {
            if (main.isTouch)
                tv_button.isDown = false
            else if (main.isTft)
                home_screen.setTvButton(false)
        }
        onClose: tv_window.hide()

        onVisibleChanged: {
            if (visible) {
                if (options.cabEnabled) {
                    var firstChannel = tv.channelList.getChannelItem(0)
                    if (firstChannel !== null) {
                        console.log("CAB: Autotuning to", firstChannel.id)
                        tv_menu.channelSelected(firstChannel.id)
                    }
                } else {
                    tv_menu.menuOpen = true
                }
                if (tv.currentChannel >= 0) {
                    audioManager.tvRequest()
                }

                print("tv window turning on tv")
                tv.on = true
            }
            else {
                if(!ipod_window.visible) {
                    print("tv window turning off tv")
                    tv.on = false
                }
                tv_menu.menuOpen = false
            }
        }

        Connections {
            target: tv_window.visible ? audioManager : null
            //onTvAllowedChanged: tv.muted = !audioManager.tvAllowed
        }

        Connections {
            target: main.isTft && home_screen.tvSelected && !tv_menu.menuOpen ? keymapper : null
            ignoreUnknownSignals: true

            onUpButtonPressed: {
                main.debugKeys("upButtonPressed", "TelevisionWindow", "(in main.qml)")
                volume_controls.volumeUp();
            }

            onDownButtonPressed: {
                main.debugKeys("downButtonPressed", "TelevisionWindow", "(in main.qml)")
                volume_controls.volumeDown();
            }
        }

        // expire current channel selected from home screen after 5 minutes with tv off
        Timer {
            running: (main.state !== "RunState") && (tv.currentChannel >=0)
            interval: 5 * 60 * 1000
            onTriggered: {
                tv.currentChannel = -1
                tv_window.hide()
            }
        }

        anchors { horizontalCenter: main.horizontalCenter; top: main.top; topMargin: 140 }
        opacity: tv_button.isDown || (isTft && home_screen.tvSelected) ? 1.0 : 0.0
        enabled: (visible && opacity > 0.0)
        visible: opacity > 0


        z:4  // place the TV window on top of main screen controls

        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }

    // Full-screen Video Frame, for TV or video workouts, with data overlay
    Rectangle {
        id: fullscreenVideoFrame

        property QtObject videoItem  // the currently hosted fullscreen player

        // The item passed here must have a "fullscreen" property
        function showFullscreen(item) {
            item.fullscreen = true
            fullscreenSurface.player = item
            fullscreenSurface.aspect = item.aspect
            fullscreenVideoFrame.visible = true
            fullscreenVideoFrame.videoItem = item
        }

        function endFullscreen() {
            if(fullscreenVideoFrame.videoItem) {
                fullscreenVideoFrame.videoItem.fullscreen = false
                fullscreenVideoFrame.videoItem = null
            }
            fullscreenVideoFrame.visible = false
        }

        anchors.fill: parent
        visible: fullscreenVideoFrame.videoItem != null
        color: "black"
        z: tv_menu.z - 1

        VLCSurface {
            id: fullscreenSurface

            playing: fullscreenVideoFrame.visible
            anchors.fill: parent
        }

        MouseArea {
            // This mouse area is masked by the VideoDataOverlay when shown,
            // so click screen to minimize is only available in a non-workout fullscreen TV mode
            onClicked: {
                fullscreenVideoFrame.endFullscreen()
            }
            anchors.fill: parent
        }

        VideoDataOverlay {
            id: videoOverlay

            visible: (main.state === "RunState")
            isTreadmill: main.isTreadmill
            leftSideDisplay: workoutScreen.sideDataDisplay.leftDisplay
            rightSideDisplay: workoutScreen.sideDataDisplay.rightDisplay

            showHR: heartRateManager.heartRateAvailable
            heartRate: heartRateManager.heartRate
            incline: workoutManager.treadIncline
            speed: workoutScreen.unitState == "ENGLISH" ? workoutManager.treadSpeed : Globals.mileToKm(workoutManager.treadSpeed).toFixed(1)
            speedUnits: (workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH") ) + application.emptyString
            rpm: machinecontrol.pedalRPM
            level: workoutManager.workloadLevel
            // show the channel change button only if it is the TV that is fullscreen
            showChannelButton: tv.fullscreen && !tv.isIpodInput
            opacity: 0.5
            enabled: (visible && opacity > 0.0)
            anchors.fill: parent
            showData: (main.state === "RunState")
            onScreenButtonPressed: fullscreenVideoFrame.endFullscreen()
            onChannelButtonPressed: {
                if (netpulseAvailable) {
                    if(netpulse.visible)
                        netpulse.hide()
                    else
                        netpulse.showPage("tv")
                }
                else
                    tv_menu.menuOpen = (tv_menu.menuOpen ? false : true)
            }
            onStopButtonPressed: workoutScreen.stopButtonPressedHandler()
        }
    }

    LanguageMenu {
        id: language_menu
        opacity: 0
        enabled: (visible && opacity > 0.0)
        z:99
        anchors.fill: parent
        focus: main.state === "LanguageMenuState"

        onLanguageSelected: {
            navbar.clear()
            if(lang) {
                application.selectLanguage(lang)
                if(netpulseAvailable)
                    netpulse.setLanguage(lang)
            }
            main.state = "HomeState"
        }
    }

    // return to default language after a period of inactivity in the home screen
    Timer {
        id: lang_timer
        running: main.state === "HomeState"
        interval: 5 * 60 * 1000
        onTriggered: {
            if (main.state === "HomeState") {
                application.selectLanguage(options.defaultLanguage) // reset language after workout
                if(netpulseAvailable)
                    netpulse.setLanguage(options.defaultLanguage)
            }
        }
    }

    Item {
        id: debugOverlay

        property string asources: ((audioManager.ipodPlaying ? "ip " : "") +
                                  (audioManager.ipodPlayingVideo ? "iv " : "") +
                                  (audioManager.virtActivePlaying ? "va " : "") +
                                  (audioManager.tvPlaying ? "tv " : "") +
                                  (audioManager.flashPlaying ? "fv " : "") +
                                  (audioManager.netpulsePlaying ? "np " : "") +
                                  (audioManager.btconnected ? "bt " : ""))

        property string aallowed: ((audioManager.ipodAllowed ? "ip " : "") +
                                  (audioManager.ipodVideoAllowed ? "iv " : "") +
                                  (audioManager.btAllowed ? "bt ": "") +
                                  (audioManager.virtActiveAllowed ? "va " : "") +
                                  (audioManager.virtActiveVideoAllowed ? "vv " : "") +
                                  (audioManager.tvAllowed ? "tv " : "") +
                                  (audioManager.tvVideoAllowed ? "tvv " : "") +
                                  (audioManager.flashAllowed ? "fv " : "") +
                                  (audioManager.netpulseAllowed ? "np " : "") +
                                  (audioManager.netpulseVideoAllowed ? "npv " : ""))

        opacity: 0.5
        visible: false
        x: 20
        y: 480
        z:99
        Rectangle {
            color:"gray"
            width: childrenRect.width
            height: childrenRect.height

            Column {
                spacing: 3
                Text { visible: true; text: "audio:" + debugOverlay.asources }
                Text { visible: true; text: "allowed:"+ debugOverlay.aallowed}
                Text { visible: true; text: "mux:"+ audioManager.currentMuxSource}
                Text { visible: isTreadmill ; text: qsTr("incline:") + application.emptyString + machinecontrol.treadmillInclineTarget / 10.0}
                Text { visible: isTreadmill ; text: qsTr("target speed:") + application.emptyString + machinecontrol.treadmillSpeedTarget / 10.0}
                Text { visible: isTreadmill ; text: qsTr("actual speed:") + application.emptyString + machinecontrol.treadmillSpeedActual / 10.0}
                Text { visible: !isTreadmill ; text: qsTr("Workload:") + application.emptyString + machinecontrol.workload }
                Text { visible: true; text: qsTr("time warp:") + application.emptyString + timeFactorControl.value + "x"}

                SliderControl{
                    id: timeFactorControl
                    thumbImage: "graphics/205/IPOD_Red_Dot.png"
                    value: main.timeFactor
                    width: 150
                    minValue: 0
                    maxValue: 30
                    showValue: false
                    onValueChanged: main.timeFactor = value
                }
            }
        }
    }

    Rectangle {
        id: languageDebug
        visible: false
        x: 20
        y: 400
        color: "#80659d32"
        width: 200
        height: childrenRect.height
        z:99999999999999999999
        Column {
            spacing: 2
            Repeater {
                model: [
                    { lang: "en", caption: qsTranslate("LanguageMenu", "ENGLISH") + application.emptyString } ,
                    { lang: "ru", caption: qsTranslate("LanguageMenu", "RUSSIAN") + application.emptyString } ,
                    { lang: "fr", caption: qsTranslate("LanguageMenu", "FRENCH") + application.emptyString } ,
                    { lang: "zh", caption: qsTranslate("LanguageMenu", "CHINESE") + application.emptyString } ,
                    { lang: "de", caption: qsTranslate("LanguageMenu", "GERMAN") + application.emptyString } ,
                    { lang: "nl", caption: qsTranslate("LanguageMenu", "DUTCH") + application.emptyString } ,
                    { lang: "ar", caption: qsTranslate("LanguageMenu", "ARABIC") + application.emptyString } ,
                    { lang: "es", caption: qsTranslate("LanguageMenu", "SPANISH") + application.emptyString } ,
                    { lang: "pt", caption: qsTranslate("LanguageMenu", "PORTUGUESE") + application.emptyString } ,
                    { lang: "ko", caption: qsTranslate("LanguageMenu", "KOREAN") + application.emptyString } ,
                    { lang: "ja", caption: qsTranslate("LanguageMenu", "JAPANESE") + application.emptyString } ,
                    { lang: "it", caption: qsTranslate("LanguageMenu", "ITALIAN") + application.emptyString } ,

                ]
                delegate: Component {
                    Text {
                        font { pixelSize: 16 }
                        font.bold: application.currentLanguage === modelData.lang
                        color: "white"
                        text: modelData.caption
                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                var lang = modelData.lang
                                application.selectLanguage(lang)
                                if(netpulseAvailable)
                                    netpulse.setLanguage(lang)
                            }
                        }
                    }
                }
            }
        }

    }

    WorkoutSimulator { // TBD: this object could take file names for simulated test data, etc.
        id: pedalSim
        // simulate holding grips and/or wearing telemetry HR strap
        holdingGrips: true
        hasTelemetryHR: false
        resistance: machinecontrol.workload  // feed resistance to simulator for simulated HR effects
        safetyKeyIn: machinecontrol.estop || !isTreadmill
        simulating: brainboard.simulate
        onWattsChanged: {
            if(simulating) {
                aWorkoutRunData.setWatts(watts)
                workoutManager.setWatts(watts)
            }
        }
    }
    // Keys to control the simulator
    Keys.onPressed: {
        switch(event.key){
            case Qt.Key_Q: { // fake stride-up for testing
                keymapper.keyPressHandler(8)
                break
            }
            case Qt.Key_A: { // fake stride-down for testing
                keymapper.keyPressHandler(3)
                break
            }
            case Qt.Key_R: { // Ctrl+r allows recording a TV stream
                if (event.modifiers & Qt.ControlModifier) {
                    console.log("ctrl-r pressed")
                    if (!tvRecordingManager.recording) {
                        tvRecordingManager.startRecording()
                    }
                }
                break
            }
            case Qt.Key_S: { // toggle on/off simulation of random pedaling / heart rate
                pedalSim.simulating = !pedalSim.simulating
                break;
            }
            case Qt.Key_G: {  // toggle holding heartrate grips
                pedalSim.holdingGrips = !pedalSim.holdingGrips
                break;
            }
            // must have simulation turned off to use the following controls
            case Qt.Key_BracketRight: { // ']' increase heartrate
                pedalSim.heartRate++
                break;
            }
            case Qt.Key_BracketLeft: { // '[' decrease heartrate
                pedalSim.heartRate--
                break;
            }
            case Qt.Key_Period: { // '.' increase pedal speed
                pedalSim.pedalRPM++
                break;
            }
            case Qt.Key_Comma: { // ',' decrease pedal speed
                pedalSim.pedalRPM--
                break;
            }
            case Qt.Key_AsciiTilde: // '~' toggle debug HUD
                debugOverlay.visible = !debugOverlay.visible
                break;
            case Qt.Key_QuoteDbl: {
                languageDebug.visible = !languageDebug.visible
                break
            }
            case Qt.Key_C: // 'C' toggle show mouse cursor
                application.showCursor = !application.showCursor
                break;
            case Qt.Key_O: // 0 key also takes screen shot
            case Qt.Key_P: {// 'P' take screen shot (photo)
                buzzer.buttonPress()
                print("taking screenshot " + main.screenShotCount)
                application.screenShot("/media/usb/screenshot_" + main.screenShotCount + ".png")
                infoBanner.showMessage("Screenshot " + main.screenShotCount + " saved.", 3);
                main.screenShotCount++
                break;
            }
            case Qt.Key_K: { // toggle safety key simulation
                pedalSim.safetyKeyIn = !pedalSim.safetyKeyIn
                print("estop:" + pedalSim.safetyKeyIn)
                break;
            }
            case Qt.Key_D: { // quick diagnostic entry
                main.state = "DiagnosticState"
                break;
            }
            case Qt.Key_T: {
                application.isTft = !application.isTft
                break
            }
        }
    }

    Keys.onReleased: {
        switch (event.key) {
            case Qt.Key_Q: { // fake stride-up for testing
                keymapper.keyReleaseHandler(62)
                break
            }
            case Qt.Key_A: { // fake stride-down for testing
                keymapper.keyReleaseHandler(63)
                break

            }
        }
    }

    Connections {
        target: options
        onSimulationModeChanged: {
            if (brainboard.initialized) {
                brainboard.simulate = options.simulationMode
            }
        }
    }

    Brainboard{
        id:brainboard
        objectName: "brainboard"
        estopOverride: options.ignoreFaults && !options.simulationMode
        disableBuzzer: options.disableBuzzer
        simulate: options.simulationMode

        property bool startupFlag: true
        property bool isUpdating: false
        property bool nucleusIsConnected: false
        property string validLowerBoardString: eepromMachineTypeString === detectedMachineTypeString ? eepromMachineTypeString : ""

        onInitializedChanged: {
            if(initialized) {
                print("brainboard firmware:" + machineinfo.brainboardFirmwareVersion)
                print("product: " + machineinfo.modelString)
                brainboard.setDebugLogLevel(options.brainboardDebugLogLevel)
                if(options.fanOption === "2-Wire")
                    brainboard.set2WireFanOption()
                else if(options.fanOption === "3-Wire")
                    brainboard.set3WireFanOption()
                brainboard.simulate = options.simulationMode
                buzzer.setBrainboardObject(this);
                messagecenter.setBrainboardObject(this);
            }
        }

        onValidLowerBoardStringChanged: {
            if(validLowerBoardString === "DCI"){
                brainboard.getTreadmillLowerBoardVersionInfo();
            }
        }

        // simulator used by the brainboard object for computing simulated workout data
        dataPollRate: 500
        simulator: pedalSim

        // Connections: IpodManager ---> Brainboard
        onIPodCurrentTrackInfoChanged: ipodmanager.updateCurrentTrackInfo(title,artist,album,time,offset)
        onIPodMenuItemReceived: ipodmanager.addMenuItem(label)
        onIPodTrackIndexChanged: ipodmanager.currentTrackIndex = index
        onIPodConnected: {
            print("iPod Connected: "+connected)
            audioManager.setVolume(24)
            ipodmanager.iPodConnectedSlot(connected);
            if(main.state == "HomeState") {
                if (main.isTft)
                    home_screen.setIPodButton(connected)
                else
                    ipod_button.setDown(connected)
            }
        }

        onProductTypeStringChanged: {
            if(productTypeString == "Treadmill") dataPollRate = 100
            else dataPollRate = 33
        }

        onTreadmillPollCycle:{
            if((treadmillInclineMaxADC != 0  || treadmillInclineMinADC != 0) && machinecontrol.isCalibrating == false){
                if(startupFlag){
                    startupFlag = false
                    machinecontrol.treadmillInclineTarget = 0
                }
            }
        }

        onModelStringChanged: {
            if(!modelString)
                return

            /* TODO: min/max incline is not yet available when model string is first received from brainboard,
                    so decline configuration is not detected correctly here. */
            var decline = (machineinfo.treadmillInclineMinPercentGrade < 0)
            var config = productConfig.getProductConfig(modelString, decline)

            if(config)
                configurationData.loadConfigurationData(config)
            if(modelString == "----" && !restartRequired)
                main.state = "SetupWizardState"
        }

        onErrorReceived: {
            console.debug("Error received: " + error)
            if(brainboard.isUpdating == false && nucleusIsConnected == false){
                errorManager.errorReceived(error,statisticManager.totalDistance)
                syncDisk.start()
            }
        }

        onTreadmillSpeedChanged: print("setting treadmill speed:" + speed)
        onTreadmillInclineChanged: print("setting treadmill incline:" + incline)
        Component.onDestruction: shutdown()

        onButtonPressed: if(keymapper.keysEnabled)keymapper.keyPressHandler(button)
        onButtonReleased: if(keymapper.keysEnabled)keymapper.keyReleaseHandler(button)

    }

    Connections {
        target: heartrate
        onTelemetryHeartRateChanged: heartRateManager.setHeartRate("telemetry", telemetryHeartRate)
        onContactHeartRateChanged: heartRateManager.setHeartRate("contact", contactHeartRate)
    }

    RecoveryManager{
        id: recoveryManager
        onStarted: brainboard.startRecovery()
        onCompleted: {
            errorManager.preventUse = false
            errorManager.degradedUse = false
            popupManager.reset()
        }
    }

    Connections{
        target: brainboard
        onRecoveryResult: {
            recoveryManager.recoveryResultSlot(result)
        }
    }

    Connections{
        target: errorManager
        onErrorEvent:{
            recoveryManager.errorTriggeredSlot(errorcode)
        }
    }

    ProductConfigData {
        id: productConfig
    }

    PersonPresentDetectionManager{
        id:personPresentDetectionManager
        isEnabled: (options.ppdEnabled) && (brainboard.detectedMachineType === Brainboard.LOWERBOARD_DCI)
    }

    KeyMapper{
        id: keymapper
        function handleStart() {
            if (blockUseMessage.visible || errorManager.fatalError || errorManager.preventUse || main.state === "SleepModeState")
                return
            if (main.isTouch) {
                quickStart()
                main.channelSelected = ""
            }
        }
        onStartQuickStartPressed: {
            keymapper.handleStart()
        }
        onStartPressed: {
            keymapper.handleStart()
        }
    }

    Connections {
        id: quickStartHandler
        target: main.isTft && !main.blockKeys && !blockUseMessage.visible && (main.state === "HomeState" || main.state === "MenuState" || main.state === "AllWorkoutsState") ? keymapper : null
        onStartPressed: {
            main.debugKeys("startPressed", "main")
            quickStart()
        }
        onStartQuickStartPressed: { // thumb quickstart button
            main.debugKeys("startQuickStartPressed", "main")
            quickStart()
        }
    }


    WorkoutRunData {
       id: aWorkoutRunData

        workoutProgram: workoutProgram.programIndex
        productTypeString: machineinfo.productTypeString
        heartRate: heartRateManager.heartRate
        workloadLevel: workoutManager.workloadLevel
        paceLevel: workoutManager.paceLevel
        wattsAt60: workoutManager.wattsAt60;
        timeFactor: main.timeFactor
        strideLength: workoutManager.strideLevel
    }

    WorkoutManager {
        id: workoutManager

        property bool withLimit: true

        productType: machineinfo.productType
        workoutData: aWorkoutRunData
        userData: userData
        heartRate: heartRateManager.heartRate

        onWattsAt60Changed: machinecontrol.workload = wattsAt60
        onSpeedChanged: machinecontrol.treadmillSpeedTarget = (main.state == "RunState") ? treadSpeed : 0 // allow non-zero speed only in RunState
        onInclineChanged: machinecontrol.treadmillInclineTarget = treadIncline

        onStrideLevelChanged: machinecontrol.strideLengthTarget = strideLevel

        workoutProgram: workoutProgram

        onWorkoutDataChanged: brainboard.sendPacket(workoutData) // TODO: NEED A SIGNAL FROM WORKOUT MANAGER FOR THIS
        timeFactor: main.timeFactor
        cooldownDuration: options.cooldownTime
        //treadInclineMax: configurationData.maxInclinePercentGrade
        //treadInclineMin: configurationData.minInclinePercentGrade

        onWorkoutModeChanged:
        {
            print("Workout Mode:" + mode)
            aWorkoutRunData.setWorkoutMode(mode);
            if (mode == WorkoutRunData.SUMMARY_MODE) {
                workoutScreen.stopWorkout()
                treadSpeed = 0
                machinecontrol.treadmillSpeedTarget = treadSpeed
            }
            if (mode == WorkoutRunData.COOL_DOWN_MODE) {
                workoutScreen.showMessage(qsTr("COOL DOWN") + application.emptyString)
            }
        }
        onDisplayHRPrompt: workoutScreen.displayHRPrompt(index);
    } // end WorkoutManager

    ProgSetup {
        id: setupScreen
        enabled: (visible && opacity > 0.0)

        focus: main.state === "SetupState"

        onStartProgram:{
            workoutProgram.setupComplete()
            conditionalStart()
//            workoutManager.start();
//            main.state = "RunState";
//            workoutScreen.start();
        }

        onSaveCustomProgram: workoutProgram.saveProgram(programName,saveLocation);

        anchors.fill: parent
        opacity: 0
        z: 8
    }

    WorkoutProgram {
        id: workoutProgram

        userData: userData
        userWeightMax: productConfig.getProductConfigMaxUserWeight(machineinfo.modelString)
        metricUnits: options.metricUnits
        saveWorkoutEnabled: options.saveWorkoutEnabled
        property int maxSegments: Math.min( profile1.count, profile2.count)
        inclineTable: machineinfo.inclineTable
        speedTable: machineinfo.speedTable
        brakeWattsTable: machineinfo.brakeWattsTable
        paceTable: machineinfo.paceTable
        brainboardProductTypeString: machineinfo.productTypeString

        onDisplayDataVarsChanged: brainboard.sendDisplayVariables(displayDataVars)
    }
    BlueToothLowEnergy {
        id: ble
    }

    WorkoutScreen {
        id: workoutScreen
        enabled: (visible && opacity > 0.0)

        visible: !fullscreenVideoFrame.visible

        onWorkoutStopped: {
            audioManager.setMute(false)
            flashWatcher.kill()
            html5watcher.kill()
            tv.currentChannel = -1
            main.state = "WorkoutSummaryState";
            workoutManager.stop();
            stopPrompt.hide()
            viewContainerBanner.hide()
            if (bluetoothAdapter.discovering) {
                bluetoothAdapter.stopDiscovery()
            }

            workout_summary.setData( aWorkoutRunData )
            //prepare data for blutooth advertisment
            ble.exportWorkoutDataForBleno(aWorkoutRunData,application,brainboard)

            // add to statistics
            statisticManager.mileage += aWorkoutRunData.totalDistance * 100
            statisticManager.runHours += aWorkoutRunData.totalTime
            statisticManager.totalDistance += aWorkoutRunData.totalDistance * 100
            statisticManager.writeFile()

            // end of workout
            if (netpulseAvailable)
            {
                dataExport.writeNetpulseData("/netpulse/upload")
                netpulse.endWorkout()
                netpulse.logOut()
            }

            brainboard.reset();
            workoutManager.reset();
            workoutManager.resetLevels();
            aWorkoutRunData.mode = WorkoutRunData.SUMMARY_MODE
            setupScreen.clearListView();

            workoutScreen.container.destroyViews()

            if(ipodmanager.nikePlusRecord && ipodmanager.iPodConnected){
                workoutScreen.showMessage(qsTr("CONGRATULATIONS! WORKOUT RECORDED! SEE YOUR WORKOUT AT NIKEPLUS.COM.") + application.emptyString)
                brainboard.sendGuiWorkoutEvent(1);
                miscStats.incrementNikePlusUsage()
            }

            // if bluetooth is connected, disconnect the bluetooth device
            if(bluetoothAdapter.connected){
                bluetoothAdapter.disconnect()
            } 		
            if (bluetoothAdapter.connectedHeadset)
                bluetoothAdapter.disconnectHeadset()

            machinecontrol.autoStrideEnabled = false

            console.debug("WORKOUT STOPPED")

        }//

        onWorkoutPaused: {
            workoutManager.pause()
            if(ipodmanager.nikePlusRecord && ipodmanager.iPodConnected){
                brainboard.sendGuiWorkoutEvent(2);
                console.debug("WORKOUT PAUSED")
            }
        }
        onWorkoutStarted: {
            quickStartLoading = false
            if (options.cabEnabled) {
                var firstChannel = tv.channelList.getChannelItem(0)
                if (firstChannel) {
                    console.log("CAB: Autotuning to", firstChannel.id)
                    tv.tuneChannelID(firstChannel.id)
                }
            }
            if(ipodmanager.nikePlusRecord && ipodmanager.iPodConnected){
                brainboard.sendGuiWorkoutEvent(0);
                var message = qsTr("RECORDING WORKOUT TO APPLE DEVICE")
                if(isTreadmill)
                    workoutScreen.showBannerMessage(message)
                else
                    workoutScreen.showMessage(message)
            }
        }
        onWorkoutResumed: {if(ipodmanager.nikePlusRecord && ipodmanager.iPodConnected){brainboard.sendGuiWorkoutEvent(3); console.debug("WORKOUT RESUMED")}}


        anchors.fill: parent
        opacity: 0


    }

    WorkoutSummary {
        id: workout_summary
        timeLeft: idle_timer.secondsLeft()
        opacity: 0
        enabled: (visible && opacity > 0.0)
        focus: main.state === "WorkoutSummaryState"
        onClose: {
            saveHasHappened = false
            main.state = "HomeState"
        }
        onSave: {
            var success = aWorkoutRunData.saveResults()
            if(success) {
                saveHasHappened = true
                infoPopup.showMessage(qsTr("Summary has been saved.") + application.emptyString, 5);
            }
            else
                infoPopup.showMessage(qsTr("Error. Check your memory device.") + application.emptyString, 5);
        }
    }

    SetupWizard{
        id: setupWizard

        focus: main.state === "SetupWizardState"

        opacity: 0
        enabled: (visible && opacity > 0.0)
        onSetupWizardComplete: {
            main.restartRequired = true
            main.state = "HomeState"
        }
        onLanguageSelected:{
            if(lang){
                application.selectLanguage(lang)
                options.defaultLanguage = lang
                if(netpulseAvailable)
                    netpulse.selectLanguage(lang)

            }
        }

    }

    BlueToothHeartRate {
        id: bluetoothLEAdapter
        onHeartrateChanged: {
            bluetoothLEAdapter.connecting=false
            heartRateManager.setHeartRate("bluetooth", bluetoothLEAdapter.heartrate)
        }
        //onScanningChanged: {
            //if(bluetoothLEAdapter.scanning == true)

        //}
        onHrTimeout: {
            console.log("Heartrate timed out...now disconnecting")
            bluetoothLEAdapter.disconnect(bluetoothLEAdapter.deviceMAC,bluetoothLEAdapter.hrHandle)
        }
        onConnected: {
            heartRateManager.currentBtAddress = bluetoothLEAdapter.deviceMAC
        }
        onHrDisconnectingComplete: {
            heartRateManager.currentBtAddress = ""
            heartRateManager.clearBluetoothHeartRateDeviceList()
        }
    }


    HeartRateManager{
        id: heartRateManager
        onBluetoothHRConnect: {
            if(heartRateManager.currentBtAddress !== address)
                bluetoothLEAdapter.getHrServiceHandle(address)
        }
    }

    Connections{
        target: workoutScreen
        onWorkoutStopped: heartRateManager.reset()

    }

    // Both SetupWizard and DiagnosticsMain use this, so put it here
    ModalDialog {
        id:productSelectedConfirmation
        property alias text: dialogText.text
        BorderImage {   // pop-up dialog background
            id: border

            x: 332
            y: 161
            width: 800
            height: 350
            source: "graphics/Pop_Up_Lrg_Grey.png"
            border { top: 152; bottom: 165; left: 145; right: 144 }

            Text {
                id: dialogText
                anchors.fill: parent
                anchors.verticalCenter: parent.verticalCenter
                color: "#ffffff"
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.pixelSize: 50

            }
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    MediaBrowser {
        id: ipod_window

        keysAccepted: opacity > 0
        allowFullscreen: false
        z:4
        opacity: ipod_button.isDown || (!isTouch && home_screen.ipodSelected) ? 1.0 : 0.0
        enabled: (visible && opacity > 0.0)
        visible: opacity > 0

        isDialogVisible: visible

        Connections {
            target: main.isTft && !main.blockKeys && ipod_window.opacity > 0 ? keymapper : null
            onBackPressed: {
                main.debugKeys("backPressed", "main", "ipod_window")
                ipod_window.close()
            }
        }

        onClose: {
            if (main.isTouch)
                ipod_button.setDown(false)
            else if (main.isTft)
                home_screen.setIPodButton(false)
        }
        onVisibleChanged: {
            if(visible)
                shown()
            else
                hidden()
        }
        anchors {   horizontalCenter: parent.horizontalCenter
                    top: parent.top
                    topMargin: 140
        }

        menu_bottom_margin: isTouch ? 60 : 160
        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }

    Connections {
        // Workout Summary has a button for resetting the timer, so ignore that state
        target: main.isTft && main.state !== "WorkoutSummaryState" ? keymapper : null
        onDownButtonPressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
        onUpButtonPressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
        onLeftButtonPressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
        onRightButtonPressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
        onEnter2Pressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
    }

    Loader {
        id: diagnostics_screen

        focus: main.state === "DiagnosticState"
        source: (main.state === "DiagnosticState" ? "Diagnostics/DiagnosticsMain.qml" : "")
        opacity: 0
        enabled: (visible && opacity > 0.0)
        anchors.fill: parent

        Connections {
            target: diagnostics_screen.item
            ignoreUnknownSignals: true

            onClose: {
                main.state = "HomeState"
                if(machinecontrol.isCalibrated){
                    machinecontrol.treadmillInclineTarget = 0
                }
            }
        }
    }

    LogUploader {
        id: logUploader
    }

    Options {
        id: options
        objectName: "options"
        onContactHRPriorityChanged: {
            heartRateManager.setContactPriority(contactHRPriority)
        }
        onLcdBrightnessChanged: main.lcdBrightness = lcdBrightness
        onCabEnabledChanged: {
            cabEnabled ? brainboard.setCabOption() : brainboard.setCsafeOption()
        }
        onIgnoreFaultsChanged: {
            if (options.ignoreFaults) {
                popupManager.reset()
            }
        }
        onLoggingEnabledChanged: {
            logger.enabled = loggingEnabled
        }        
        onEn957EnabledChanged: {
            brainboard.setEn957(en957Enabled)
        }
    }

    property int lcdBrightness: options.lcdBrightness
    onLcdBrightnessChanged: {
        main.setLcdBrightness(lcdBrightness)
    }

    function setLcdBrightness(percent) {
        if(main.consoleType === "Transcend10")
            lcdBrightnessProcess.setBrightness(percent)
        else
            brainboard.setLcdBrightness(percent)
    }

    Process {
        id: lcdBrightnessProcess
        program: "scripts/set_brightness.sh"

        function setBrightness(percent) {
            lcdBrightnessProcess.arguments = [ (percent*255/100).toFixed(0) ]
            lcdBrightnessProcess.start()
        }
    }

    Binding {
        target: application
        property: "showCursor" ; value: options.showCursor
    }

    // Idle timer
    // Time out interval is set by state transition table below.
    // The timer is reset by a screen touch (event sent by application object.)
    // Timeout action is defined in onTriggered event.  Many screens return to HomeScreen state.
    Timer {
        id: idle_timer

        repeat: true
        running: true
        interval: 10000
        onRunningChanged: idleSecondsTimer.reset()
        onIntervalChanged: idleSecondsTimer.reset()

        function secondsLeft() {
            var sec = (interval / 1000) - idleSecondsTimer.seconds
            if(sec < 0 )
                return 0
            else
                return sec
        }

        onTriggered: {
            switch(main.state) {
                case "HomeState":{  // idle timeout on home-screen goes to screen-saver or sleep
                    // close warning screen
                    if (main.isTouch) {
                        home_screen.showingWarningGuide = false
                    }
                    // close TV and iPod windows
                    tv_button.setDown(false)
                    //ipod_button.setDown(false)
                    netpulse.logOut()

                    if(options.screenSaveTimeout) {  // screensaver enabled
                        main.state = "ScreenSaverState"
                        screensaver.start()
                    } // otherwise, go to sleep mode if enabled
                    else if(options.sleepModeTimeout) {
                        main.state = "SleepModeState"
                    }

                    // disconnect bluetooth device if connected
                    if (bluetoothAdapter.connected) {
                        bluetoothAdapter.disconnect()
                    }
                    if (bluetoothAdapter.connectedHeadset)
                        bluetoothAdapter.disconnectHeadset()
                    break;
                }
                case "LoginState":
                case "RegisterState":
                {
                    // the login window has its own timeout timer
                    // TODO: should there be a watchdog here in case of communication loss?
                   break;
                }
                case "WorkoutSummaryState": {
                    workout_summary.close() // This will make sure the bt advertise dialog closes on timeou
                    break
                }
                case "MenuState":
                case "AllWorkoutsState":
                case "SetupState":
                case "LanguageMenuState": {
                    // disconnect bluetooth device if connected
                    if (bluetoothAdapter.connected) {
                        bluetoothAdapter.disconnect()
                    }
                    if (bluetoothAdapter.connectedHeadset)
                        bluetoothAdapter.disconnectHeadset()
                    main.state = "HomeState";
                    break;
                }
                case "ScreenSaverState": {
                    if (options.sleepModeTimeout) {
                        main.state = "SleepModeState"
                    } else if(options.erpEnabled){
                        brainboard.erpSleep()
                    }
                    break;
                }
                case "SetupWizardState":
                {
                    break;
                }
                case "SleepModeState":
                {
                    if(options.erpEnabled){
                        brainboard.erpSleep()
                    }

                    break;
                }
            } // end switch
        } // end onTriggered
    }

    Timer {
        id: dim_timer
        interval: options.dimDisplayTimeout * 1000
        running: false
        repeat: false
        onTriggered: {
            if (main.state != "HomeState" && main.state != "ScreenSaverState")
                return

            // Set to 40% of brightness setting
            var brightnessLevel = parseInt(0.40 * options.lcdBrightness)
            main.lcdBrightness = brightnessLevel
            console.log("Dim timer fired. Set brightness to", brightnessLevel)
        }
    }

    Timer {
        id:idleSecondsTimer

        property int seconds: 0

        function reset() {
            idleSecondsTimer.seconds = 0
            idleSecondsTimer.restart()
        }

        interval: 1000
        running: true
        onRunningChanged: seconds = 0
        repeat: true
        onTriggered: seconds++
    }

    Timer {
        id: playBluetooth
        interval: 1000; repeat: false; running: false

        onTriggered: {
            if (bluetoothAdapter.currentConnectedDevice) {
                if (bluetoothAdapter.currentConnectedDevice.player) {
                    bluetoothAdapter.currentConnectedDevice.player.play()
                }
            }
        }
    }

    TrueBlueAdapter{
        id: bluetoothAdapter

        onConnectedChanged: {
            if (connected) {
                btMute.setMute(0)
                audioManager.setVolume(24)
                playBluetooth.start()
            }
        }

        onConnectedHeadsetChanged: {
            if (tv.isIpodInput) {
                // restart tv if iPod is currently playing, so that it will start streaming the ipod
                // input to pulseaudio, which is required for the headset to function
                tv.mrl = ""
                tv.tuneChannelID("IPODINPUT")
            }
        }
    }
    // Expire bluetooth connection after 5 minutes without starting a workout
    Timer {
        id: bluetoothSessionTimer
        running: (main.state !== "RunState") && bluetoothAdapter.connected
        interval: 2 * 60 * 1000
        onTriggered: {
            ipod_window.close()
            if (bluetoothAdapter.connected)
                bluetoothAdapter.disconnect()
            if (bluetoothAdapter.connectedHeadset)
                bluetoothAdapter.disconnectHeadset()
        }
    }

    // Expire bluetooth Heartrate connection after 5 minutes without starting a workout
    Timer {
        id: bluetoothLESessionTimer
        running: (main.state !== "RunState") && bluetoothLEAdapter.hrConnected
        interval: 2 * 60 * 1000
        onTriggered: {
            if (bluetoothLEAdapter.hrConnected)
                bluetoothLEAdapter.disconnect()
        }
    }

    KeypadProcess {
        id: urlKeypad
        name: "urlKeypad"
    }

    KeypadProcess {
        id: inputKeypad
        name: "inputKeypad"
    }

    BroadcastVisionManager{
        id: broadcastVisionManager
        Component.onCompleted: setBrainboard(brainboard)
    }
    Connections{
        target: brainboard
        onBroadcastVisionReplyReceived: broadcastVisionManager.ReplyReceivedSlot(command,data)
        onBroadcastVisionIsPresentChanged: {broadcastVisionManager.isAvailable = broadcastVisionIsPresent; console.debug("bv:" + broadcastVisionManager.isAvailable)}
    }

    Rectangle {
        id: touchSplat

        property int size: 10

        opacity: 0.3
        width: size
        height: width
        radius: width
        z: 100

        transform: Translate { x:-touchSplat.size/2 ; y: -touchSplat.size/2 }

        color: "#0099FF"

        ParallelAnimation {
            id:splat
            NumberAnimation { target: touchSplat; properties: "size" ; from: 15 ; to: 0; duration: 500; easing.type: Easing.InOutQuad }
            NumberAnimation { target: touchSplat; properties: "opacity" ; from: 0.3 ; to: 0; duration: 500; easing.type: Easing.InOutQuad }
        }
    }

    Connections {
        // reset the idle timer when there's a screen touch
        target: application
        onScreenTouched: {
            splat.stop() // Stop in-progress splat
            idle_timer.restart()
            lang_timer.restart()
            touchSplat.x = x
            touchSplat.y = y
            touchSplat.opacity = 0.3
            touchSplat.size = 15
            if(main.state == "SleepModeState")
                main.state = "HomeState"
            else if (main.state === "HomeState") {
                main.lcdBrightness = options.lcdBrightness
                if (!options.screenSaveTimeout && options.dimDisplayTimeout)
                    dim_timer.restart()
            }
        }
        onMouseReleased: {
            touchSplat.x = x
            touchSplat.y = y
            splat.restart()
        }
    }
    Timer{
        property int index: 0
        id:ellipticalHomeScreenMsgCtrTimer
        repeat: true
        interval: 3000
        onTriggered: {
            switch(index){
                case 0: messagecenter.setString("     SELECT WORKOUT", "     OR PRESS START", 0); break;
                case 1: messagecenter.setString("     SELECT WORKOUT", "     TRY  CARDIO360", 0); break;
            }
            index++;
            index = index % 2

        }
        onRunningChanged: if(running) triggered()
    }

    Connections {
        target: machineinfo
        onDetectionComplete: {
            if(lowerSystem === "NO_LOWERSYSTEM") {
                blockUseMessage.hide()
                errorManager.errorReceived(ErrorManager.ERRORNO_BB_BRAINBOARDNOTPRESENT,statisticManager.totalDistance)
            } else if(lowerSystem === "BRAINBOARD_BOOT") {
                blockUseMessage.hide()
                brainboard.goToBootloader()
                brainboard.isUpdating = true
                main.firmwareUpdateRequired = true
            }
        }
        onIsReadyChanged: {
            if(isReady) {
                if(options.cabEnabled) {
                    brainboard.setCabOption()
                } else {
                    brainboard.setCsafeOption()
                }
            }
        }
    }

    onStateChanged: {
        print("UI state: " + main.state)

        if(state === "HomeState"){
            brainboard.sendTouchAppState(Brainboard.TOUCHAPP_HOMESCREEN)

            if(machineinfo.productTypeString === "Elliptical") ellipticalHomeScreenMsgCtrTimer.running = true
            else messagecenter.setString("     SELECT WORKOUT", "     OR PRESS START", 0);

            if(machineinfo.productTypeString === "Elliptical") {
                machinecontrol.workload = 0
            }
        }
        else if(state === "RunState"){
            brainboard.sendTouchAppState(Brainboard.TOUCHAPP_RUNSCREEN)
            if(machineinfo.productTypeString === "Elliptical") ellipticalHomeScreenMsgCtrTimer.running = false

        }
        else{
            //brainboard.sendTouchAppState(Brainboard.TOUCHAPP_DIAGSCREEN)
            if(machineinfo.productTypeString === "Elliptical") ellipticalHomeScreenMsgCtrTimer.running = false
            messagecenter.setString("        VISIT", "   WWW.TRUEFITNESS.COM", 0);
        }
    }

    states: [
        State {
            name: "HomeState"
            PropertyChanges {target: home_screen_loader ; opacity: 1 }
            PropertyChanges {target: screen_lock_button ; opacity: isTouch ? 1 : 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 1 }
            PropertyChanges {target: language_button ; visible: true }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 1 }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            // set timeout for screen-saver or dim mode or sleep mode
            PropertyChanges {target: idle_timer ; interval: options.screenSaveTimeout ? options.screenSaveTimeout*1000 : options.sleepModeTimeout*1000 }
            PropertyChanges { target: dim_timer; running: options.dimDisplayTimeout > 0 && !options.screenSaveTimeout }
            PropertyChanges {target: workout_summary ; dialogVisible: false }
        },
        State {
            name: "LoginState"
            PropertyChanges {target: home_screen_loader ; opacity: 1 }
            // netpulse
            StateChangeScript { script: netpulse.showPage("login") }
            PropertyChanges {target: screen_lock_button ; opacity: isTouch ? 1 : 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 1 }
            PropertyChanges {target: language_button ; visible: true }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // currently the idle timer does nothing in this state.
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 30000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "RegisterState"
            PropertyChanges {target: home_screen_loader ; opacity: 1 }
            // netpulse
            StateChangeScript { script: netpulse.showPage("registration") }
            PropertyChanges {target: screen_lock_button ; opacity: isTouch ? 1 : 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 1 }
            PropertyChanges {target: language_button ; visible: true }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // currently the idle timer does nothing in this state.
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 30000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "LanguageMenuState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 1 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 0 }
            PropertyChanges {target: language_button ; visible: false }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
        },

        State {
            name: "MenuState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 1 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 1 }
            PropertyChanges {target: language_button ; visible: true }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "AllWorkoutsState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 1 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; visible: true }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "LoadState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; visible: false }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: loadingProgramNotice ; opacity: 1 }
            PropertyChanges {target: warningText ; opacity: 0 }
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
        },
        State {
            name: "SetupState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; visible: false }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 1 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutSetupTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "RunState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; visible: false }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 1 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // turn off idle timer - run screen will have its own
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 0}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "WorkoutSummaryState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 0 }
            PropertyChanges {target: language_button ; visible: false }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 1 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutSummaryTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }

        },
        State {
            name: "DiagnosticState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; visible: false }
            PropertyChanges {target: diagnostics_screen ; opacity: 1 ; z: 50}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0  }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 300000}
            PropertyChanges {target: dim_timer; running: false }

        },
        State {
            name: "SetupWizardState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; visible: false }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0  }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 1 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 0}
            PropertyChanges {target: dim_timer; running: false }

        },
        State {
            name: "ScreenSaverState"
             // set timeout until moving from screensaver to dim-mode, sleep-mode, or power-down
            PropertyChanges {target: idle_timer ; interval: options.sleepModeTimeout ? options.sleepModeTimeout * 1000 : options.erpTimeout * 1000 }
            PropertyChanges { target: dim_timer; running: options.dimDisplayTimeout > 0 }
        },
        State {
            name: "SleepModeState"
             // set timeout until moving from screensaver to power-down/sleep mode
            PropertyChanges {target: main; lcdBrightness: 0; restoreEntryValues: true ; explicit: true }
            PropertyChanges {target: idle_timer ; interval: options.erpTimeout * 1000 }
            PropertyChanges {target: dim_timer; running: false }
        }

    ] // end states
    transitions: [
        Transition {
                NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
        },

        Transition {
            from: "HomeState"
            to: "MenuState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    navbar.push("graphics/navbar/Navigation_Home", "HomeState", undefined )
                    workout_menu.changeMenu(main.topmenu) //Menus.menu_elliptical_top)
                }
            }
        },
        Transition {
            from: "HomeState"
            to: "AllWorkoutsState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    navbar.push("graphics/navbar/Navigation_Home", "HomeState", undefined )
                    // refresh menu
                    selectWorkoutMenu()
                    workout_menu.changeMenu(main.allWorkoutsMenu)
                }
            }
        },
        Transition {
            from: "MenuState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "AllWorkoutsState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "SetupState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "DiagnosticState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                    syncDisk.start()
                    if (main.isTft) home_screen.resetCursor()
                }
            }
        },
        Transition {
            from: "MenuState"
            to: "AllWorkoutsState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: { navbar.push("graphics/navbar/Navigation_WF", "MenuState", undefined ) }
            }
        },
        Transition {
            from: "WorkoutSummaryState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    workout_summary.saveHasHappened = false
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    audioManager.reset()
                    main.quickStartLoading = false
                    if (main.isTft) home_screen.resetCursor()
                }
            }
        }
    ]
}
