#!/bin/bash
#set -x
# This script takes one or zero arguments.  If one arguments is passed it is
# of the form <media_type>/<archive_name>
# e.g.:
#   /media/usb/003-5330001-diag-1449514379
#  With one argument it will save the archive on
# a usb-key mounted on /media/usb
#
# It the argument is empty it will upload the archive to the server
# specified by $SERVER below.
#
SCRIPTDIR=$(cd "$( dirname "$0" )" && pwd )
SERIAL=$(sh $SCRIPTDIR/product_serial.sh)
if [ -z "$SERIAL" ] ; then
  SERIAL="unknown-serial"
fi

if [[ "$#" -eq 0 ]]; then
  DIAGDIR=net/$SERIAL-dist-$(date "+%s")
elif [[ "$#" -eq 1 ]]; then
  DIAGDIR=$1
else
  echo "Too many arguments"
  exit 1
fi

DIAGBASE=$(basename $DIAGDIR )
TMP=/tmp/net
TMPBASE=$TMP/$DIAGBASE
SRC=
# Clean up previous tmp files
sudo rm -rf $TMP/*
mkdir -p $TMPBASE

#
# Set logging type parameters
#
# If this is a usb type operation, store the temp files on the usb.
#
if [[ $DIAGDIR =~ usb ]]; then
  ARCHIVE="usb"
  DESTDIR=$DIAGDIR/
  mkdir $DESTDIR
elif [[ $DIAGDIR =~ net ]]; then
  #
  # If this is a network operation, use /tmp storage and set
  # the server parameters
  #
  ARCHIVE="remote"
  USER=rlog
  #SERVER=diagnostics.truefitness.com
  SERVER=truedev.truefitness.com
  if [[ $SERVER =~ truedev ]]; then
    echo "Script is configured with SERVER=$SERVER, for testing only"
    echo "this needs to be modified before release."
    #statements
  fi
  echo "Server configured "
  DESTTOP=/home/$USER/logs/$DIAGBASE
  DESTDIR=$USER@$SERVER:$DESTTOP
  SSHDIR=$SCRIPTDIR/ssh
  KEY=$SSHDIR/id_truelogs_rsa
  KNOWN_HOSTS=$SSHDIR/known_hosts
elif [[ $DIAGDIR =~ recovery ]]; then
  # Reserved for future USER
  ARCHIVE="recovery"
else
  echo "Unable to determine logging type... (usb, network, recovery )"
  exit 1000
fi

#
# If we are transferring to a remote server, test the connection to the server.
#
if [[ $ARCHIVE = remote ]]; then
  # Get public keys from server, so we can ssh non-interactively
  sudo chown -R true.true $SSHDIR
  sudo chmod 700 $SSHDIR
  sudo chmod -R 600 $SSHDIR/*
  ssh-keyscan -Ht rsa $SERVER >> $KNOWN_HOSTS
  # Test ssh connection before trying Transfer
  ssh -q -o BatchMode=yes -o ConnectTimeout=10 -i $KEY $USER@$SERVER -l rlog -o UserKnownHostsFile=$KNOWN_HOSTS exit
  STATUS=$?
  if [[ $STATUS -ne 0  ]]; then
    echo "Problem connecting via ssh"
    if [[ $STATUS -eq 255 ]]; then
      echo "ssh timed out"
    fi
    exit $STATUS
  fi
fi

#
# Hardware Information
#
PRODUCTVER=$(sh $SCRIPTDIR/product_version.sh)
if [ -z "$PRODUCTVER" ] ; then
  PRODUCTVER="unknown-product-version"
fi

BIOSVER=$( $SCRIPTDIR/bios_version.sh )
TUNERTYPE=$(sh $SCRIPTDIR/tunertype.sh)
echo $SERIAL > $TMPBASE/serialnumber
echo $PRODUCTVER > $TMPBASE/productversion
echo $TUNERTYPE > $TMPBASE/tunertype
echo $BIOSVER > $TMPBASE/biosversion
lsusb > $TMPBASE/lsusb.log
dmesg -T > $TMPBASE/dmesg.log
#
# Application Data
#
QML_DB_PATH=/home/true/.local/share/data/QML/OfflineStorage/Databases
mkdir $TMPBASE/statistics
sqlite3 $QML_DB_PATH/*.sqlite ".dump workoutcount" > $TMPBASE/statistics/workoutcount.sql
sqlite3 $QML_DB_PATH/*.sqlite ".dump statisticcount" > $TMPBASE/statistics/statisticcount.sql
cp --preserve=timestamps /data/logs/stats.log $TMPBASE/statistics/stats.log

SRC="$SRC $TMPBASE/"


#
# System logs
#
SRC="$SRC /data/logs"
SRC="$SRC /var/log/auth*"
SRC="$SRC /var/log/syslog*"
SRC="$SRC /var/log/kern.log"
SRC="$SRC /var/log/dmesg*"
SRC="$SRC /var/log/udev"
SRC="$SRC /var/log/Xorg.0.log*"
SRC="$SRC /data/tv_conf.xml"
#
# Netpulse logs
#
NETPULSE=$(find /data -maxdepth 1 -name 'error.log*' -print -quit)
if [[ -n $NETPULSE ]]; then
  mkdir $TMPBASE/netpulse
  cp --preserve=timestamps /netpulse/logs/error.log* $TMPBASE/netpulse
  SRC="$SRC TMPBASE/netpulse"
fi

#
# Debian package logs
#
mkdir $TMPBASE/apt
dpkg -l > $TMPBASE/dpkg-list.log
sudo cp --preserve=timestamps /var/log/apt/* $TMPBASE/apt
sudo cp --preserve=timestamps /var/log/dpkg.log* $TMPBASE/apt
SRC="$SRC $TMPBASE/apt"

#
# Core files
#
CORES=$(find /data -maxdepth 1 -name 'core-*' -print -quit)
if [[ -n $CORES ]]; then
  SRC="$SRC /data/core-*"
fi

#
# Transfer archive
#
echo "Starting transfer"
TWEAK_BUF="stdbuf -oL"
if [[ $ARCHIVE = remote ]]; then
  eval sudo rsync -az --info=progress2  --no-i-r -e '"'" ssh -i $KEY -o UserKnownHostsFile=$KNOWN_HOSTS"'"' $SRC $DESTDIR | $TWEAK_BUF tr '\r' '\n'| $TWEAK_BUF tr -s " " | $TWEAK_BUF cut -d " " -f 3
  STATUS=$?
elif [[ $ARCHIVE = usb ]]; then
  sudo rsync -a --no-o --no-g --info=progress2 --no-i-r $SRC $DESTDIR | $TWEAK_BUF tr '\r' '\n'| $TWEAK_BUF tr -s " " | $TWEAK_BUF cut -d " " -f 3
  STATUS=$?
fi

if [[ $STATUS -eq 0 ]]; then
  echo "Transfer complete!"
  # Remove core files
  if [[ -n $CORES ]]; then
    sudo rm /data/core-*
  fi
	# Tar archive on remote server disabled for current release
	# ssh -q -f -o BatchMode=yes -o ConnectTimeout=10 -i $KEY $USER@$SERVER -l rlog -o UserKnownHostsFile=$KNOWN_HOSTS "mkdir -p /home/rlog/logs/$SERIAL && tar -cf /home/rlog/logs/$SERIAL/$TAR $DESTTOP && rm -rf $DESTTOP"
else
  echo "Transfer error"
  case $STATUS in
    30 )
      echo "Rsync: Timeout in data send/receive"
      ;;
    35 )
      echo "Rsync: Timeout waiting for daemon connection"
      ;;
    255 )
      echo "SSH: Timeout"
  esac
fi
sync

exit $STATUS
